// (C) Copyright Carnegie Mellon University 2006, 2007
// All Rights Reserved.
// This code is published under the Common Public License.
//
// Authors :
// P. Bonami, Carnegie Mellon University
//
// Date :  03/17/2006


#if defined(_MSC_VER)
// Turn off compiler warning about long names
#  pragma warning(disable:4786)
#endif
#include <iomanip>
#include <fstream>
using namespace std;

#include "CoinTime.hpp"
#include "CoinError.hpp"

#include "BonOsiTMINLPInterface.hpp"
#include "BonIpoptSolver.hpp"
#include "MyTMINLP.hpp"
#include "BonCbc.hpp"
#include "BonBonminSetup.hpp"

#include "BonOACutGenerator2.hpp"
#include "BonEcpCuts.hpp"
#include "BonOaNlpOptim.hpp"

//#include "SimOpt.hpp"
//#define REDIRECT

//Sheri
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <time.h>
#include <string>
using namespace std;

int main (int argc, char *argv[])
{
  WindowsErrorPopupBlocker();

  using namespace Ipopt;
  using namespace Bonmin;
  
//Pointers
  SmartPtr<MyTMINLP> tminlp = new MyTMINLP;
  SmartPtr<MyTMILP> tmilp = new MyTMILP;
  FP_Ptr fp_Ptr = new Feasibility_Pump;
  //PH1_Ptr ph1_ptr = new ProblemHeuristic1; 

#ifdef REDIRECT
  FILE * fp = fopen("log.out","w");
  CoinMessageHandler handler(fp);
  BonminSetup bonmin(&handler);
#else
  BonminSetup bonmin;
#endif
  bonmin.initializeOptionsAndJournalist();
  //Register an additional option
  bonmin.roptions()->AddStringOption2("print_solution","Do we print the solution or not?",
                                 "yes",
                                 "no", "No, we don't.",
                                 "yes", "Yes, we do.",
                                 "A longer comment can be put here");
  
  
  
  // Here we can change the default value of some Bonmin or Ipopt option
  bonmin.options()->SetNumericValue("bonmin.time_limit", 5); //changes bonmin's time limit
  bonmin.options()->SetStringValue("mu_oracle","loqo");
  
  //Here we read several option files
  bonmin.readOptionsFile("Mybonmin.opt");
  bonmin.readOptionsFile();// This reads the default file "bonmin.opt"
  
  // Options can also be set by using a string with a format similar to the bonmin.opt file
  bonmin.readOptionsString("bonmin.algorithm B-BB\n");
  
  
  
  // Now we can obtain the value of the new option
  int printSolution;
  bonmin.options()->GetEnumValue("print_solution", printSolution,"");
  if(printSolution == 1){
    tminlp->printSolutionAtEndOfAlgorithm();
    tmilp->printSolutionAtEndOfAlgorithm();
  }

  
  
  
  //Set problem values
  const double* nlp_sol; //Solutions of the NLP(ipopt) will be saved here
  const double* milp_sol;
  const int MaxIt=19;
  int It=0;
  int tt=0;
  double Best_Sol=0;
  
  tminlp->init_tangentValues();
  tminlp->set_itt();
  tminlp->readWells();
  tminlp->readPipes();
  tminlp->readminmax_p();
  tminlp->readminmax_gl();
  tminlp->set_nlp();
  tminlp->set_It(It);
  tminlp->set_bounds();
  tminlp->setWW(It);
  
  tmilp->init_tangentValues();
  tmilp->set_itt();
  tmilp->readWells();
  tmilp->readPipes();
  tmilp->readminmax_p();
  tmilp->readminmax_gl();
  tmilp->set_nlp();
  tmilp->set_It(It);
  tmilp->setWW(It); 
  
 
  
  
  
  

// ---------------------INITIALIZATION PROBLEM HEURISTIC-------------------

   
//    ph1_ptr->set_CapacityS();
//    ph1_ptr->priorityWells ();
// 
// 
//    int run=0;  
//    while(run<2){
  
  
  
  time_t start,endt;
  int fp_time;
  
  
   
//---------------------INITIALIZATION FEASIBILITY PUMP---------------------
start = time(NULL);
  
bonmin.initialize(GetRawPtr(tminlp)); 
    
    try {
  	  Bab bb;
  	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
      
      nlp_sol = bb.bestSolution();
      if (nlp_sol != NULL){
          //Feasibility Pump
           fp_Ptr->save_sol_nlp(It,nlp_sol);
           tminlp->save_sol(nlp_sol);
           tmilp->save_sol(nlp_sol);
           tmilp->set_tangentValues(nlp_sol,tminlp->FPJac_w, tminlp->FPJac_p);  
           //tminlp->print_solValues(nlp_sol);
           //tminlp->lock_bounds(nlp_sol);
           
           //Problem Heuristic
           //ph1_ptr->save_nlp(nlp_sol);
           
      
       //Test integer feasibility           
       if(fp_Ptr->test_int_feasibility(nlp_sol)==true){
           cout<<"NLP soltuion is integer feasible, no feasibility pump nessesary"<<endl;
           It=MaxIt;
           }
          else{
          cout<<"NLP soltuion is not integer feasible"<<endl;
          }
      }
 	  	else{
             cout<<"Got NULL for solution"<<endl; 
         }
    }
    
    
	  catch(TNLPSolver::UnsolvedError *E) {
	    //There has been a failure to solve a problem with Ipopt.
	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
	  }
	  catch(OsiTMINLPInterface::SimpleError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
	  catch(CoinError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
    




    
//-------------------------TEST PROBLEM HEURISTIC--------------------------
 
//      if(run==0){   
//  
//          ph1_ptr->evaluate_nlp();
//          ph1_ptr->process_nlp();
//          ph1_ptr->print_solValues(); 
//          tminlp->lock_BoundPH1(ph1_ptr->roundedSol);
// 
//          } 
//          run++;  
//      }
    
    




 

//--------------------------FEASIBILITY PUMP-------------------------------    




  
  //It=MaxIt;
  while (It<MaxIt){
    
    
    
    
      
//--------------------------------MILP-------------------------------------
    
      //Initialize 
      tmilp->set_milp();
      tminlp->set_milp();
      bonmin.initialize(GetRawPtr(tmilp)); 
     
      
      try {
  	  Bab bb;
  	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
      milp_sol = bb.bestSolution();
     
      if (milp_sol != NULL){
            
             fp_Ptr->save_sol_milp(milp_sol);
             tminlp->save_sol(milp_sol);
             tmilp->save_sol(milp_sol);
             tmilp->print_solValues(milp_sol);
             
            //Test NLP feasibility
            if(fp_Ptr->test_feasibility()==true){
            cout<<"MILP soltuion is NLP feasible, stop!"<<endl;
            It=MaxIt;
            }
            else{
            cout<<"MILP soltuion is not NLP feasible"<<endl;
            }    
        }
 	  	else{
             cout<<"Got NULL for solution"<<endl;
             //Hvis dette skjer da, hva skal gjres med startpunkt og parametere til nlp?
             //Lage en funksjon som generer nye startpunkt
         }
   
      }
    
	  catch(TNLPSolver::UnsolvedError *E) {
	    //There has been a failure to solve a problem with Ipopt.
	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
	  }
	  catch(OsiTMINLPInterface::SimpleError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
	  catch(CoinError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
      
    //Test integer solution
    tminlp->set_It(0);
    tminlp->set_nlp();
    tminlp->set_boundsmilp(fp_Ptr->sol_milp);
    bonmin.initialize(GetRawPtr(tminlp)); 
    
    cout<<"Test integer feasibility"<<endl;
    try {
  	  Bab bb;
  	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
      
      nlp_sol = bb.bestSolution();
      if (nlp_sol != NULL){
           fp_Ptr->save_sol_nlp(It,nlp_sol);
           fp_Ptr->save_sol_milp(nlp_sol);
           //tminlp->save_sol(nlp_sol);
           tminlp->print_solValues(nlp_sol);
                           
          if(fp_Ptr->test_int_feasibility(nlp_sol)==true){
          if(fp_Ptr->test_feasibility()==true){
          cout<<"NLP soltuion is integer feasible! "<<endl;
          cout<<"tminlp->tot_prod "<<tminlp->tot_prod<<endl;
          if(tminlp->tot_prod > Best_Sol){
              Best_Sol=tminlp->tot_prod;
              tt++;
              cout<<"Best_Sol: "<<Best_Sol<<endl;
              fp_Ptr->save_best_minlp(It, nlp_sol);
          }
          //It=MaxIt;
                }
          else{
            cout<<"NLP soltuion is not NLP feasible"<<endl;   
          }
           }
          else{
          cout<<"NLP soltuion is not integer feasible"<<endl;
          }
        }
 	  	else{
             cout<<"Got NULL for solution"<<endl;    
         }
    }
    
    
	  catch(TNLPSolver::UnsolvedError *E) {
	    //There has been a failure to solve a problem with Ipopt.
	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
	  }
	  catch(OsiTMINLPInterface::SimpleError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
	  catch(CoinError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
    
    
    
    
    
   
      
      
//---------------------------------NLP-------------------------------------
     
     
      //Initialize model
      It++;
      tmilp->set_It(It);
      tmilp->setWW(It);
      tminlp->set_It(It);
      tminlp->setWW(It);
      tmilp->set_nlp();
      tminlp->set_nlp();
      tminlp->restore_bounds();
      bonmin.initialize(GetRawPtr(tminlp));
      
      
      //It=MaxIt;
      if(It<MaxIt){
      
      try {
  	  Bab bb;
  	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
      nlp_sol = bb.bestSolution();//NLP solution

      if (nlp_sol != NULL){
           //Save NLP solution
           fp_Ptr->save_sol_nlp(It,nlp_sol);
           tminlp->save_sol(nlp_sol);
           tmilp->save_sol(nlp_sol);
           tmilp->set_tangentValues(nlp_sol,tminlp->FPJac_w, tminlp->FPJac_p);  
           tminlp->print_solValues(nlp_sol);
           //tminlp->lock_bounds(nlp_sol);
           
           //Test integer feasibility
           if(fp_Ptr->test_int_feasibility(nlp_sol)==true){
           It=MaxIt-1;
           tminlp->intSol(nlp_sol);
           cout<<"NLP soltuion is integer feasible, stop!"<<endl;
           }
           else{
            cout<<"NLP soltuion is not integer feasible"<<endl;
           }
        }
 	  	else{
             cout<<"Got NULL for solution"<<endl;
         }
      }

	  catch(TNLPSolver::UnsolvedError *E) {
	    //There has been a failure to solve a problem with Ipopt.
	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
	  }
	  catch(OsiTMINLPInterface::SimpleError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
	  catch(CoinError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  } 
  }
      cout<<"Iteration number: "<<It<<endl;
}
    

endt = time(NULL);
fp_time=difftime(endt,start);
cout<<"tt = "<<tt<<endl;
cout<<"Antall sekunder "<<fp_time<<endl;

    
//----------------------SOLUTION FEASIBILITY PUMP--------------------------
    tminlp->set_It(0);
    //for(int i=0; i<50; i++){
    //cout<<"fp_Ptr->minlp["<<i<<"]: "<<fp_Ptr->minlp[i]<<endl;}
    tminlp->set_boundsmilp(fp_Ptr->minlp);
    bonmin.initialize(GetRawPtr(tminlp)); 
    
   
    try {
  	  Bab bb;
  	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
      
      nlp_sol = bb.bestSolution();
      if (nlp_sol != NULL){
          //Feasibility Pump
           fp_Ptr->save_sol_nlp(It,nlp_sol);
           tminlp->save_sol(nlp_sol);
           tminlp->print_solValues(nlp_sol);
        }
 	  	else{
             cout<<"Got NULL for solution"<<endl;
         }
        }
    
    
	  catch(TNLPSolver::UnsolvedError *E) {
	    //There has been a failure to solve a problem with Ipopt.
	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
	  }
	  catch(OsiTMINLPInterface::SimpleError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
	  catch(CoinError &E) {
	    std::cerr<<E.className()<<"::"<<E.methodName()
		     <<std::endl
		     <<E.message()<<std::endl;
	  }
    
 
    
   
    
    
    
    
    delete fp_Ptr;
    //delete ph1_ptr;
    fp_Ptr = 0;
    //ph1_ptr = 0;
    return 0;
}

