// (C) Copyright Carnegie Mellon University 2006
// All Rights Reserved.
// This code is published under the Common Public License.
//
// Authors :
// P. Bonami, Carnegie Mellon University
//
// Date :  03/17/2006
#ifndef MyTNLP_HPP
#define MyTNLP_HPP
#include "BonTMINLP.hpp"
using namespace  Ipopt;
using namespace Bonmin;

using namespace std;
#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <math.h>
#include <cstdlib>




 static const int M = 2;    //Subsea manifolds
 static const int I = 6;    //Platform wells
 static const int J = 10;   //Subsea wells
 static const int J1= 6;    //Subsea wells Manifold 1
 static const int J2= 4;    //Subsea wells Manifold 2
 static const int L = 4;    //Pipelines
 static const int L1 = 2;
 static const int L2 = 2;
 static const int P = 3;    //Phases
 static const int S = 3;    //Separators
    
 static const int NV = 150;  //Total number of variables
 static const int NB = 50;   //NUmber of binary variables
 //static const int MC = 139;  //Number of constraints  
 
 static const double SeA = 10;
 static const double SeB = 12;
 static const double SeC = 15;

class MyTMILP;
class MyTMINLP;
class Feasibility_Pump;

class ProblemHeuristic1
{
private:
//Global constants belonging to the class, related to size of problem
 
 double CapacityS[S];
 int PriorityManifold[I+J];
 int PriorityTotal[I+J];
 int PriorityPipe[L];
 int WellSepPriority[I];
 int WellS1Priority[J1];
 int WellS2Priority[J2];
 int PipePriority[L];
 
 double nlp[NB];
 
 double compareSM1[L1][J1];
 double compareSM2[L2][J2];
 double compareS[S][L];
 double compareW[S][I];
 
 double sorted_WellSep[S*I];
 double sorted_WellPipe[J*L];
 double sorted_PipeSep[S*L];
 
 bool binary_evaluate[NB];
 
public:
 
 Number roundedSol[NB];
 
 void set_CapacityS ();
 void priorityWells ();
 void save_nlp(const double*);
 void evaluate_nlp();
 void process_nlp();
 void print_solValues();
 
 
 
 void routing_pipeSeparator();
 void routing_wellSeparator();
 void routing_wellPipe();
 void get_roundedSol();
 
 bool illigalSM1(int l, double j[L1][J1]);
 bool illigalSM2(int l, double j[L2][J2]);
 bool illigalS(int s, double j[S][L]);
 bool illigalW(int s, double j[S][L], double k[S][I]);
 
};

typedef ProblemHeuristic1* PH1_Ptr; 



 class Feasibility_Pump
{
 private:
 //Global constants belonging to the class, related to size of problem
 static const int MaxIt = 20;
 static const int NV_MILP = 427;
 static const int NV_MINLP = 250;
 
 public:

//Feasibility Pump
double minlp[NB];
double minlp_sol[NV];
double temp_sol[150];
double sol_milp[150];
double FParameter_f[NB];
double tot_prod;

void init_Parameters();
void save_sol_nlp(int,const double*);
void save_best_minlp(int, const double* );
    
void save_sol_milp(const double*);
bool test_feasibility();
bool test_int_feasibility(const double* adr);
 
double Jac_w [I+J][2][MaxIt];
double Jac_p [L][3][MaxIt];
double sol_w [I+J][3][MaxIt];
double sol_p [L][4][MaxIt];
 
friend class MyTMILP;
friend class MyTMINLP;
 };
 
typedef Feasibility_Pump* FP_Ptr;  


 
class MyTMINLP : public TMINLP
{
public:   
    
    int itt; 
    int It_nlp;
    bool nlp;
    bool milp;
    
    double WW;
    double NND;

    //Global constants belonging to the class, related to size of proble       
    static const int NV_MINLP = 150;  //Total number of variables
    static const int MC_MINLP = 139;  //Number of constraints
    static const int MaxIt = 20;

    //Dimension of the welldata files:
    static const int gdim = 12;  //gas injection break points,
    static const int pdim = 12;  //p^W break points
    static const int ddim = 3;   //columns in the data file

    //Dimension of the pressuredrop files
    static const int dim = 30;
    static const int xdim = 30;
    static const int ydim = 30;
    static const int zdim = 30;
    
    //Separator Pressures, set in function readWells
    double PSEPA;              
    double PSEPB;
    double PSEPC;

    //Platform Well Data matrices
    double P01[gdim][pdim][ddim]; 
    double P02[gdim][pdim][ddim]; 
    double P03[gdim][pdim][ddim]; 
    double P04[gdim][pdim][ddim]; 
    double P05[gdim][pdim][ddim]; 
    double P06[gdim][pdim][ddim]; 

    //Subsea Well Data matrices
    double S01[gdim][pdim][ddim];
    double S02[gdim][pdim][ddim];
    double S03[gdim][pdim][ddim];
    double S04[gdim][pdim][ddim]; 
    double S05[gdim][pdim][ddim]; 
    double S06[gdim][pdim][ddim];
    double S07[gdim][pdim][ddim];
    double S08[gdim][pdim][ddim]; 
    double S09[gdim][pdim][ddim]; 
    double S010[gdim][pdim][ddim];

    //Fill inn WC and GOR information
    double wellspecI[I][2];
    double wellspecJ[J][2];
    double minmax_w[I+J][3][2];   
    void readWspecs();
    
    void readminmax_p();
    void readminmax_gl();
    double minmax_gl[I+J][2];       //Array containing min and max values of gas lift for all wells minmax_gl[well][min/max]
    double minmax_p[L][4][2];   //Array containing min and max values for all pipelines minmax_p[well][gas/oil/water/pressure][min/max]
    
    //Save Solution
    double tot_prod;
    int sol_feasible;
    void get_sol();
    void save_sol(const double*);
    void print_solValues(const double* );
    void intSol(const double*);
    void set_boundsmilp(double minlp [NB]);
    
    //Feasibility Pump   
    void set_It(int);
    void setWW(int It);
    void set_milp();
    void set_nlp();
    void set_milpValues();
    void set_bounds();
    void save_bounds();
    void restore_bounds();
    void lock_bounds(const double* x);
    void lock_BoundPH1(Number milp[NB]);
    Number l_bound[NB];
    Number u_bound[NB];
    Number l[NB];
    Number u[NB];
    
    double Parameter_f[NV_MINLP]; // Array containing the parameters for the objective funtion in the MILP problem 
    double Parameter_start[NV_MINLP];
    double dummy_start[NV_MINLP];
    
    double FPJac_w[J+I][2][MaxIt];  //[Well][pressure/gas lift]
    double FPJac_p[L][3][MaxIt];
    long double minDist[MaxIt];
    double maxOFV[MaxIt];
    
    double tangent_w[I+J][3][20]; //Point and gradient for tangent plane [Well][Gas/Oil/Water/PressureDrop/Gradient][MaxIt] 
    double tangent_p[L][4][20];
    double tangent_jacw[I+J][2][20];
    double tangent_jacp[L][3][20];
    
    void init_tangentValues();

    
    // Default constructor
    MyTMINLP(): printSol_(false){}

    // Virtual destructor
    virtual ~MyTMINLP(){}
  
    /** Copy constructor.*/   
    MyTMINLP(const MyTMINLP &other): printSol_(other.printSol_){}
  
    //Functions to read Well data
    void readWells(); //called from main to read all the well data
    void readWellData(const string &filen, double data[gdim][pdim][ddim]);//called from readWells() -> reads and saves one well
    void minmax_wells(int, double data[gdim][pdim][ddim]);                  //Saves the min/max values into the array minmax_w
    void set_arrptr();

    //Functions to read Pressure Drop data
    double gas[dim];
    double oil[dim];
    double wat[dim];
    double dP1[dim][dim][dim][4];
    double dP2[dim][dim][dim][4];
    double dP3[dim][dim][dim][4];
    double dP4[dim][dim][dim][4];
    void readDeltaPData(const string &filen, double data[dim][dim][dim][4]); //called form readPipes() -> reads and saves one pipeline
    void readPipes(); //called from main to read all the pipeline data

    //Making an array to hold pointers to the Pipeline Tables
    typedef double (*dPtrArray)[dim][dim][4];
    dPtrArray dParrptr[L];
    void set_dParrPtr();

    typedef double (*PtrArray)[pdim][ddim];
    //Making an array to hold pointers to the Platform Well Tables
    PtrArray parrptr[I];

    //Making an array to hold pointers to the Subsea Well Tables
    PtrArray sarrptr[J];

    //Functions to write output files
    void printsome(int, int, int, double, double, double, double, double val[12]); //for  printe ut og sjekke datainnlesing, kan slettes etterhvert
    void set_itt(){itt=0;};

    //Simulators
    double * ratesim(int,int, double, double, double (*)[pdim][ddim]);
    double * PressureDrop(int,int, double, double, double, double (*)[dim][dim][4]);

    //General Functions
    int round(double);
    int min_val(int, double a[]);//returnerer index til minste
    int max_val(int, double a[]);//returnerer index til strste
    int closest_zero(int, double a[]);//returnerer index til positivt tall nrmest null

  virtual bool get_variables_types(Index n, VariableType* var_types);
  virtual bool get_variables_linearity(Index n, Ipopt::TNLP::LinearityType* var_types);
  virtual bool get_constraints_linearity(Index m, Ipopt::TNLP::LinearityType* const_types);
  virtual bool get_nlp_info(Index& n, Index&m, Index& nnz_jac_g,
                            Index& nnz_h_lag, TNLP::IndexStyleEnum& index_style);
  virtual bool get_bounds_info(Index n, Number* x_l, Number* x_u,
                               Index m, Number* g_l, Number* g_u);
  virtual bool get_starting_point(Index n, bool init_x, Number* x,
                                  bool init_z, Number* z_L, Number* z_U,
                                  Index m, bool init_lambda,
                                  Number* lambda);
  virtual bool eval_f(Index n, const Number* x, bool new_x, Number& obj_value);
  virtual bool eval_grad_f(Index n, const Number* x, bool new_x, Number* grad_f);
  virtual bool eval_g(Index n, const Number* x, bool new_x, Index m, Number* g);
  virtual bool eval_jac_g(Index n, const Number* x, bool new_x,
                          Index m, Index nele_jac, Index* iRow, Index *jCol,
                          Number* values);  
  virtual bool eval_h(Index n, const Number* x, bool new_x,
                      Number obj_factor, Index m, const Number* lambda,
                      bool new_lambda, Index nele_hess, Index* iRow,
                      Index* jCol, Number* values);
  /** Method called by Ipopt at the end of optimization.*/  
  virtual void finalize_solution(TMINLP::SolverReturn status,
                                 Index n, const Number* x, Number obj_value);
  virtual const SosInfo * sosConstraints() const{return NULL;}
  virtual const BranchingInfo* branchingInfo() const{return NULL;}
  
  
  void printSolutionAtEndOfAlgorithm(){
    printSol_ = true;}
  
   bool printSol_;
   friend class Feasibility_Pump;
   friend class MyTMINLP_MILP;
};

typedef MyTMINLP* NLP_Ptr;  

class MyTMILP : public TMINLP
{
public:   
    
    int itt; 
    int It_nlp;
    bool nlp;
    bool milp;
    double WW;
    double NND;
    
    //Global constants belonging to the class, related to size of problem
    static const int NV_MILP = 427;  //Total number of variables
    static const int MC_MILP = 806;  //Number of constraints
    static const int MaxIt = 20;

    //Dimension of the welldata files:
    static const int gdim = 12;  //gas injection break points,
    static const int pdim = 12;  //p^W break points
    static const int ddim = 3;   //columns in the data file

    //Dimension of the pressuredrop files
    static const int dim = 30;
    static const int xdim = 30;
    static const int ydim = 30;
    static const int zdim = 30;
    
    //Separator Pressures, set in function readWells
    double PSEPA;              
    double PSEPB;
    double PSEPC;

    //Platform Well Data matrices
    double P01[gdim][pdim][ddim]; 
    double P02[gdim][pdim][ddim]; 
    double P03[gdim][pdim][ddim]; 
    double P04[gdim][pdim][ddim]; 
    double P05[gdim][pdim][ddim]; 
    double P06[gdim][pdim][ddim]; 

    //Subsea Well Data matrices
    double S01[gdim][pdim][ddim];
    double S02[gdim][pdim][ddim];
    double S03[gdim][pdim][ddim];
    double S04[gdim][pdim][ddim]; 
    double S05[gdim][pdim][ddim]; 
    double S06[gdim][pdim][ddim];
    double S07[gdim][pdim][ddim];
    double S08[gdim][pdim][ddim]; 
    double S09[gdim][pdim][ddim]; 
    double S010[gdim][pdim][ddim];

    //Fill inn WC and GOR information
    double wellspecI[I][2];
    double wellspecJ[J][2];
    double minmax_w[I+J][3][2];   
    void readWspecs();
    
    void readminmax_p();
    void readminmax_gl();
    double minmax_gl[I+J][2];       //Array containing min and max values of gas lift for all wells minmax_gl[well][min/max]
    double minmax_p[L][4][2];   //Array containing min and max values for all pipelines minmax_p[well][gas/oil/water/pressure][min/max]
    
    //Save Solution
    int sol_feasible;
    void get_sol();
    void save_sol(const double* adr);
    void print_solValues(const double* );
    
    //Feasibility Pump   
    void set_It(int);
    void setWW(int It);
    void set_milp();
    void set_nlp();
    void set_milpValues();
    
    double Parameter_f[NV_MILP]; // Array containing the parameters for the objective funtion in the MILP problem 
    double Parameter_start[NV_MILP];
    
    double FPJac_w[J+I][2][MaxIt];  //[Well][pressure/gas lift]
    double FPJac_p[L][3][MaxIt];
    
    
    double tangent_w[I+J][3][20]; //Point and gradient for tangent plane [Well][Gas/Oil/Water/PressureDrop/Gradient][MaxIt] 
    double tangent_p[L][4][20];
    double tangent_jacw[I+J][2][20];
    double tangent_jacp[L][3][20];
    void  check_tangentw(int,double (*)[pdim][ddim]);
    void  check_tangentp(int,double (*)[dim][dim][4]);
    
    void init_tangentValues();
    void set_tangentValues (const double* adr, double t_jacw[I+J][2][MaxIt], double t_jacp[L][3][MaxIt]);
    void print_tangentValues();

    
    // Default constructor
    MyTMILP(): printSol_(false){}

    // Virtual destructor
    virtual ~MyTMILP(){}
  
    /** Copy constructor.*/   
    MyTMILP(const MyTMILP &other): printSol_(other.printSol_){}

    /** Assignment operator. no data = nothing to assign*/
    //MyTMILP& operator=(const MyTMILP&) {}

  
    //Functions to read Well data
    void readWells(); //called from main to read all the well data
    void readWellData(const string &filen, double data[gdim][pdim][ddim]);//called from readWells() -> reads and saves one well
    void minmax_wells(int, double data[gdim][pdim][ddim]);                  //Saves the min/max values into the array minmax_w
    void set_arrptr();

    //Functions to read Pressure Drop data
    double gas[dim];
    double oil[dim];
    double wat[dim];
    double dP1[dim][dim][dim][4];
    double dP2[dim][dim][dim][4];
    double dP3[dim][dim][dim][4];
    double dP4[dim][dim][dim][4];
    void readDeltaPData(const string &filen, double data[dim][dim][dim][4]); //called form readPipes() -> reads and saves one pipeline
    void readPipes(); //called from main to read all the pipeline data

    //Making an array to hold pointers to the Pipeline Tables
    typedef double (*dPtrArray)[dim][dim][4];
    dPtrArray dParrptr[L];
    void set_dParrPtr();

    typedef double (*PtrArray)[pdim][ddim];
    //Making an array to hold pointers to the Platform Well Tables
    PtrArray parrptr[I];

    //Making an array to hold pointers to the Subsea Well Tables
    PtrArray sarrptr[J];

    //Functions to write output files
    void printsome(int, int, int, double, double, double, double, double val[12]); //for  printe ut og sjekke datainnlesing, kan slettes etterhvert
    void set_itt(){itt=0;};

    //Simulators
    double * ratesim(int,int, double, double, double (*)[pdim][ddim]);
    double * PressureDrop(int,int, double, double, double, double (*)[dim][dim][4]);

    //General Functions
    int round(double);
    int min_val(int, double a[]);//returnerer index til minste
    int max_val(int, double a[]);//returnerer index til strste
    int closest_zero(int, double a[]);//returnerer index til positivt tall nrmest null

  virtual bool get_variables_types(Index n, VariableType* var_types);
  virtual bool get_variables_linearity(Index n, Ipopt::TNLP::LinearityType* var_types);
  virtual bool get_constraints_linearity(Index m, Ipopt::TNLP::LinearityType* const_types);
  virtual bool get_nlp_info(Index& n, Index&m, Index& nnz_jac_g,
                            Index& nnz_h_lag, TNLP::IndexStyleEnum& index_style);
  virtual bool get_bounds_info(Index n, Number* x_l, Number* x_u,
                               Index m, Number* g_l, Number* g_u);
  virtual bool get_starting_point(Index n, bool init_x, Number* x,
                                  bool init_z, Number* z_L, Number* z_U,
                                  Index m, bool init_lambda,
                                  Number* lambda);
  virtual bool eval_f(Index n, const Number* x, bool new_x, Number& obj_value);

  virtual bool eval_grad_f(Index n, const Number* x, bool new_x, Number* grad_f);

  virtual bool eval_g(Index n, const Number* x, bool new_x, Index m, Number* g);

  virtual bool eval_jac_g(Index n, const Number* x, bool new_x,
                          Index m, Index nele_jac, Index* iRow, Index *jCol,
                          Number* values);

  virtual bool eval_h(Index n, const Number* x, bool new_x,
                      Number obj_factor, Index m, const Number* lambda,
                      bool new_lambda, Index nele_hess, Index* iRow,
                      Index* jCol, Number* values);

  
  /** Method called by Ipopt at the end of optimization.*/  
  virtual void finalize_solution(TMINLP::SolverReturn status,
                                 Index n, const Number* x, Number obj_value);

  //@}

  virtual const SosInfo * sosConstraints() const{return NULL;}
  virtual const BranchingInfo* branchingInfo() const{return NULL;}
  
  
  void printSolutionAtEndOfAlgorithm(){
    printSol_ = false;}
  
   bool printSol_;
   friend class Feasibility_Pump;
   friend class MyTMILP_MILP;
};


typedef MyTMILP* MILP_Ptr;  


  




#endif
