#AMPL model file for the production optimization on Marlim field 


#SETS

set Beta; # Betas used to generate function for wellhead flow and pressure drop through pipeline
set Function_Well;
set Manifold;	# Manifolds connected to the platform.
set Manifold_Subsea within Manifold; # Subsea manifolds connected to the platform.
set Manifold_Topside within Manifold; # Topside manifolds connected to the platform.
set Phase;	# Phases through the system, gas, oil and water.
set Pipeline;	#Pipelines connected to the platform.
set Pipeline_Manifold {Manifold};	#Pipelines connected to the platform.
set Separator;
set Separator_Prod within Separator; 
set Well; 	# Wells connected to both the topside manifold and the subsea manifold.
set Well_Manifold {Manifold} within Well; #Wells connected to the topside manifolds in the platform
set Function_Pipeline{m in Manifold_Subsea, l in Pipeline_Manifold[m]} ; #Functions used for well model and pressure loss model
set Well_Two_Functions{m in Manifold};



#PARAMETERS

param beta_Pipeline {m in Manifold_Subsea,Beta, l in Pipeline_Manifold[m], Function_Pipeline[m,l]}; 
param beta_Well {m in Manifold,Beta, Well_Manifold[m],Function_Well};  
param Cap_Gas;# capactiy limit of lift gas.
param Cap_LiftGas;
param Cap_Liquid;  #Capacity limit on liquids for the platform.
param Cap_Water{Separator}; # Capacity limit on flow of phase p for the platform.
param GOR_Well{m in Manifold, Well_Manifold[m]};
param P_Separator{Separator};#Pressure at Separator system
param P_Well_Max {m in Manifold, Well_Manifold[m]}; #Maximum wellhead pressure for well j
param P_Well_Min {m in Manifold, Well_Manifold[m]}; # Minimum wellhead pressure for well j
param Q_Injected_Max {m in Manifold, Well_Manifold[m]}; 	#Maximum amount of gas injection to well j connected to manifold m
param Q_Injected_Min {m in Manifold, Well_Manifold[m]}; 	#Minimum amount of gas injection to well j connected to manifold m
param Q_Well_Max {m in Manifold, Well_Manifold[m]}; # Maximum flow of phase p from well j
param Q_Well_Manifold {m in Manifold, Well_Manifold[m],Phase}; # Given flow of oil from well j connected to manifold m
param Q_Well_Min {m in Manifold, Well_Manifold[m]}; # Minimum flow of phase p from well j
param WC_Well{m in Manifold, Well_Manifold[m]};




#Variables

var p_Drop_Pipeline {m in Manifold_Subsea, Pipeline_Manifold [m]} >=0; # Pressuredrop trough pipeline l connected to manifold m
var p_Pipeline_Manifold {m in Manifold_Subsea, Pipeline_Manifold [m]} >= 0; #Pressure in pipeline l connected to manifold m,after manifold
var p_Pipeline_Platform {m in Manifold_Subsea, Pipeline_Manifold [m]} >= 0; #Pressure in pipeline l connected to manifold m,at platform
var p_Well {m in Manifold, Well_Manifold [m]} >= 0; # Pressure at wellhead for well j connected to manifold m
var q_Injected {m in Manifold, Well_Manifold [m]} >= 0; #Flow of liftgas injected to well j connected to manifold m
var q_Pipeline {m in Manifold_Subsea, Pipeline_Manifold [m], Phase} >= 0; # Flow of phase p thorugh pipeline p connected to manifold m 
var q_Pipeline_Liquid {m in Manifold_Subsea, Pipeline_Manifold [m]} >= 0; # Flow of phase p thorugh pipeline p connected to manifold m 
var q_Well {m in Manifold, Well_Manifold [m], Phase} >= 0; # Flow of phase p from well j connected to manifold m
var s_Well {m in Manifold, Well_Two_Functions[m],Function_Well} binary; #Liquid flow through pipelines
var s_Pipeline {m in Manifold_Subsea, l in Pipeline_Manifold [m], Function_Pipeline[m,l]} binary;
var wc_Pipeline {m in Manifold_Subsea, Pipeline_Manifold [m]} >= 0;
var y {m in Manifold_Subsea,Well_Manifold [m], Pipeline_Manifold [m]} binary; #Equals 1 if flow is routed from well j to pipeline l, connected to subsea manifold, 0 otherwise
var x {m in Manifold_Topside,Well_Manifold [m],Separator} binary; #Equals 1 if  there is flow from well j, 0 otherwise
var z {m in Manifold_Subsea, Pipeline_Manifold[m],Separator} binary;
var q_Liquid;
var q_Water{Separator};





#OBJECTIVE FUNCTION - Restriction 5.1

maximize total_Oilproduction :
sum{j in Well_Manifold ["Manifold_Top"], s in Separator}q_Well["Manifold_Top",j,"OIL"]*x["Manifold_Top",j,s]
 + sum {m in Manifold_Subsea, l in Pipeline_Manifold[m]} q_Pipeline [m,l,"OIL"] ;



#CONSTRAINTS

#CAPACITY CONSTRAINTS


#Gas Capacity - Restriction 5.4
subject to Capacity_Gas : 
sum {j in Well_Manifold["Manifold_Top"],s in Separator}(q_Well["Manifold_Top",j,"GAS"]*x["Manifold_Top",j,s])
+ sum{m in Manifold_Subsea,l in Pipeline_Manifold[m]} q_Pipeline[m,l,"GAS"] <= Cap_Gas;

#Gas Lift Capacity - Restriction 5.2
subject to Capacity_LiftGas: 
sum{m in Manifold_Topside, j in Well_Manifold[m], s in Separator} (q_Injected[m,j]*x[m,j,s] )
+ sum{m in Manifold_Subsea, j in Well_Manifold[m]}(q_Injected[m,j]*sum {l in Pipeline_Manifold [m]}y[m,j,l])<= Cap_LiftGas;


#Water Capacity - Restriction 5.3
subject to Capacity_Water{s in Separator}: 
sum {j in Well_Manifold["Manifold_Top"]} (q_Well["Manifold_Top",j,"WATER"]*x["Manifold_Top",j,s])
+ sum{m in Manifold_Subsea,l in Pipeline_Manifold[m]}q_Pipeline[m,l,"WATER"]*z[m,l,s] <= Cap_Water[s];


#MAXIMUM AND MINIMUM CONSTRAINTS FOR FLOW AND PRESSURE - Restrictions 5.16



subject to Boundaries_Pressures_Well_SUb1 {m in Manifold_Subsea, j in Well_Manifold[m]} :
P_Well_Min[m, j] <= p_Well[m,j] <=P_Well_Max[m, j];


subject to Boundaries_Pressures_Well_Top1 {m in Manifold_Topside, j in Well_Manifold[m], s in Separator} :
p_Well[m,j] <=P_Well_Max[m,j];


subject to Boundaries_Pressures_Well_Top2 {m in Manifold_Topside, j in Well_Manifold[m], s in Separator} :
P_Well_Min[m, j]<= p_Well[m,j];


subject to Boundaries_InjectedGas_Well_Sub {m in Manifold, j in Well_Manifold[m]} :
Q_Injected_Min[m, j] <= q_Injected[m,j]<= Q_Injected_Max[m, j];


#ROUTING CONSTRAINTS

#Restriction 5.9
subject to Routing_Well_Separator {m in Manifold_Topside, j in Well_Manifold[m]}:
sum{s in Separator} x[m,j,s] <= 1;

#Restriction 5.8
subject to Routing_Well_Pipeline {m in Manifold_Subsea, j in Well_Manifold[m]}:
sum{l in Pipeline_Manifold[m]} y[m,j,l] <= 1;

#Restriction 5.10
subject to Routing_Pipeline_Separator {m in Manifold_Subsea, l in Pipeline_Manifold[m]}:
sum{s in Separator} z[m,l,s] == 1;


#MASS BALANCE

#Restriction 5.11
subject to Production_MassBalance {m in Manifold_Subsea, l in Pipeline_Manifold[m], p in Phase}:
q_Pipeline [m,l,p] = sum {j in Well_Manifold[m]}( q_Well[m,j,p]*y[m,j,l]);


#Restriction 5.13
subject to Gas_Oil_Ratio_Well{m in Manifold, j in Well_Manifold[m]}:
q_Well[m,j,"GAS"] = q_Well [m,j,"OIL"]*GOR_Well[m,j]/1000 + q_Injected [m,j];


#Restriction 5.14
subject to Water_cut_Well {m in Manifold,j in Well_Manifold[m]}:
q_Well[m,j,"WATER"] = (q_Well [m,j,"OIL"]*WC_Well[m,j])/(1-WC_Well[m,j]);




#PRESSURE RELATIONSHIP

#Restriction 5.6
subject to Well_Manifold_Relationships{m in Manifold_Subsea, j in Well_Manifold[m],l in Pipeline_Manifold[m]}:
p_Well [m,j] >= p_Pipeline_Manifold[m,l]*y[m,j,l];

#Restriction 5.5
subject to Well_Separator_Relationship{j in Well_Manifold["Manifold_Top"],s in Separator}:
p_Well["Manifold_Top",j] >= P_Separator[s]*x["Manifold_Top",j,s];

#Restriction 5.7
subject to Pipeline_Separator_Relationship{m in Manifold_Subsea, l in Pipeline_Manifold[m],s in Separator}:
p_Pipeline_Platform[m,l] >= P_Separator[s]*z[m,l,s];


#Restriction
subject to Pressure_Drop_Function {m in Manifold_Subsea, l in Pipeline_Manifold[m]}:
p_Pipeline_Manifold[m,l] = p_Pipeline_Platform[m,l] + p_Drop_Pipeline[m,l];








#OIL FLOW FROM ALL THE WELLS AS A FUNCTION OF WELLHEAD PRESSURE AND GAS LIFT

#Restriction 5.12



#Function 14
subject to Oil_Flow_Well_7:
q_Well ["Manifold_Sub1","Well_7","OIL"] = 
beta_Well["Manifold_Sub1","beta1","Well_7","Function_1"]
+ beta_Well["Manifold_Sub1","beta2","Well_7","Function_1"]*p_Well["Manifold_Sub1","Well_7"] 
+ beta_Well["Manifold_Sub1","beta3","Well_7","Function_1"]*q_Injected["Manifold_Sub1","Well_7"]
+ beta_Well["Manifold_Sub1","beta4","Well_7","Function_1"]*p_Well["Manifold_Sub1","Well_7"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_7","Function_1"]*p_Well["Manifold_Sub1","Well_7"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_7","Function_1"]*q_Injected["Manifold_Sub1","Well_7"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_7","Function_1"]*q_Injected["Manifold_Sub1","Well_7"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_7","Function_1"]*p_Well["Manifold_Sub1","Well_7"] + 1)^(-1)
+(beta_Well["Manifold_Sub1","beta9","Well_7","Function_1"]*q_Injected["Manifold_Sub1","Well_7"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta10","Well_7","Function_1"]*p_Well["Manifold_Sub1","Well_7"]*q_Injected["Manifold_Sub1","Well_7"]
+ beta_Well["Manifold_Sub1","beta11","Well_7","Function_1"]*log(p_Well["Manifold_Sub1","Well_7"]+1)
+ beta_Well["Manifold_Sub1","beta12","Well_7","Function_1"]*log(q_Injected["Manifold_Sub1","Well_7"]+1); 

#Function_1 13 + Function_2 16
subject to Oil_Flow_Well_8:
q_Well ["Manifold_Sub1","Well_8","OIL"] == 
(beta_Well["Manifold_Sub1","beta1","Well_8","Function_1"]
+ beta_Well["Manifold_Sub1","beta2","Well_8","Function_1"]*p_Well["Manifold_Sub1","Well_8"] 
+ beta_Well["Manifold_Sub1","beta3","Well_8","Function_1"]*q_Injected["Manifold_Sub1","Well_8"]
+ beta_Well["Manifold_Sub1","beta4","Well_8","Function_1"]*p_Well["Manifold_Sub1","Well_8"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_8","Function_1"]*p_Well["Manifold_Sub1","Well_8"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_8","Function_1"]*q_Injected["Manifold_Sub1","Well_8"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_8","Function_1"]*q_Injected["Manifold_Sub1","Well_8"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_8","Function_1"]*p_Well["Manifold_Sub1","Well_8"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta9","Well_8","Function_1"]*p_Well["Manifold_Sub1","Well_8"]*q_Injected["Manifold_Sub1","Well_8"]
+ beta_Well["Manifold_Sub1","beta10","Well_8","Function_1"]*log(p_Well["Manifold_Sub1","Well_8"]+1)
+ beta_Well["Manifold_Sub1","beta11","Well_8","Function_1"]*log(q_Injected["Manifold_Sub1","Well_8"]+1))*s_Well["Manifold_Sub1","Well_8","Function_1"]

+(beta_Well["Manifold_Sub1","beta1","Well_8","Function_2"]
+ beta_Well["Manifold_Sub1","beta2","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"] 
+ beta_Well["Manifold_Sub1","beta3","Well_8","Function_2"]*q_Injected["Manifold_Sub1","Well_8"]
+ beta_Well["Manifold_Sub1","beta4","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"]*q_Injected["Manifold_Sub1","Well_8"]
+ beta_Well["Manifold_Sub1","beta5","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"]^(2)
+ beta_Well["Manifold_Sub1","beta6","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"]^(3)
+ beta_Well["Manifold_Sub1","beta7","Well_8","Function_2"]*q_Injected["Manifold_Sub1","Well_8"]^(2)
+(beta_Well["Manifold_Sub1","beta8","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"] + 1)^(-1)
+(beta_Well["Manifold_Sub1","beta9","Well_8","Function_2"]*q_Injected["Manifold_Sub1","Well_8"] + 1)^(-1)
+(beta_Well["Manifold_Sub1","beta10","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"] + 1)^(-2)
+(beta_Well["Manifold_Sub1","beta11","Well_8","Function_2"]*p_Well["Manifold_Sub1","Well_8"] + 1)^(-3)
+ beta_Well["Manifold_Sub1","beta12","Well_8","Function_2"]*log(p_Well["Manifold_Sub1","Well_8"]+1)
+ beta_Well["Manifold_Sub1","beta13","Well_8","Function_2"]*log(q_Injected["Manifold_Sub1","Well_8"]+1))*s_Well["Manifold_Sub1","Well_8","Function_2"]; 

subject to Well_8_Function:
s_Well["Manifold_Sub1","Well_8","Function_1"]+s_Well["Manifold_Sub1","Well_8","Function_2"]<=1;

subject to Oil_Flow_Well_8_Function_1:
q_Injected["Manifold_Sub1","Well_8"] <= 70 + 200*s_Well["Manifold_Sub1","Well_8","Function_2"];

subject to Oil_Flow_Well_8_Function_2:
q_Injected["Manifold_Sub1","Well_8"] >= 70-70*s_Well["Manifold_Sub1","Well_8","Function_1"];

#Function 13
subject to Oil_Flow_Well_9:
q_Well ["Manifold_Sub1","Well_9","OIL"] = 
beta_Well["Manifold_Sub1","beta1","Well_9","Function_1"]
+ beta_Well["Manifold_Sub1","beta2","Well_9","Function_1"]*p_Well["Manifold_Sub1","Well_9"] 
+ beta_Well["Manifold_Sub1","beta3","Well_9","Function_1"]*q_Injected["Manifold_Sub1","Well_9"]
+ beta_Well["Manifold_Sub1","beta4","Well_9","Function_1"]*p_Well["Manifold_Sub1","Well_9"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_9","Function_1"]*p_Well["Manifold_Sub1","Well_9"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_9","Function_1"]*q_Injected["Manifold_Sub1","Well_9"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_9","Function_1"]*q_Injected["Manifold_Sub1","Well_9"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_9","Function_1"]*p_Well["Manifold_Sub1","Well_9"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta9","Well_9","Function_1"]*p_Well["Manifold_Sub1","Well_9"]*q_Injected["Manifold_Sub1","Well_9"]
+ beta_Well["Manifold_Sub1","beta10","Well_9","Function_1"]*log(p_Well["Manifold_Sub1","Well_9"]+1)
+ beta_Well["Manifold_Sub1","beta11","Well_9","Function_1"]*log(q_Injected["Manifold_Sub1","Well_9"]+1);


#Function_1 12 Function_2 14
subject to Oil_Flow_Well_10:
q_Well ["Manifold_Sub1","Well_10","OIL"] =
(beta_Well["Manifold_Sub1","beta1","Well_10","Function_1"]
+ beta_Well["Manifold_Sub1","beta2","Well_10","Function_1"]*p_Well["Manifold_Sub1","Well_10"]
+ beta_Well["Manifold_Sub1","beta3","Well_10","Function_1"]*q_Injected["Manifold_Sub1","Well_10"]  
+ beta_Well["Manifold_Sub1","beta4","Well_10","Function_1"]*p_Well["Manifold_Sub1","Well_10"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_10","Function_1"]*p_Well["Manifold_Sub1","Well_10"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_10","Function_1"]*q_Injected["Manifold_Sub1","Well_10"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_10","Function_1"]*q_Injected["Manifold_Sub1","Well_10"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_10","Function_1"]*q_Injected["Manifold_Sub1","Well_10"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta9","Well_10","Function_1"]*p_Well["Manifold_Sub1","Well_10"]*q_Injected["Manifold_Sub1","Well_10"]
+ beta_Well["Manifold_Sub1","beta10","Well_10","Function_1"]*log(p_Well["Manifold_Sub1","Well_10"] + 1)
+ beta_Well["Manifold_Sub1","beta11","Well_10","Function_1"]*log(q_Injected["Manifold_Sub1","Well_10"] + 1))*s_Well["Manifold_Sub1","Well_10","Function_1"]

+(beta_Well["Manifold_Sub1","beta1","Well_10","Function_2"]
+ beta_Well["Manifold_Sub1","beta2","Well_10","Function_2"]*p_Well["Manifold_Sub1","Well_10"] 
+ beta_Well["Manifold_Sub1","beta3","Well_10","Function_2"]*q_Injected["Manifold_Sub1","Well_10"]
+ beta_Well["Manifold_Sub1","beta4","Well_10","Function_2"]*p_Well["Manifold_Sub1","Well_10"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_10","Function_2"]*p_Well["Manifold_Sub1","Well_10"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_10","Function_2"]*q_Injected["Manifold_Sub1","Well_10"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_10","Function_2"]*q_Injected["Manifold_Sub1","Well_10"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_10","Function_2"]*p_Well["Manifold_Sub1","Well_10"] + 1)^(-1)
+(beta_Well["Manifold_Sub1","beta9","Well_10","Function_2"]*q_Injected["Manifold_Sub1","Well_10"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta10","Well_10","Function_2"]*p_Well["Manifold_Sub1","Well_10"]*q_Injected["Manifold_Sub1","Well_10"]
+ beta_Well["Manifold_Sub1","beta11","Well_10","Function_2"]*log(p_Well["Manifold_Sub1","Well_10"]+1)
+ beta_Well["Manifold_Sub1","beta12","Well_10","Function_2"]*log(q_Injected["Manifold_Sub1","Well_10"]+1))* s_Well["Manifold_Sub1","Well_10","Function_2"]; 

subject to Well_10_Function:
s_Well["Manifold_Sub1","Well_10","Function_1"]+s_Well["Manifold_Sub1","Well_10","Function_2"]<=1;

subject to Oil_Flow_Well_10_Function_1:
q_Injected["Manifold_Sub1","Well_10"] <= 110 + 200*s_Well["Manifold_Sub1","Well_10","Function_2"];

subject to Oil_Flow_Well_10_Function_2:
q_Injected["Manifold_Sub1","Well_10"] >= 110-110*s_Well["Manifold_Sub1","Well_10","Function_1"];


#Function 13
subject to Oil_Flow_Well_11:
q_Well ["Manifold_Sub1","Well_11","OIL"] = 
beta_Well["Manifold_Sub1","beta1","Well_11","Function_1"]
+ beta_Well["Manifold_Sub1","beta2","Well_11","Function_1"]*p_Well["Manifold_Sub1","Well_11"] 
+ beta_Well["Manifold_Sub1","beta3","Well_11","Function_1"]*q_Injected["Manifold_Sub1","Well_11"]
+ beta_Well["Manifold_Sub1","beta4","Well_11","Function_1"]*p_Well["Manifold_Sub1","Well_11"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_11","Function_1"]*p_Well["Manifold_Sub1","Well_11"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_11","Function_1"]*q_Injected["Manifold_Sub1","Well_11"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_11","Function_1"]*q_Injected["Manifold_Sub1","Well_11"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_11","Function_1"]*p_Well["Manifold_Sub1","Well_11"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta9","Well_11","Function_1"]*p_Well["Manifold_Sub1","Well_11"]*q_Injected["Manifold_Sub1","Well_11"]
+ beta_Well["Manifold_Sub1","beta10","Well_11","Function_1"]*log(p_Well["Manifold_Sub1","Well_11"]+1)
+ beta_Well["Manifold_Sub1","beta11","Well_11","Function_1"]*log(q_Injected["Manifold_Sub1","Well_11"]+1);


#Function 13
subject to Oil_Flow_Well_12:
q_Well ["Manifold_Sub1","Well_12","OIL"] = 
beta_Well["Manifold_Sub1","beta1","Well_12","Function_1"]
+ beta_Well["Manifold_Sub1","beta2","Well_12","Function_1"]*p_Well["Manifold_Sub1","Well_12"] 
+ beta_Well["Manifold_Sub1","beta3","Well_12","Function_1"]*q_Injected["Manifold_Sub1","Well_12"]
+ beta_Well["Manifold_Sub1","beta4","Well_12","Function_1"]*p_Well["Manifold_Sub1","Well_12"]^(2)
+ beta_Well["Manifold_Sub1","beta5","Well_12","Function_1"]*p_Well["Manifold_Sub1","Well_12"]^(3)
+ beta_Well["Manifold_Sub1","beta6","Well_12","Function_1"]*q_Injected["Manifold_Sub1","Well_12"]^(2)
+ beta_Well["Manifold_Sub1","beta7","Well_12","Function_1"]*q_Injected["Manifold_Sub1","Well_12"]^(3)
+(beta_Well["Manifold_Sub1","beta8","Well_12","Function_1"]*p_Well["Manifold_Sub1","Well_12"] + 1)^(-1)
+ beta_Well["Manifold_Sub1","beta9","Well_12","Function_1"]*p_Well["Manifold_Sub1","Well_12"]*q_Injected["Manifold_Sub1","Well_12"]
+ beta_Well["Manifold_Sub1","beta10","Well_12","Function_1"]*log(p_Well["Manifold_Sub1","Well_12"]+1)
+ beta_Well["Manifold_Sub1","beta11","Well_12","Function_1"]*log(q_Injected["Manifold_Sub1","Well_12"]+1);



#Function 12
subject to Oil_Flow_Well_13:
q_Well ["Manifold_Sub2","Well_13","OIL"] = 
(beta_Well["Manifold_Sub2","beta1","Well_13","Function_1"]
+ beta_Well["Manifold_Sub2","beta2","Well_13","Function_1"]*p_Well["Manifold_Sub2","Well_13"]
+ beta_Well["Manifold_Sub2","beta3","Well_13","Function_1"]*q_Injected["Manifold_Sub2","Well_13"]  
+ beta_Well["Manifold_Sub2","beta4","Well_13","Function_1"]*p_Well["Manifold_Sub2","Well_13"]^(2)
+ beta_Well["Manifold_Sub2","beta5","Well_13","Function_1"]*p_Well["Manifold_Sub2","Well_13"]^(3)
+ beta_Well["Manifold_Sub2","beta6","Well_13","Function_1"]*q_Injected["Manifold_Sub2","Well_13"]^(2)
+ beta_Well["Manifold_Sub2","beta7","Well_13","Function_1"]*q_Injected["Manifold_Sub2","Well_13"]^(3)
+(beta_Well["Manifold_Sub2","beta8","Well_13","Function_1"]*q_Injected["Manifold_Sub2","Well_13"] + 1)^(-1)
+ beta_Well["Manifold_Sub2","beta9","Well_13","Function_1"]*p_Well["Manifold_Sub2","Well_13"]*q_Injected["Manifold_Sub2","Well_13"]
+ beta_Well["Manifold_Sub2","beta10","Well_13","Function_1"]*log(p_Well["Manifold_Sub2","Well_13"] + 1)
+ beta_Well["Manifold_Sub2","beta11","Well_13","Function_1"]*log(q_Injected["Manifold_Sub2","Well_13"] + 1)) ; 


#Function 12
subject to Oil_Flow_Well_14:
q_Well ["Manifold_Sub2","Well_14","OIL"] = 
(beta_Well["Manifold_Sub2","beta1","Well_14","Function_1"]
+ beta_Well["Manifold_Sub2","beta2","Well_14","Function_1"]*p_Well["Manifold_Sub2","Well_14"]
+ beta_Well["Manifold_Sub2","beta3","Well_14","Function_1"]*q_Injected["Manifold_Sub2","Well_14"]  
+ beta_Well["Manifold_Sub2","beta4","Well_14","Function_1"]*p_Well["Manifold_Sub2","Well_14"]^(2)
+ beta_Well["Manifold_Sub2","beta5","Well_14","Function_1"]*p_Well["Manifold_Sub2","Well_14"]^(3)
+ beta_Well["Manifold_Sub2","beta6","Well_14","Function_1"]*q_Injected["Manifold_Sub2","Well_14"]^(2)
+ beta_Well["Manifold_Sub2","beta7","Well_14","Function_1"]*q_Injected["Manifold_Sub2","Well_14"]^(3)
+(beta_Well["Manifold_Sub2","beta8","Well_14","Function_1"]*q_Injected["Manifold_Sub2","Well_14"] + 1)^(-1)
+ beta_Well["Manifold_Sub2","beta9","Well_14","Function_1"]*p_Well["Manifold_Sub2","Well_14"]*q_Injected["Manifold_Sub2","Well_14"]
+ beta_Well["Manifold_Sub2","beta10","Well_14","Function_1"]*log(p_Well["Manifold_Sub2","Well_14"] + 1)
+ beta_Well["Manifold_Sub2","beta11","Well_14","Function_1"]*log(q_Injected["Manifold_Sub2","Well_14"] + 1)) ;



#Function 12
subject to Oil_Flow_Well_15:
q_Well ["Manifold_Sub2","Well_15","OIL"] = 
(beta_Well["Manifold_Sub2","beta1","Well_15","Function_1"]
+ beta_Well["Manifold_Sub2","beta2","Well_15","Function_1"]*p_Well["Manifold_Sub2","Well_15"]
+ beta_Well["Manifold_Sub2","beta3","Well_15","Function_1"]*q_Injected["Manifold_Sub2","Well_15"]  
+ beta_Well["Manifold_Sub2","beta4","Well_15","Function_1"]*p_Well["Manifold_Sub2","Well_15"]^(2)
+ beta_Well["Manifold_Sub2","beta5","Well_15","Function_1"]*p_Well["Manifold_Sub2","Well_15"]^(3)
+ beta_Well["Manifold_Sub2","beta6","Well_15","Function_1"]*q_Injected["Manifold_Sub2","Well_15"]^(2)
+ beta_Well["Manifold_Sub2","beta7","Well_15","Function_1"]*q_Injected["Manifold_Sub2","Well_15"]^(3)
+(beta_Well["Manifold_Sub2","beta8","Well_15","Function_1"]*q_Injected["Manifold_Sub2","Well_15"] + 1)^(-1)
+ beta_Well["Manifold_Sub2","beta9","Well_15","Function_1"]*p_Well["Manifold_Sub2","Well_15"]*q_Injected["Manifold_Sub2","Well_15"]
+ beta_Well["Manifold_Sub2","beta10","Well_15","Function_1"]*log(p_Well["Manifold_Sub2","Well_15"] + 1)
+ beta_Well["Manifold_Sub2","beta11","Well_15","Function_1"]*log(q_Injected["Manifold_Sub2","Well_15"] + 1)) ;


#Function 14
subject to Oil_Flow_Well_16:
q_Well ["Manifold_Sub2","Well_16","OIL"] == 
(beta_Well["Manifold_Sub2","beta1","Well_16","Function_1"]
+ beta_Well["Manifold_Sub2","beta2","Well_16","Function_1"]*p_Well["Manifold_Sub2","Well_16"] 
+ beta_Well["Manifold_Sub2","beta3","Well_16","Function_1"]*q_Injected["Manifold_Sub2","Well_16"]
+ beta_Well["Manifold_Sub2","beta4","Well_16","Function_1"]*p_Well["Manifold_Sub2","Well_16"]^(2)
+ beta_Well["Manifold_Sub2","beta5","Well_16","Function_1"]*p_Well["Manifold_Sub2","Well_16"]^(3)
+ beta_Well["Manifold_Sub2","beta6","Well_16","Function_1"]*q_Injected["Manifold_Sub2","Well_16"]^(2)
+ beta_Well["Manifold_Sub2","beta7","Well_16","Function_1"]*q_Injected["Manifold_Sub2","Well_16"]^(3)
+(beta_Well["Manifold_Sub2","beta8","Well_16","Function_1"]*p_Well["Manifold_Sub2","Well_16"] + 1)^(-1)
+(beta_Well["Manifold_Sub2","beta9","Well_16","Function_1"]*q_Injected["Manifold_Sub2","Well_16"] + 1)^(-1)
+ beta_Well["Manifold_Sub2","beta10","Well_16","Function_1"]*p_Well["Manifold_Sub2","Well_16"]*q_Injected["Manifold_Sub2","Well_16"]
+ beta_Well["Manifold_Sub2","beta11","Well_16","Function_1"]*log(p_Well["Manifold_Sub2","Well_16"]+1)
+ beta_Well["Manifold_Sub2","beta12","Well_16","Function_1"]*log(q_Injected["Manifold_Sub2","Well_16"]+1) );




#Function_1 14 Function_2 14
subject to Oil_Flow_Well_1:
q_Well ["Manifold_Top","Well_1","OIL"] == 
(beta_Well["Manifold_Top","beta1","Well_1","Function_1"]
+ beta_Well["Manifold_Top","beta2","Well_1","Function_1"]*p_Well["Manifold_Top","Well_1"] 
+ beta_Well["Manifold_Top","beta3","Well_1","Function_1"]*q_Injected["Manifold_Top","Well_1"]
+ beta_Well["Manifold_Top","beta4","Well_1","Function_1"]*p_Well["Manifold_Top","Well_1"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_1","Function_1"]*p_Well["Manifold_Top","Well_1"]^(3)
+ beta_Well["Manifold_Top","beta6","Well_1","Function_1"]*q_Injected["Manifold_Top","Well_1"]^(2)
+ beta_Well["Manifold_Top","beta7","Well_1","Function_1"]*q_Injected["Manifold_Top","Well_1"]^(3)
+(beta_Well["Manifold_Top","beta8","Well_1","Function_1"]*p_Well["Manifold_Top","Well_1"] + 1)^(-1)
+(beta_Well["Manifold_Top","beta9","Well_1","Function_1"]*q_Injected["Manifold_Top","Well_1"] + 1)^(-1)
+ beta_Well["Manifold_Top","beta10","Well_1","Function_1"]*p_Well["Manifold_Top","Well_1"]*q_Injected["Manifold_Top","Well_1"]
+ beta_Well["Manifold_Top","beta11","Well_1","Function_1"]*log(p_Well["Manifold_Top","Well_1"]+1)
+ beta_Well["Manifold_Top","beta12","Well_1","Function_1"]*log(q_Injected["Manifold_Top","Well_1"]+1) )*s_Well["Manifold_Top","Well_1","Function_1"]


+(beta_Well["Manifold_Top","beta1","Well_1","Function_2"]
+ beta_Well["Manifold_Top","beta2","Well_1","Function_2"]*p_Well["Manifold_Top","Well_1"] 
+ beta_Well["Manifold_Top","beta3","Well_1","Function_2"]*q_Injected["Manifold_Top","Well_1"]
+ beta_Well["Manifold_Top","beta4","Well_1","Function_2"]*p_Well["Manifold_Top","Well_1"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_1","Function_2"]*p_Well["Manifold_Top","Well_1"]^(3)
+ beta_Well["Manifold_Top","beta6","Well_1","Function_2"]*q_Injected["Manifold_Top","Well_1"]^(2)
+ beta_Well["Manifold_Top","beta7","Well_1","Function_2"]*q_Injected["Manifold_Top","Well_1"]^(3)
+(beta_Well["Manifold_Top","beta8","Well_1","Function_2"]*p_Well["Manifold_Top","Well_1"] + 1)^(-1)
+(beta_Well["Manifold_Top","beta9","Well_1","Function_2"]*q_Injected["Manifold_Top","Well_1"] + 1)^(-1)
+ beta_Well["Manifold_Top","beta10","Well_1","Function_2"]*p_Well["Manifold_Top","Well_1"]*q_Injected["Manifold_Top","Well_1"]
+ beta_Well["Manifold_Top","beta11","Well_1","Function_2"]*log(p_Well["Manifold_Top","Well_1"]+1)
+ beta_Well["Manifold_Top","beta12","Well_1","Function_2"]*log(q_Injected["Manifold_Top","Well_1"]+1))*s_Well["Manifold_Top","Well_1","Function_2"];

subject to Well_1_Function:
s_Well["Manifold_Top","Well_1","Function_1"]+s_Well["Manifold_Top","Well_1","Function_2"]<=1;

subject to Oil_Flow_Well_1_Function_1:
p_Well["Manifold_Top","Well_1"] <= 30 + 200*s_Well["Manifold_Top","Well_1","Function_2"];

subject to Oil_Flow_Well_1_Function_2:
p_Well["Manifold_Top","Well_1"] >= 30-30*s_Well["Manifold_Top","Well_1","Function_1"];


#Function 8 

subject to Oil_Flow_Well_2:
q_Well ["Manifold_Top","Well_2","OIL"] = 
beta_Well["Manifold_Top","beta1","Well_2","Function_1"]
+ beta_Well["Manifold_Top","beta2","Well_2","Function_1"]*p_Well["Manifold_Top","Well_2"]
+ beta_Well["Manifold_Top","beta3","Well_2","Function_1"]*q_Injected["Manifold_Top","Well_2"]  
+ beta_Well["Manifold_Top","beta4","Well_2","Function_1"]*p_Well["Manifold_Top","Well_2"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_2","Function_1"]*q_Injected["Manifold_Top","Well_2"]^(2)
+ beta_Well["Manifold_Top","beta6","Well_2","Function_1"]*p_Well["Manifold_Top","Well_2"]*q_Injected["Manifold_Top","Well_2"]
+(beta_Well["Manifold_Top","beta7","Well_2","Function_1"]*p_Well["Manifold_Top","Well_2"] + 1)^(-2)
+(beta_Well["Manifold_Top","beta8","Well_2","Function_1"]*q_Injected["Manifold_Top","Well_2"] + 1)^(-2)
+ beta_Well["Manifold_Top","beta9","Well_2","Function_1"]*log(p_Well["Manifold_Top","Well_2"] + 1)
+ beta_Well["Manifold_Top","beta10","Well_2","Function_1"]*log(q_Injected["Manifold_Top","Well_2"] + 1);


#Function 11
subject to Oil_Flow_Well_3:
q_Well ["Manifold_Top","Well_3","OIL"] = 
(beta_Well["Manifold_Top","beta1","Well_3","Function_1"]
+ beta_Well["Manifold_Top","beta2","Well_3","Function_1"]*p_Well["Manifold_Top","Well_3"]
+ beta_Well["Manifold_Top","beta3","Well_3","Function_1"]*q_Injected["Manifold_Top","Well_3"]  
+ beta_Well["Manifold_Top","beta4","Well_3","Function_1"]*p_Well["Manifold_Top","Well_3"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_3","Function_1"]*p_Well["Manifold_Top","Well_3"]^(3)
+ beta_Well["Manifold_Top","beta6","Well_3","Function_1"]*q_Injected["Manifold_Top","Well_3"]^(2)
+ beta_Well["Manifold_Top","beta7","Well_3","Function_1"]*q_Injected["Manifold_Top","Well_3"]^(3)
+ beta_Well["Manifold_Top","beta8","Well_3","Function_1"]*p_Well["Manifold_Top","Well_3"]*q_Injected["Manifold_Top","Well_3"]
+ beta_Well["Manifold_Top","beta9","Well_3","Function_1"]*log(p_Well["Manifold_Top","Well_3"] + 1)
+ beta_Well["Manifold_Top","beta10","Well_3","Function_1"]*log(q_Injected["Manifold_Top","Well_3"] + 1));

#Function 14
subject to Oil_Flow_Well_4:
q_Well ["Manifold_Top","Well_4","OIL"] =
beta_Well["Manifold_Top","beta1","Well_4","Function_1"]
+ beta_Well["Manifold_Top","beta2","Well_4","Function_1"]*p_Well["Manifold_Top","Well_4"] 
+ beta_Well["Manifold_Top","beta3","Well_4","Function_1"]*q_Injected["Manifold_Top","Well_4"]
+ beta_Well["Manifold_Top","beta4","Well_4","Function_1"]*p_Well["Manifold_Top","Well_4"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_4","Function_1"]*p_Well["Manifold_Top","Well_4"]^(3)
+ beta_Well["Manifold_Top","beta6","Well_4","Function_1"]*q_Injected["Manifold_Top","Well_4"]^(2)
+ beta_Well["Manifold_Top","beta7","Well_4","Function_1"]*q_Injected["Manifold_Top","Well_4"]^(3)
+(beta_Well["Manifold_Top","beta8","Well_4","Function_1"]*p_Well["Manifold_Top","Well_4"] + 1)^(-1)
+(beta_Well["Manifold_Top","beta9","Well_4","Function_1"]*q_Injected["Manifold_Top","Well_4"] + 1)^(-1)
+ beta_Well["Manifold_Top","beta10","Well_4","Function_1"]*p_Well["Manifold_Top","Well_4"]*q_Injected["Manifold_Top","Well_4"]
+ beta_Well["Manifold_Top","beta11","Well_4","Function_1"]*log(p_Well["Manifold_Top","Well_4"]+1)
+ beta_Well["Manifold_Top","beta12","Well_4","Function_1"]*log(q_Injected["Manifold_Top","Well_4"]+1); 


#Function 12
subject to Oil_Flow_Well_5:
q_Well ["Manifold_Top","Well_5","OIL"] = 
beta_Well["Manifold_Top","beta1","Well_5","Function_1"]
+ beta_Well["Manifold_Top","beta2","Well_5","Function_1"]*p_Well["Manifold_Top","Well_5"]
+ beta_Well["Manifold_Top","beta3","Well_5","Function_1"]*q_Injected["Manifold_Top","Well_5"]  
+ beta_Well["Manifold_Top","beta4","Well_5","Function_1"]*p_Well["Manifold_Top","Well_5"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_5","Function_1"]*p_Well["Manifold_Top","Well_5"]^(3)
+ beta_Well["Manifold_Top","beta6","Well_5","Function_1"]*q_Injected["Manifold_Top","Well_5"]^(2)
+ beta_Well["Manifold_Top","beta7","Well_5","Function_1"]*q_Injected["Manifold_Top","Well_5"]^(3)
+(beta_Well["Manifold_Top","beta8","Well_5","Function_1"]*q_Injected["Manifold_Top","Well_5"] + 1)^(-1)
+ beta_Well["Manifold_Top","beta9","Well_5","Function_1"]*p_Well["Manifold_Top","Well_5"]*q_Injected["Manifold_Top","Well_5"]
+ beta_Well["Manifold_Top","beta10","Well_5","Function_1"]*log(p_Well["Manifold_Top","Well_5"] + 1)
+ beta_Well["Manifold_Top","beta11","Well_5","Function_1"]*log(q_Injected["Manifold_Top","Well_5"] + 1);

#Function 13
subject to Oil_Flow_Well_6:
q_Well ["Manifold_Top","Well_6","OIL"] = 
beta_Well["Manifold_Top","beta1","Well_6","Function_1"]
+ beta_Well["Manifold_Top","beta2","Well_6","Function_1"]*p_Well["Manifold_Top","Well_6"] 
+ beta_Well["Manifold_Top","beta3","Well_6","Function_1"]*q_Injected["Manifold_Top","Well_6"]
+ beta_Well["Manifold_Top","beta4","Well_6","Function_1"]*p_Well["Manifold_Top","Well_6"]^(2)
+ beta_Well["Manifold_Top","beta5","Well_6","Function_1"]*p_Well["Manifold_Top","Well_6"]^(3)
+ beta_Well["Manifold_Top","beta6","Well_6","Function_1"]*q_Injected["Manifold_Top","Well_6"]^(2)
+ beta_Well["Manifold_Top","beta7","Well_6","Function_1"]*q_Injected["Manifold_Top","Well_6"]^(3)
+(beta_Well["Manifold_Top","beta8","Well_6","Function_1"]*p_Well["Manifold_Top","Well_6"] + 1)^(-1)
+ beta_Well["Manifold_Top","beta9","Well_6","Function_1"]*p_Well["Manifold_Top","Well_6"]*q_Injected["Manifold_Top","Well_6"]
+ beta_Well["Manifold_Top","beta10","Well_6","Function_1"]*log(p_Well["Manifold_Top","Well_6"]+1)
+ beta_Well["Manifold_Top","beta11","Well_6","Function_1"]*log(q_Injected["Manifold_Top","Well_6"]+1);











#PRESSURE DROP AS A FUNCTION OF GAS, OIL AND WATER FLOW IN THE PIPELINES CONNECTING SUBSEA MANIFOLDS TO THE PLATFORM


#Restriction 5.15





subject to Choise_Functions_Pipe_1 :
s_Pipeline["Manifold_Sub1","Pipe_1","Function_11"] + s_Pipeline["Manifold_Sub1","Pipe_1","Function_12"] + s_Pipeline["Manifold_Sub1","Pipe_1","Function_21"] + s_Pipeline["Manifold_Sub1","Pipe_1","Function_22"] = 1;


subject to Choise_Functions_Pipe_2 :
s_Pipeline["Manifold_Sub1","Pipe_2","Function_11"] + s_Pipeline["Manifold_Sub1","Pipe_2","Function_12"] + s_Pipeline["Manifold_Sub1","Pipe_2","Function_21"] + s_Pipeline["Manifold_Sub1","Pipe_2","Function_22"] = 1;



subject to Choise_Functions_Pipe_3 :
s_Pipeline["Manifold_Sub2","Pipe_3","Function_11"] + s_Pipeline["Manifold_Sub2","Pipe_3","Function_12"] + s_Pipeline["Manifold_Sub2","Pipe_3","Function_21"] + s_Pipeline["Manifold_Sub2","Pipe_3","Function_22"] = 1;


subject to Choise_Functions_Pipe_4 :
s_Pipeline["Manifold_Sub2","Pipe_4","Function_11"] + s_Pipeline["Manifold_Sub2","Pipe_4","Function_12"] + s_Pipeline["Manifold_Sub2","Pipe_4","Function_21"] + s_Pipeline["Manifold_Sub2","Pipe_4","Function_22"] = 1;





subject to GOR_Limit_ManSub1 {l in Pipeline_Manifold ["Manifold_Sub1"]}:
q_Pipeline ["Manifold_Sub1",l,"GAS"]*1000/q_Pipeline ["Manifold_Sub1",l,"OIL"] <= 550;

subject to WC_Limit_ManSub1 {l in Pipeline_Manifold ["Manifold_Sub1"]}:
wc_Pipeline["Manifold_Sub1",l] >= 0.13;

subject to GOR_Limit_ManSub2 {l in Pipeline_Manifold ["Manifold_Sub2"]}:
q_Pipeline ["Manifold_Sub2",l,"GAS"]*1000/q_Pipeline ["Manifold_Sub2",l,"OIL"] <= 550;

subject to WC_Limit_ManSub2 {l in Pipeline_Manifold ["Manifold_Sub2"]}:
wc_Pipeline["Manifold_Sub2",l] >= 0.13;

subject to Oil_FLow_Pipeline {m in Manifold_Subsea, l in Pipeline_Manifold[m]}:
q_Pipeline [m,l,"OIL"] = q_Pipeline_Liquid [m,l]*(1-wc_Pipeline[m, l]);


subject to Water_FLow_Pipeline {m in Manifold_Subsea, l in Pipeline_Manifold[m]}:
q_Pipeline [m,l,"WATER"] = q_Pipeline_Liquid [m,l]*wc_Pipeline[m, l];



subject to Pressure_Drop_Pipe_1_WC_Big:
wc_Pipeline["Manifold_Sub1","Pipe_1"] >= 0.444444 - 0.444444*(s_Pipeline["Manifold_Sub1","Pipe_1","Function_11"]+s_Pipeline["Manifold_Sub1","Pipe_1","Function_21"]);

subject to Pressure_Drop_Pipe_1_WC_Small:
wc_Pipeline["Manifold_Sub1","Pipe_1"]<= 0.444444 + 0.444444* (s_Pipeline["Manifold_Sub1","Pipe_1","Function_12"]+s_Pipeline["Manifold_Sub1","Pipe_1","Function_22"]);


subject to Pressure_Drop_Pipe_1_GAS_Big:
q_Pipeline["Manifold_Sub1","Pipe_1","GAS"] >= 400 - 400*(s_Pipeline["Manifold_Sub1","Pipe_1","Function_11"]+s_Pipeline["Manifold_Sub1","Pipe_1","Function_12"]);

subject to Pressure_Drop_Pipe_1_GAS_Small:
q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]<= 400 + 800* (s_Pipeline["Manifold_Sub1","Pipe_1","Function_21"]+s_Pipeline["Manifold_Sub1","Pipe_1","Function_22"]);





subject to Pressure_Drop_Pipe_2_WC_Big:
wc_Pipeline["Manifold_Sub1","Pipe_2"] >= 0.444444 - 0.444444*(s_Pipeline["Manifold_Sub1","Pipe_2","Function_11"]+s_Pipeline["Manifold_Sub1","Pipe_2","Function_21"]);

subject to Pressure_Drop_Pipe_2_WC_Small:
wc_Pipeline["Manifold_Sub1","Pipe_2"]<= 0.444444 + 0.55* (s_Pipeline["Manifold_Sub1","Pipe_2","Function_12"]+s_Pipeline["Manifold_Sub1","Pipe_2","Function_22"]);


subject to Pressure_Drop_Pipe_2_GAS_Big:
q_Pipeline["Manifold_Sub1","Pipe_2","GAS"] >= 400 - 400* (s_Pipeline["Manifold_Sub1","Pipe_2","Function_11"]+s_Pipeline["Manifold_Sub1","Pipe_2","Function_12"]);

subject to Pressure_Drop_Pipe_2_GAS_Small:
q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]<= 400 + 800*(s_Pipeline["Manifold_Sub1","Pipe_2","Function_21"]+s_Pipeline["Manifold_Sub1","Pipe_2","Function_22"]);



subject to Pressure_Drop_Pipe_3_WC_Big:
wc_Pipeline["Manifold_Sub2","Pipe_3"] >= 0.444444 - 0.444444*(s_Pipeline["Manifold_Sub2","Pipe_3","Function_11"]+s_Pipeline["Manifold_Sub2","Pipe_3","Function_21"]);

subject to Pressure_Drop_Pipe_3_WC_Small:
wc_Pipeline["Manifold_Sub2","Pipe_3"]<= 0.444444 + 0.444444* (s_Pipeline["Manifold_Sub2","Pipe_3","Function_12"]+s_Pipeline["Manifold_Sub2","Pipe_3","Function_22"]);


subject to Pressure_Drop_Pipe_3_GAS_Big:
q_Pipeline["Manifold_Sub2","Pipe_3","GAS"] >= 400 - 400*(s_Pipeline["Manifold_Sub2","Pipe_3","Function_11"]+s_Pipeline["Manifold_Sub2","Pipe_3","Function_12"]);

subject to Pressure_Drop_Pipe_3_GAS_Small:
q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]<= 400 + 800* (s_Pipeline["Manifold_Sub2","Pipe_3","Function_21"]+s_Pipeline["Manifold_Sub2","Pipe_3","Function_22"]);





subject to Pressure_Drop_Pipe_4_WC_Big:
wc_Pipeline["Manifold_Sub2","Pipe_4"] >= 0.444444 - 0.444444*(s_Pipeline["Manifold_Sub2","Pipe_4","Function_11"]+s_Pipeline["Manifold_Sub2","Pipe_4","Function_21"]);

subject to Pressure_Drop_Pipe_4_WC_Small:
wc_Pipeline["Manifold_Sub2","Pipe_4"]<= 0.444444 + 0.55* (s_Pipeline["Manifold_Sub2","Pipe_4","Function_12"]+s_Pipeline["Manifold_Sub2","Pipe_4","Function_22"]);


subject to Pressure_Drop_Pipe_4_GAS_Big:
q_Pipeline["Manifold_Sub2","Pipe_4","GAS"] >= 400 - 400* (s_Pipeline["Manifold_Sub2","Pipe_4","Function_11"]+s_Pipeline["Manifold_Sub2","Pipe_4","Function_12"]);

subject to Pressure_Drop_Pipe_4_GAS_Small:
q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]<= 400 + 800*(s_Pipeline["Manifold_Sub2","Pipe_4","Function_21"]+s_Pipeline["Manifold_Sub2","Pipe_4","Function_22"]);





subject to Liquid_Limit {m in Manifold_Subsea, l in Pipeline_Manifold[m]}:
q_Pipeline_Liquid[m, l] <= 1200/wc_Pipeline[m, l];




subject to Pressure_Drop_Pipe_1:
p_Drop_Pipeline ["Manifold_Sub1", "Pipe_1"] =
(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_1", "Function_11"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_1", "Function_11"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_1", "Function_11"]*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_1", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_1", "Function_11"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_1", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta7", "Pipe_1", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_1", "Function_11"]*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_1", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_1", "Function_11"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_1", "Function_11"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_1", "Function_11"]*((log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_1","Function_11"]


+(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_1", "Function_12"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_1", "Function_12"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_1", "Function_12"]*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_1", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_1", "Function_12"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_1", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta7", "Pipe_1", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_1", "Function_12"]*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_1", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_1", "Function_12"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_1", "Function_12"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_1", "Function_12"]*((log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_1","Function_12"]


+(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_1", "Function_21"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_1", "Function_21"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_1", "Function_21"]*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_1", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_1", "Function_21"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_1", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_1", "Function_21"]*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_1", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_1", "Function_21"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_1", "Function_21"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_1", "Function_21"]*((log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_1","Function_21"]


+(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_1", "Function_22"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_1", "Function_22"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_1", "Function_22"]*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_1", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_1", "Function_22"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_1", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta7", "Pipe_1", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1)
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_1", "Function_22"]*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_1", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_1", "Function_22"]*(log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_1", "Function_22"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_1"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_1", "Function_22"]*((log(q_Pipeline["Manifold_Sub1","Pipe_1","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_1"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_1","Function_22"];

subject to Pressure_Drop_Pipe_2:
p_Drop_Pipeline ["Manifold_Sub1", "Pipe_2"] =
(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_2", "Function_11"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_2", "Function_11"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_2", "Function_11"]*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_2", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_2", "Function_11"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_2", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta7", "Pipe_2", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_2", "Function_11"]*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_2", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_2", "Function_11"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_2", "Function_11"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_2", "Function_11"]*((log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_2","Function_11"]


+(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_2", "Function_12"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_2", "Function_12"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_2", "Function_12"]*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_2", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_2", "Function_12"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_2", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta7", "Pipe_2", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_2", "Function_12"]*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_2", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_2", "Function_12"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_2", "Function_12"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_2", "Function_12"]*((log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_2","Function_12"]


+(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_2", "Function_21"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_2", "Function_21"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_2", "Function_21"]*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_2", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_2", "Function_21"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_2", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_2", "Function_21"]*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_2", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_2", "Function_21"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_2", "Function_21"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_2", "Function_21"]*((log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_2","Function_21"]


+(beta_Pipeline["Manifold_Sub1","beta1", "Pipe_2", "Function_22"] 
+ beta_Pipeline["Manifold_Sub1","beta2", "Pipe_2", "Function_22"]* log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1) 
+ beta_Pipeline["Manifold_Sub1","beta3", "Pipe_2", "Function_22"]*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub1","beta4", "Pipe_2", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta5", "Pipe_2", "Function_22"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub1","beta6", "Pipe_2", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3 
+ beta_Pipeline["Manifold_Sub1","beta7", "Pipe_2", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1)
+ beta_Pipeline["Manifold_Sub1","beta8", "Pipe_2", "Function_22"]*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta9", "Pipe_2", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)  
+ beta_Pipeline["Manifold_Sub1","beta10", "Pipe_2", "Function_22"]*(log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118) 
+ beta_Pipeline["Manifold_Sub1","beta11", "Pipe_2", "Function_22"]*((log(q_Pipeline_Liquid["Manifold_Sub1", "Pipe_2"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118)
+ beta_Pipeline["Manifold_Sub1","beta12", "Pipe_2", "Function_22"]*((log(q_Pipeline["Manifold_Sub1","Pipe_2","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub1", "Pipe_2"]-0.118))*s_Pipeline["Manifold_Sub1","Pipe_2","Function_22"];




subject to Pressure_Drop_Pipe_3:
p_Drop_Pipeline ["Manifold_Sub2", "Pipe_3"] =
(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_3", "Function_11"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_3", "Function_11"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_3", "Function_11"]*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_3", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_3", "Function_11"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_3", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta7", "Pipe_3", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_3", "Function_11"]*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_3", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_3", "Function_11"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_3", "Function_11"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_3", "Function_11"]*((log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_3","Function_11"]


+(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_3", "Function_12"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_3", "Function_12"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_3", "Function_12"]*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_3", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_3", "Function_12"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_3", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta7", "Pipe_3", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_3", "Function_12"]*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_3", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_3", "Function_12"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_3", "Function_12"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_3", "Function_12"]*((log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_3","Function_12"]


+(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_3", "Function_21"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_3", "Function_21"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_3", "Function_21"]*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_3", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_3", "Function_21"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_3", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_3", "Function_21"]*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_3", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_3", "Function_21"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_3", "Function_21"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_3", "Function_21"]*((log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_3","Function_21"]


+(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_3", "Function_22"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_3", "Function_22"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_3", "Function_22"]*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_3", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_3", "Function_22"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_3", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta7", "Pipe_3", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1)
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_3", "Function_22"]*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_3", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_3", "Function_22"]*(log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_3", "Function_22"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_3"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_3", "Function_22"]*((log(q_Pipeline["Manifold_Sub2","Pipe_3","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_3"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_3","Function_22"];

subject to Pressure_Drop_Pipe_4:
p_Drop_Pipeline ["Manifold_Sub2", "Pipe_4"] =
(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_4", "Function_11"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_4", "Function_11"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_4", "Function_11"]*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_4", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_4", "Function_11"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_4", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta7", "Pipe_4", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_4", "Function_11"]*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_4", "Function_11"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_4", "Function_11"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_4", "Function_11"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_4", "Function_11"]*((log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_4","Function_11"]


+(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_4", "Function_12"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_4", "Function_12"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_4", "Function_12"]*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_4", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_4", "Function_12"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_4", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta7", "Pipe_4", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_4", "Function_12"]*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_4", "Function_12"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_4", "Function_12"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_4", "Function_12"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_4", "Function_12"]*((log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_4","Function_12"]


+(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_4", "Function_21"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_4", "Function_21"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_4", "Function_21"]*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_4", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_4", "Function_21"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_4", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_4", "Function_21"]*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_4", "Function_21"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_4", "Function_21"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_4", "Function_21"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_4", "Function_21"]*((log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_4","Function_21"]


+(beta_Pipeline["Manifold_Sub2","beta1", "Pipe_4", "Function_22"] 
+ beta_Pipeline["Manifold_Sub2","beta2", "Pipe_4", "Function_22"]* log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1) 
+ beta_Pipeline["Manifold_Sub2","beta3", "Pipe_4", "Function_22"]*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)  
+ beta_Pipeline["Manifold_Sub2","beta4", "Pipe_4", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta5", "Pipe_4", "Function_22"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^2 
+ beta_Pipeline["Manifold_Sub2","beta6", "Pipe_4", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3 
+ beta_Pipeline["Manifold_Sub2","beta7", "Pipe_4", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1)
+ beta_Pipeline["Manifold_Sub2","beta8", "Pipe_4", "Function_22"]*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta9", "Pipe_4", "Function_22"]*(log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)  
+ beta_Pipeline["Manifold_Sub2","beta10", "Pipe_4", "Function_22"]*(log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121) 
+ beta_Pipeline["Manifold_Sub2","beta11", "Pipe_4", "Function_22"]*((log(q_Pipeline_Liquid["Manifold_Sub2", "Pipe_4"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121)
+ beta_Pipeline["Manifold_Sub2","beta12", "Pipe_4", "Function_22"]*((log(q_Pipeline["Manifold_Sub2","Pipe_4","GAS"]+1))^3)*(wc_Pipeline["Manifold_Sub2", "Pipe_4"]-0.1121))*s_Pipeline["Manifold_Sub2","Pipe_4","Function_22"];

