// (C) Copyright Carnegie Mellon University 2006, 2007
// All Rights Reserved.
// This code is published under the Common Public License.
//
// Authors :
// P. Bonami, Carnegie Mellon University
//
// Date :  03/17/2006


#if defined(_MSC_VER)
// Turn off compiler warning about long names
#pragma warning(disable:4786)
#endif
#include <iomanip>
#include <fstream>
#include <list>
#include <queue>
using namespace std;

#include "IpIpoptApplication.hpp"
#include "MyTNLP.hpp"
#include "MyTMINLP.hpp" //------------------------------------------

#include "CoinTime.hpp"
#include "CoinError.hpp"

#include "BonOsiTMINLPInterface.hpp"
#include "BonIpoptSolver.hpp"

#include "BonCbc.hpp"
#include "BonBonminSetup.hpp"

#include "BonOACutGenerator2.hpp"
#include "BonEcpCuts.hpp"
#include "BonOaNlpOptim.hpp"

//Sheri
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <time.h>
#include <string>
#include <sys/time.h>
#include <sys/resource.h>
using namespace std;
using namespace Ipopt;
using namespace Bonmin;

bool SortObj(const Node& n1, const Node& n2){
    double n1o=n1.get_obj();
    double n2o=n2.get_obj();
    return (n1o < n2o);
}

double getthetime(){
    struct timeval tim;        
    struct rusage ru;        
    getrusage(RUSAGE_SELF, &ru);        
    //tim=ru.ru_utime;        
    //double t=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;        
    tim=ru.ru_stime;
    double t=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;   
    //t+=(double)tim.tv_sec + (double)tim.tv_usec / 1000000.0;        
    return t;
}

int main (int argc, char *argv[])
{
    WindowsErrorPopupBlocker(); //-----------------------
    
    //Variables to record time
    time_t Start_t, End_t;
    int time_msr, time_mss, time_bbt;

    //Pointers
    SmartPtr<MyTMINLP> tminlp = new MyTMINLP;
    SmartPtr<MyTMILP> tmilp = new MyTMILP;
    FP_Ptr fp_Ptr = new Feasibility_Pump;
    
//     #ifdef REDIRECT
//     FILE * fp = fopen("logg.out","w");
//     CoinMessageHandler handler(fp);
//     BonminSetup bonmin(&handler);
//     #else
//     BonminSetup bonmin;
//     #endif
//     bonmin.initializeOptionsAndJournalist();
//     //Register an additional option
//     bonmin.roptions()->AddStringOption2("print_solution","Do we print the solution or not?",
//                                  "yes",
//                                  "no", "No, we don't.",
//                                  "yes", "Yes, we do.",
//                                  "A longer comment can be put here");
    
    //Set problem values
        const double* fp_nlp_sol; //Solutions of the NLP(ipopt) will be saved here
        const double* milp_sol;
        const int MaxIt=1;
        int It=0;
        double Best_Sol=0;
  
//         tminlp->init_tangentValues();
//         tminlp->set_itt();
//         tminlp->readWells();
//         tminlp->readPipes();
//         tminlp->readminmax_p();
//         tminlp->readminmax_gl();
//         tminlp->set_nlp();
//         tminlp->set_It(It);
//         tminlp->set_bounds();
//   
//         tmilp->init_tangentValues();
//         tmilp->set_itt();
//         tmilp->readWells();
//         tmilp->readPipes();
//         tmilp->readminmax_p();
//         tmilp->readminmax_gl();
//         tmilp->set_nlp();
//         tmilp->set_It(It);
        
    SmartPtr<MyTNLP> tnlp = new MyTNLP;
    SmartPtr<IpoptApplication> ipoptApplication =IpoptApplicationFactory();
//ipoptApplication->Options()->SetIntegerValue("max_iter", 1000)());
    
    tnlp->set_itt();
    tnlp->readWells();
    tnlp->readPipes();
    
//Testing on userdefined Stack class
//which includes Standard List Container Class as a member
    Stack bbList;
    bbList.init_LB();

//MAKING RANDOM GOR/WATERCUTS    
//     srand(time(NULL));
//         for(int k=0; k<32; k++){
//         cout<<"0."<<bbList.random_int(75,100)<<endl;
//         }
//         for(int k=0; k<32; k++){
//         cout<<bbList.random_double(0.99,1.25)<<endl;
//         }
    
   
    int int_sol = 0;
    double ofu;
    int lev_lim=1;
    int frac_lim=20;
    
//Initialize the static groups array    
    Node::Groups();
    
//Make the root node
    Node root(NULL);
    tnlp->extract_mmw(root); 
    
//MULTIPLE START TESTING
    int rounds = 0;
    Start_t=time(NULL);
//     bbList.multiple_start(root.mmw);
//         
//     //Run each random starting point, in the list rand_start once  
//     list<Node>::iterator rand_it = bbList.rand_start.begin();
//     for(rand_it; rand_it!=bbList.rand_start.end(); rand_it++){
//         rounds++;
//         //rand_it->print_bounds_start();
// 
//         //Set the bounds and startpoints of the ipopt problem tnlp equal
//         //to the bounds and startpoints of the node chosen to be solved
//         tnlp->equate_bounds(*rand_it);
//         tnlp->equate_start(*rand_it);
//             
//         //Setup completed, call ipopt and return status of run
//         ApplicationReturnStatus status = ipoptApplication->Initialize();
//         status=ipoptApplication->OptimizeTNLP(GetRawPtr(tnlp));
//         rand_it->save_stat(tnlp->s);
//         cout<<"  STATUS1: "<<status<<"  STATUS2: "<<tnlp->s<<"  ROUND: "<<rounds<<endl;
//             
//         if (tnlp->s < 1){
//             //Increase the level of the node if s==1 -> IpOpt has reached max_iter
// //             if (tnlp->s == 1){
// //                 rand_it->inc_level();
// //             }
//             
//             //Save the variable- and objective function values of
//             //the NLP into the (root)node member array sol and var objf
//             rand_it->save_sol(tnlp->solution);
//             rand_it->save_obj(tnlp->obj);
//             
//             rand_it->set_parent_obj(tnlp->obj);
//             rand_it->equate_sol_to_start();
//                 
//             //Here the solution is tested to see if it is integer feasible.
//             int_sol = rand_it->is_int_feasible(frac_lim);            
//             
//             //If the solution is integer feasible AND s==0, 
//             //LB is checked and updated, then the node is removed
//             if ( int_sol == 1 ){
//                 cout<<"    CONGRATS!!      ";
//                 int a=0;
//                 if (tnlp->s == 0){
//                 	//Update the Lower Bound
//                     a=bbList.check_LB(rand_it->get_obj(), *rand_it, 0);
//                 }
//              }
//         }
//         else{
//         	cout<<"   Status has Failed"<<endl;
//             rand_it->save_obj(-1);
//         }
//     }
    End_t = time(NULL);
    time_msr=difftime(End_t, Start_t);  //Time it took to generate and solve the startpoints   
    cout<<"time_msr: "<<time_msr<<endl;
      
    Start_t=time(NULL);    
//     bbList.rand_start.sort(SortObj);   
// 
//     //Print the nodes in rand_start list 
//     for(rand_it=bbList.rand_start.begin(); rand_it!=bbList.rand_start.end(); rand_it++){
//         rand_it->print_node();
//     }
//     
//     //Put the x best nodes into the BB list
//     rand_it = bbList.rand_start.end();
//     rand_it--;
//     cout<<"FIRST MULTISTART NODE"<<endl;
//     bbList.BB.push_front(*rand_it); //Use the best one as the first cluster
//     rand_it->print_node();
//     list<Node>::iterator rand_it2=bbList.rand_start.end(); rand_it2--;rand_it2--; //Make another pointer pointing to the previous element
//     //cout<<"PRINTING THE START OF MULTISTART TESTING"<<endl;
//     //rand_it->print_node();
//     //rand_it2->print_node();
//     cout<<"xxxxxxxxxxxxxCLUSTERING AND PUTTING STARTPOINTS IN BB LISTxxxxxxxxxxx"<<endl;
//     int i_ms=0;
//     while (i_ms<clusters){
//         
//         if ( bbList.nodes_in_cluster(*rand_it, *rand_it2)){
//              rand_it2--;
//         }
//         else{
//             rand_it=rand_it2;
//             bbList.BB.push_front(*rand_it);
//             rand_it->print_node();
//             rand_it2--;
//             i_ms++;
//         }
//         
//         
//        
//     }
// 
//     //Clear the rand_start container to free memory
//     bbList.rand_start.clear();
        
    End_t = time(NULL);
    time_mss=difftime(End_t, Start_t); //Time it took to sort the list of multiple starting points 
    cout<<"time_mss: "<<time_mss<<endl;
    
//Setting the start- and variabel bound values of the root node
    root.set_start();
    root.set_boundz();
    
    //Read the multistart nodes and put them in the BB list
//     Node m1(NULL);
//     tnlp->extract_mmw(m1);
//     m1.read_save_start("start1.txt", 1);
//     m1.set_boundz();
//     bbList.BB.push_front(m1);
//     //m1.print_bounds_start();
    
//     Node m2(NULL);
//     tnlp->extract_mmw(m2);
//     m2.read_save_start("feas_start2.txt", 2);
//     m2.set_boundz();
//     bbList.BB.push_front(m2); 
//     //m2.print_bounds_start();
    
//     Node m3(NULL);
//     tnlp->extract_mmw(m3);
//     m3.read_save_start("feas_start3.txt", 3);
//     m3.set_boundz();
//     bbList.BB.push_front(m3); 
//     //m3.print_bounds_start();
//     
//     Node m4(NULL);
//     tnlp->extract_mmw(m4);
//     m4.read_save_start("feas_start4.txt", 4);  
//     m4.set_boundz();
//     bbList.BB.push_front(m4); 
//     //m4.print_bounds_start();
    
//Push the root node into the BB list    
    bbList.BB.push_front(root); 

//     //FEas
//     Node froot(root);
//     
//         //Set the bounds and startpoints of the ipopt problem tnlp equal
//         //to the bounds and startpoints of the node chosen to be solved
//         tnlp->equate_bounds(froot);
//         tnlp->equate_start(froot);
//         
//         //Setup completed, call ipopt and return status of run
//         ApplicationReturnStatus stat_feas_pump = ipoptApplication->Initialize();
//         stat_feas_pump=ipoptApplication->OptimizeTNLP(GetRawPtr(tnlp));
//         froot.save_stat(tnlp->s);
//         cout<<"STATUS IS: "<<stat_feas_pump<<"   "<<tnlp->s<<endl;
//         
//         //Save the variable- and objective function values of
//         //the NLP into the (root)node member array sol and var objf
//         froot.save_sol(tnlp->solution);
//         froot.save_obj(tnlp->obj);
//         
//         tnlp->stineprint();
//     
//          //---------------------INITIALIZATION FEASIBILITY PUMP---------------------
//   
//       if (tnlp->const_sol != NULL){
//           //Feasibility Pump
//            fp_Ptr->save_sol_nlp(It,tnlp->const_sol);
//            tminlp->save_sol(tnlp->const_sol);
//            tmilp->save_sol(tnlp->const_sol);
//            tmilp->set_tangentValues(tnlp->const_sol,tnlp->FPJac_w, tnlp->FPJac_p);  
//            tminlp->print_solValues(tnlp->const_sol);
//            //tminlp->lock_bounds(tnlp->const_sol);
//            
//            //Problem Heuristic
//            //ph1_ptr->save_nlp(tnlp->const_sol);
//            
//       
//            //Test integer feasibility           
//            if(fp_Ptr->test_int_feasibility(tnlp->const_sol)==true){
//                 cout<<"NLP soltuion is integer feasible, no feasibility pump nessesary"<<endl;
//                 It=MaxIt;
//            }
//            else{
//                 cout<<"NLP soltuion is not integer feasible"<<endl;
//            }
//       }
//  	  else{
//             cout<<"Got NULL for solution"<<endl; 
//       }
//         
//        
//         
//  //--------------------------FEASIBILITY PUMP-------------------------------    
// 

//   
//   //It=MaxIt;
//   while (It<MaxIt){
   
// //--------------------------------MILP-------------------------------------
//     
//       //Initialize 
//       tmilp->set_milp();
//       tminlp->set_milp();
//       bonmin.initialize(GetRawPtr(tmilp)); 
//      
//       
//       try {
//   	  Bab bb;
//   	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
//       milp_sol = bb.bestSolution();
//      
//       if (milp_sol != NULL){
//             
//              fp_Ptr->save_sol_milp(milp_sol);
//              tminlp->save_sol(milp_sol);
//              tmilp->save_sol(milp_sol);
//              tmilp->print_solValues(milp_sol);
//              
//             //Test NLP feasibility
//             if(fp_Ptr->test_feasibility()==true){
//             cout<<"MILP soltuion is NLP feasible, stop!"<<endl;
//             It=MaxIt;
//             }
//             else{
//             cout<<"MILP soltuion is not NLP feasible"<<endl;
//             }    
//         }
//  	  	else{
//              cout<<"Got NULL for solution"<<endl;
//              //Hvis dette skjer da, hva skal gjres med startpunkt og parametere til nlp?
//              //Lage en funksjon som generer nye startpunkt
//          }
//    
//       }
//     
// 	  catch(TNLPSolver::UnsolvedError *E) {
// 	    //There has been a failure to solve a problem with Ipopt.
// 	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
// 	  }
// 	  catch(OsiTMINLPInterface::SimpleError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
// 	  catch(CoinError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
//       
//     //Test integer solution
//     tminlp->set_It(0);
//     tminlp->set_nlp();
//     tminlp->set_boundsmilp(fp_Ptr->sol_milp);
//     bonmin.initialize(GetRawPtr(tminlp)); 
//     
//     cout<<"Test integer feasibility"<<endl;
//     try {
//   	  Bab bb;
//   	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
//       
//       fp_nlp_sol = bb.bestSolution();
//       if (fp_nlp_sol != NULL){
//            fp_Ptr->save_sol_nlp(It,fp_nlp_sol);
//            fp_Ptr->save_sol_milp(fp_nlp_sol);
//            //tminlp->save_sol(fp_nlp_sol);
//            tminlp->print_solValues(fp_nlp_sol);
//                            
//           if(fp_Ptr->test_int_feasibility(fp_nlp_sol)==true){
//           if(fp_Ptr->test_feasibility()==true){
//           cout<<"NLP soltuion is integer feasible! "<<endl;
//           if(tminlp->tot_prod >= Best_Sol){
//               Best_Sol=tminlp->tot_prod;
//               //cout<<"Best_Sol: "<<Best_Sol<<endl;
//               fp_Ptr->save_best_minlp();
//           }
//           //It=MaxIt;
//                 }
//           else{
//             cout<<"NLP soltuion is not NLP feasible"<<endl;   
//           }
//            }
//           else{
//           cout<<"NLP soltuion is not integer feasible"<<endl;
//           }
//         }
//  	  	else{
//              cout<<"Got NULL for solution"<<endl;    
//          }
//     }
//     
//     
// 	  catch(TNLPSolver::UnsolvedError *E) {
// 	    //There has been a failure to solve a problem with Ipopt.
// 	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
// 	  }
// 	  catch(OsiTMINLPInterface::SimpleError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
// 	  catch(CoinError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
//     
//      
// //---------------------------------NLP-------------------------------------
//      
//      
//       //Initialize model
//       It++;
//       tmilp->set_It(It);
//       tminlp->set_It(It);
//       tmilp->set_nlp();
//       tminlp->set_nlp();
//       tminlp->restore_bounds();
//       bonmin.initialize(GetRawPtr(tminlp));
//       
//       
//       //It=MaxIt;
//       if(It<MaxIt){
//       
//       try {
//   	  Bab bb;
//   	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
//       fp_nlp_sol = bb.bestSolution();//NLP solution
// 
//       if (fp_nlp_sol != NULL){
//            //Save NLP solution
//            fp_Ptr->save_sol_nlp(It,fp_nlp_sol);
//            tminlp->save_sol(fp_nlp_sol);
//            tmilp->save_sol(fp_nlp_sol);
//            tmilp->set_tangentValues(fp_nlp_sol,tminlp->FPJac_w, tminlp->FPJac_p);  
//            tminlp->print_solValues(fp_nlp_sol);
//            //tminlp->lock_bounds(fp_nlp_sol);
//            
//            //Test integer feasibility
//            if(fp_Ptr->test_int_feasibility(fp_nlp_sol)==true){
//            It=MaxIt;
//            tminlp->intSol(fp_nlp_sol);
//            cout<<"NLP soltuion is integer feasible, stop!"<<endl;
//            }
//            else{
//             cout<<"NLP soltuion is not integer feasible"<<endl;
//            }
//         }
//  	  	else{
//              cout<<"Got NULL for solution"<<endl;
//          }
//       }
// 
// 	  catch(TNLPSolver::UnsolvedError *E) {
// 	    //There has been a failure to solve a problem with Ipopt.
// 	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
// 	  }
// 	  catch(OsiTMINLPInterface::SimpleError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
// 	  catch(CoinError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  } 
//   }
//       //cout<<"Iteration number: "<<It<<endl;
// }
//     
// 
// 
// 
// 
//     
// //----------------------SOLUTION FEASIBILITY PUMP--------------------------
//     tminlp->set_It(0);
//     //for(int i=0; i<50; i++){
//     //cout<<"fp_Ptr->minlp["<<i<<"]: "<<fp_Ptr->minlp[i]<<endl;}
//     tminlp->set_boundsmilp(fp_Ptr->minlp);
//     bonmin.initialize(GetRawPtr(tminlp)); 
//     
//    
//     try {
//   	  Bab bb;
//   	  bb(bonmin);//process parameter file using Ipopt and do branch and bound using Cbc
//       
//       fp_nlp_sol = bb.bestSolution();
//       if (fp_nlp_sol != NULL){
//           //Feasibility Pump
//            fp_Ptr->save_sol_nlp(It,fp_nlp_sol);
//            tminlp->save_sol(fp_nlp_sol);
//            tminlp->print_solValues(fp_nlp_sol);
//         }
//  	  	else{
//              cout<<"Got NULL for solution"<<endl;
//          }
//         }
//     
//     
// 	  catch(TNLPSolver::UnsolvedError *E) {
// 	    //There has been a failure to solve a problem with Ipopt.
// 	    std::cerr<<"Ipopt has failed to solve a problem"<<std::endl;
// 	  }
// 	  catch(OsiTMINLPInterface::SimpleError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
// 	  catch(CoinError &E) {
// 	    std::cerr<<E.className()<<"::"<<E.methodName()
// 		     <<std::endl
// 		     <<E.message()<<std::endl;
// 	  }
//     
//  
//     
//  Node* feas_pump_sol;
//  feas_pump_sol->save_sol(fp_nlp_sol);
//  feas_pump_sol->save_obj(-(tminlp->tot_prod));
//  
//  bbList.check_LB(feas_pump_sol->get_obj(), feas_pump_sol, 0);
//     

// ---------------------INITIALIZATION PROBLEM HEURISTIC-------------------

    clock_t St;
    clock_t Et;
    clock_t Tickst;
    double time_fs, time_h1, time_h2, time_search;
    clock_t bbEnd;
    clock_t bbtick;
    double bbmili;

    list<Node>::iterator head = bbList.BB.begin();
   
    //Get Lower Bound from Feasibility Pump
    St=clock();    
    Node fplb(*head);
    fplb.feas_pump_start_bound();
    tnlp->equate_bounds(fplb);
    tnlp->equate_start(fplb);
    //fplb.print_bounds_start();
    //Setup completed, call ipopt and return status of run
    ApplicationReturnStatus so_test = ipoptApplication->Initialize();
    so_test=ipoptApplication->OptimizeTNLP(GetRawPtr(tnlp));
    fplb.save_stat(tnlp->s);
    fplb.save_sol(tnlp->solution);
    fplb.save_obj(tnlp->obj);
    fplb.print_node();
    fplb.is_feasible();
    int teit = fplb.is_int_feasible(frac_lim);        
        if ( (so_test == 0) && (teit==1)){
            int b=bbList.check_LB(fplb.get_obj(), fplb, 3);
        }
    
    delete fp_Ptr;
    fp_Ptr = 0;    
    
    Et = clock();    Tickst=Et-St;
    time_fs=Tickst/(double)CLOCKS_PER_SEC;
  
//bbList.print_BBlist();
    
    int print_count = 1;
    int feas_count=180;
    int heur2 =0;
    int heur1 =1;
    rounds = 0;
    int iters = 0;
    int h2SN = 0;
    double funktime1 = getthetime();
    double time_count = 0.001;
    
    Start_t=time(NULL);
    clock_t bbStart=clock();
    
    while (bbList.BB.size() != 0){  //   while (bbList.BB.size() != 0) for (int i=0; i<10; i++)
    rounds++; 
        //Update time count
        bbEnd=clock();
        bbtick=bbEnd-bbStart;
        bbmili = bbtick/(double)CLOCKS_PER_SEC;
    
        //Print RES fiile
        if (rounds == print_count*50){
            bbList.res_file(bbList.BB.size(), print_count, iters, bbmili);
            print_count++;
        } 
    
        //Print TIME file
        //cout<<"--"<<3600*time_count<<"   "<<bbmili<<endl;
        if ( (bbmili > 3600*time_count) && (bbmili < (3600*time_count)+5)){
            bbList.time_file(bbList.bestNode, time_msr, time_mss, funktime1, bbmili, time_fs, -1, -1, rounds, iters, heur1, h2SN);
            time_count=time_count+1;
        }
            
    
    
    if ( ( (head->get_parent_obj()) > (A2C*bbList.get_best_LB()) )    && ( (head->get_parent_obj()) <  (3*bbList.get_best_LB()) ) ){
        iters++;
        
        //Decrease level limit to limit size of tree
//         if (Node::total_nodes == 3000){
//             lev_lim=2;
//         }       
        //Decrease fractional limit
//         if (Node::total_nodes==2000){
//             frac_lim=15;
//         }
//         else if (Node::total_nodes==4000){
//             frac_lim=10;
//         }
//         else if (Node::total_nodes==6000){
//             frac_lim=5;
//         }
              
        printf("\nPROBLEM NR: %3d:%3d   LEVEL NR: %2d  LISTSIZE: %3d  PARENT: %10.2f ROUND: %6d ", head->get_probnum(), Node::total_nodes, head->get_level(), bbList.BB.size(), head->get_parent_obj(), rounds);
        
        //Set the bounds and startpoints of the ipopt problem tnlp equal
        //to the bounds and startpoints of the node chosen to be solved
        tnlp->equate_bounds(*head);
        tnlp->equate_start(*head);
        
        //Setup completed, call ipopt and return status of run
        ApplicationReturnStatus status = ipoptApplication->Initialize();
        status=ipoptApplication->OptimizeTNLP(GetRawPtr(tnlp));
        head->save_stat(tnlp->s);
        //cout<<"STATUS IS: "<<status<<"   "<<tnlp->s<<"____";
        
        
        if (head->get_stat() < 2){
            //Increase the level of the node if s==1 -> IpOpt has reached max_iter
            if (head->get_stat() == 1){
                bbList.increase_ipopt_iter();
                head->inc_level();
            }
            
            //Save the variable- and objective function values of
            //the NLP into the (root)node member array sol and var objf
            head->save_sol(tnlp->solution);
            head->save_obj(tnlp->obj);
            //head->print_bounds_start();
         
//             if ( (Node::total_nodes>17999)&&(Node::total_nodes==feas_count*100) ){
//                 //head->is_feasible();
//                 //head->print_node();
//                 head->printing_(bbList.get_best_LB());
//                 feas_count++;
//             }
            
            //Here the solution is tested to see if it is integer feasible.
            int_sol = head->is_int_feasible(frac_lim);  
            
            St=clock();
            
            if (rounds==heur1*10){  
                cout<<"     round="<<rounds<<"    "<<"heur1="<<heur1<<"   ";
                if( (int_sol != 1) && (head->get_stat() == 0) ){
                heur1++;
                //cout<<"PROBLEM HEURISTIC 1    ";
                PH1_Ptr ph1_ptr = new ProblemHeuristic1;
                
                ph1_ptr->set_CapacityS();
                ph1_ptr->priorityWells();
                
                //Problem Heuristic 1
                    ph1_ptr->save_nlp(tnlp->solution);
                    ph1_ptr->evaluate_nlp();
                    ph1_ptr->process_nlp();
                    ph1_ptr->print_solValues(); 
                    
                    Node int_test(*head);
                                        
                    int_test.set_heur_bin(ph1_ptr->roundedSol, int_test);
                    
                    tnlp->equate_bounds(int_test);
                    tnlp->equate_start(int_test);
                    
                    cout<<" Heur1 ";
                    //Setup completed, call ipopt and return status of run
                    ApplicationReturnStatus s_test = ipoptApplication->Initialize();
                    s_test=ipoptApplication->OptimizeTNLP(GetRawPtr(tnlp));
                    int_test.save_stat(tnlp->s);
                    //cout<<"HEUR STATUS IS: "<<s_test<<"   "<<int_test.get_stat()<<tnlp->s<<endl;
                    
                    int_test.save_sol(tnlp->solution);
                    int_test.save_obj(tnlp->obj);
                    //Here the solution is tested to see if it is integer feasible.
                    int t = int_test.is_int_feasible(frac_lim);  
                            
                    if ( (s_test == 0) && (t==1)){
                        //Update the Lower Bound
                        //int_test.print_bounds_start();
                        int b=bbList.check_LB(int_test.get_obj(), int_test, 1);
                    }
                    
                    delete ph1_ptr;
                    ph1_ptr=0;
                }
                else{
                heur1++;
                }
            }
            
            Et = clock();    Tickst=Et-St;
            time_h1=Tickst/(double)CLOCKS_PER_SEC; //cout<<"H1TIME: "<<time_h1<<endl;
            
            St=clock();      
            
//             if (rounds==(heur2*10)+5){
//                 cout<<"     round="<<rounds<<"    "<<"heur2="<<heur2<<"   ";
//                 if( (int_sol != 1) && (head->get_stat() == 0) ){
//                 heur2++;
//                 //cout<<"PROBLEM HEURISTIC 2    ";
//                 PH2_Ptr ph2_ptr = new ProblemHeuristic2;
//             
//                 ph2_ptr->set_CapacityS();
//                 ph2_ptr->priorityWells();
//                 
//                 int run=0;  
//                 int tt=0;
//                 ph2_ptr->save_nlp(tnlp->solution);
//                 
//                 ph2_ptr->evaluate_nlp();
//                 ph2_ptr->process_nlp();
//                 ph2_ptr->print_solValues();
//                 Node int_test2(*head);
//                 
//                 ApplicationReturnStatus s2_test;
//                 while(run==0){
//       
//                     if(tt==50){
//                     run=1000;   
//                     }
//                     
//                     h2SN++;
//                     int_test2.lock_BoundPH2(ph2_ptr->locked, ph2_ptr->roundedSol);
//                     tt=int_test2.count;
//                     //cout<<"run: "<<tt<<endl;
//                     
//                     tnlp->equate_bounds(int_test2);
//                     tnlp->equate_start(int_test2);
//                     cout<<"   snlpcount="<<h2SN<<" Heur2 ";
//                     //Setup completed, call ipopt and return status of run
//                     s2_test = ipoptApplication->Initialize();
//                     s2_test=ipoptApplication->OptimizeTNLP(GetRawPtr(tnlp));
//                     int_test2.save_stat(tnlp->s);
//                     //cout<<"HEUR STATUS IS: "<<s_test<<"   "<<int_test.get_stat()<<tnlp->s<<endl;
//                     
//                     
//                     
//                     int_test2.save_sol(tnlp->solution);
//                     int_test2.save_obj(tnlp->obj);
//                     
//                     ph2_ptr->save_nlp(tnlp->solution);
//                     
//                          if(tt<50){   
//          
//                             ph2_ptr->evaluate_nlp();
//                             ph2_ptr->process_nlp();
//                             ph2_ptr->print_solValues();
//          
//                               
//                             }
//                 }
//                     int t = int_test2.is_int_feasible(frac_lim);  
//                             
//                     if ( (s2_test == 0) && (t==1)){
//                         //Update the Lower Bound
//                         //int_test.print_bounds_start();
//                         int b=bbList.check_LB(int_test2.get_obj(), int_test2, 2);
//                         int_test2.is_feasible();
//                         int_test2.print_node();
//                     }
//                  delete ph2_ptr;
//                  ph2_ptr=0;
//      
//                 }
//                 else{
//                     heur2++;
//                 }
//             }
            
            Et = clock();    Tickst=Et-St;
            time_h2=Tickst/(double)CLOCKS_PER_SEC; //cout<<"H2TIME: "<<time_h2<<endl;
            
            //If the solution is integer feasible AND s==0, 
            //LB is checked and updated, then the node is removed
            if ( int_sol == 1 ){
                //cout<<"    CONGRATS!!      ";
                
                int a=0;  
                if (head->get_stat() == 0){
                    
                    //Update the Lower Bound
                    a=bbList.check_LB(head->get_obj(), *head, 0);
                }
                //PS: Something must be done with the int_feasible solutions that have s==1
                //because they might not be NLP feasible! Now they just get erased.
//                 else if (head->is_feasible()){
//                     cout<<"OMG"<<endl;
//                         a=bbList.check_LB(head->get_obj(), *head, 4);
//                 }
                
 
                //Remove the top node
                head = bbList.BB.erase(head); //bbList.BB.begin()
            }
            //NOT int.feasible
            else{
                //First Alternative: OFV is better than best_LB
                if ( (head->get_obj() > bbList.get_best_LB())){ // && (head->get_frac()< frac_lim) )
                    cout<<"---ALT 1---    "<<head->get_obj()<<" > "<<bbList.get_best_LB()<<" Level:"<<head->get_level();
                    Stack::increase_alt1();
                    
                    if (head->get_stat() == 0){
                        head->clear_level();
                    }
                    
                    //1. Choose which variabel to branch on
                    int var_branch = head->var_choice();
                    //cout<<"   Variable to branch on: x["<<var_branch<<"]"<<endl;
                    
                    //2. Make the two new nodes by using the overloaded assignment operator
                    //cout<<"Making first kid";
                    Node kid1;
                    kid1 = *head;
                    
                    //cout<<"Making second kid";
                    Node kid2;
                    kid2 = *head;
                    
                    //3. Fix the bounds of variabel to branch on in the two new problems
                    head->next_generation(kid1, kid2, var_branch);
                    //kid1.print_node();
                    //kid2.print_node();
                    
                    //4. Remove the node
                    head = bbList.BB.erase(head); //bbList.BB.begin()
                    
                    //5. Put the next generation nodes into the BB list theList
                    //cout<<"Pushing kids"<<endl;
                    bbList.BB.push_front(kid1);
                    bbList.BB.push_front(kid2);
                }
                //Second Alternative: Node is good enough to make the next generation
                else if ( (head->get_obj() > (A2C*bbList.get_best_LB())) && (head->get_level() < lev_lim) ){
                        
                    cout<<"---ALT 2---     "<<head->get_obj()<<" > "<<A2C*bbList.get_best_LB()<<" Level:"<<head->get_level();
                    Stack::increase_alt2();
                    
                    //1. Choose which variabel to branch on
                    int var_branch = head->var_choice();
                    //cout<<"   Variable to branch on: x["<<var_branch<<"]"<<endl;
                      
                    //2. Make the two new nodes by using the overloaded assignment operator
                    //Also increase their level by one
                    //cout<<"Making first kid";
                    Node kid1;
                    kid1 = *head;
                    kid1.inc_level();
                    
                    //cout<<"Making second kid";
                    Node kid2;
                    kid2 = *head;
                    kid2.inc_level();
                    
                    //3. Fix the bounds of variabel to branch on in the two new problems
                    head->next_generation(kid1, kid2, var_branch);
                    //kid1.print_node();
                    //kid2.print_node();
                    
                    //4. Remove the top node
                    head = bbList.BB.erase(head); //bbList.BB.begin()
                    
                    //5. Put the next generation nodes into the BB list theList
                    //This will use the copy constructor
                    //cout<<"Pushing kids"<<endl;
                    bbList.BB.push_front(kid1);
                    bbList.BB.push_front(kid2);
                }
                else{
                    cout<<"------ALT 3------DELETING NODE"<<endl;
                    Stack::increase_alt3();
                    //Remove the node
                    head = bbList.BB.erase(head); //bbList.BB.begin()
                }
              } 
        }        
        else{
            //cout<<"Status Failed"<<endl;
            Stack::increase_stat_fail();
            
            head = bbList.BB.erase(head); //bbList.BB.begin()
            
            heur1=ceil(rounds/10)+1;
            cout<<"heur11="<<heur1<<endl;
            
            heur2=ceil(rounds/10)+1;
            cout<<"heur22="<<heur2<<endl;
        }
    }
    else{
        Stack::increase_alt3();
        head = bbList.BB.erase(head);
        
        heur1=ceil(rounds/10)+1;
        cout<<"heur1="<<heur1<<endl;
        
        heur2=ceil(rounds/10)+1;
        cout<<"heur2="<<heur2<<endl;
    }
        
        

      St=clock(); 
    
        //bbList.print_BBlist();
//      if ( bbList.get_best_LB() == 0){
//         cout<<"  DFCODE";
//         //---------Depth First Code--------------
//         head = bbList.BB.begin();
//      }
//      else{
        cout<<"  BFCODE";
        
        //---------Best First Code--------------
        double temp=0;
        list<Node>::iterator ii;
        for(ii = bbList.BB.begin(); ii!= bbList.BB.end(); ii++){
            if ( (ii->get_parent_obj()) > temp){
                temp = ii->get_parent_obj();
                head = ii;
                bbList.set_upper_bound(temp);
                
                //cout<<"Found better node; "<<head->get_parent_obj()<<"   "<<head->get_probnum()<<endl;
            }
            
//             if ( (ii->get_parent_obj()) < bbList.get_best_LB()){
//                 ii = bbList.BB.erase(ii);
//             }
        } 
        
        
    // }  //END BFCODE
    
     Et = clock();    Tickst=Et-St;
     time_search=Tickst/(double)CLOCKS_PER_SEC; //cout<<"Search Time: "<<time_search<<endl;
        
    }//END WHILE
    
    bbEnd=clock();
    bbtick=bbEnd-bbStart;
    bbmili = bbtick/(double)CLOCKS_PER_SEC;
    End_t = time(NULL);
    time_bbt=difftime(End_t, Start_t);
    //cout<<"BB time: "<<endl;
    double funktime2 = getthetime();
    bbList.time_file(bbList.bestNode, time_msr, time_mss, funktime1, bbmili, time_fs, time_h1, funktime2, rounds, iters, heur1, h2SN);
    
    //cout<<"Best found Solution"<<endl;
//     int ttt=bbList.bestNode.is_feasible();
//     bbList.bestNode.print_node();
    
    //cout<<"Number of nodes that went to the roof: "<<bbList.get_imi()<<endl;
    
        
    
    

    
//         SmartPtr<MyTNLP> myprob = new MyTNLP;
//         SmartPtr<IpoptApplication> ipoptApp =IpoptApplicationFactory();
//     
//         //Initialisering og innlesing av data
//         myprob->set_itt();
//         myprob->readWells();
//         myprob->readPipes(); 
//     
//         //Lager listen
//         list<Node> newlist;
//     
//         //Putter noden root som tidligere er laget inn i listen
//         newlist.push_front(root); 
//         
//         //Lager en iterator(peker) som peker p frste Node i listen
//         list<Node>::iterator pit = newlist.begin();
//         
//         //Teller runder
//         int i=0;
//         
//         //While lkke som kjrer s lenge det er noder i listen
//     while (newlist.size() !=0 ){
//         i++;
//         
//         if (i<10000){ //Bare for  se at minnebruken IKKE ker nr lister kun blir
//                     //lagt inn i listen
//         cout<<"Iteration "<<i<<"   size of list: "<<newlist.size()<<endl;
//         
//         //Setter grenser og startpunkt til myprob(MyTNLP)
//         //Disse blir lest inn fra noden pit peker p
//         myprob->equate_bounds(*pit);
//         myprob->equate_start(*pit);
//         
//         //Lager to kopier av pit med overloaded assignment operator
//         Node nn = *pit;
//         Node mm = *pit;
//         
//         //Putter de to nye nodene i listen, og sletter noden pit peker p
//         newlist.push_front(nn);
//         newlist.push_front(mm);
//         newlist.erase(pit);
//         
//         //Pit blir satt til  peke p frste element i listen
//         pit = newlist.begin();
//         }
//         else{//Her kjres Ipopt p problemet kopiert inn fra node
//              //Her begynner minnebruken  ke jevnt og trutt
//         cout<<"Iteration "<<i<<"   size of list: "<<newlist.size()<<endl;
//         
//         myprob->equate_bounds(*pit);
//         myprob->equate_start(*pit);
//         
//         //Setup completed, call ipopt and return status of run
//         ApplicationReturnStatus status = ipoptApp->Initialize();
//         status=ipoptApp->OptimizeTNLP(GetRawPtr(myprob)); //
// 
//         //Funksjoner for  lagre lsningen fra Ipopt
//         pit->save_stat(myprob->s);
//         pit->save_sol(myprob->solution);
//         pit->save_obj(myprob->obj);
//         
//         //Sletter noden pit peker p
//         newlist.erase(pit);
//         //Pit blir satt til  peke p frste element i listen
//         pit = newlist.begin();
//         }
//             
//     }
        
      
    return 0;
}