// (C) Copyright Carnegie Mellon University 2006
// All Rights Reserved.
// This code is published under the Common Public License.
//
// Authors :
// P. Bonami, Carnegie Mellon University
//
// Date :  03/17/2006
#ifndef MyTNLP_HPP
#define MyTNLP_HPP
//#include "BonTMINLP.hpp"
#include "IpTNLP.hpp"
using namespace Ipopt;
//using namespace Bonmin;

using namespace std;
#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <math.h>
#include <ctime>


    //Constants related to size of problem
    #define M 2;
    //static const int M = 2;    //Subsea manifolds
    static const int I = 6;    //Platform wells
    static const int J = 10;   //Subsea wells
    static const int J1= 6;    //Subsea wells Manifold 1
    static const int J2= 4;    //Subsea wells Manifold 2
    static const int L = 4;    //Pipelines
    static const int P = 3;    //Phases
    static const int S = 3;    //Separators   

    static const int NV = 150;  //Total number of variables
    static const int NB = 50;   //Number of binary variables
    static const int MC = 139;  //Number of constraints
    
    static const double SeA = 10;
    static const double SeB = 12;
    static const double SeC = 15;
    
    static const double A2C = 1.0; //Alternative 2 coefficient
    
    //static const int M = 2;    //Subsea manifolds
 
    static const int L1 = 2;
    static const int L2 = 2;
    
    //Clustering parameters
    static const int cluster_nv=82;
    static const double cluster_q=20;
    static const double cluster_p=5;
    static const double cluster_bin=0.2;
    static const double cluster_percentage=0.90;
    static const int num_multi=1000;
    static const int clusters=5;


    
class MyTNLP;

class Node {
    private:
        //Variables related to the node itself
        int probnum;
        double objf;
        int stat;     
        int level;
        int rootnum;
        double boundz[NV][2]; //bytt navn til var_bounds
        double start[NV];
        double sol[NV];
        double fraction;
        
        //Variables related to the parent node
        double parent_objf;
        int parent_stat;
        
        //Static members
        static int groups[NB][2];     
        
    public:
        Node(){} //Default constructor
        Node(Node* n){nodes_iterate(); probnum=total_nodes; level=0; parent_objf=20000; rootnum=0;} //Overloaded constructor
        Node(int i, int j){level=0;} //Overloaded constructor for multiple startpoints
        //Node(double d, const double* s) : objf(d) { save_sol(s);}
        Node(const Node& n); //Copy Constructor
        Node& operator=(const Node& rtSide); //Overloaded assignment operator
        //~Node(); //Destructor
        
        double mmw[I+J][3][2];
        static int total_nodes;
        
        void read_save_start(const string &filen, int r);
        void set_mmw(double minmax_w[I+J][3][2]);
        void set_boundz();
        void set_start();
        void set_multi_start();
        void set_rootnum(int r){rootnum=r;};
        void set_parent_obj(double o){parent_objf=-o;};
        void equate_sol_to_start();
        void inc_level(){level=level+1;};
        void clear_level(){level=0;};
        int get_level(){return level;};
        void save_sol(const double a[]);
        void save_obj(const double o){objf=-o;};
        void save_stat(const int s){stat=s;};
        static void Groups();
        static void save_mmw();
        double get_obj() const{return objf;};
        double get_parent_obj(){return parent_objf;};
        double get_stat(){return stat;};
        double get_frac(){return fraction;};
        
        int is_int_feasible(int);
        int var_choice();
        void next_generation(Node& k1, Node& k2, int var);
        static void nodes_iterate();
        int get_probnum() const {return probnum;};
        void print_node() const;
        void print_start();
        void print_bounds_start();
        void print_binary(int s, int e){for (int i=s; i<e; i++){cout<<sol[i]<<"  ";}cout<<endl;}
        void set_heur_bin(double a[], Node&);
        
        bool is_feasible();
        bool c3();
        bool c4();
        bool c6();
        bool c7();
        bool c8();
        bool c9();
        bool c10();
        bool c11();
        bool cap();
        
        void twells_open() const;
        void swells_open() const;
        void line_to_sep() const;
        
        void printing_(double); //Kan slettes etterhvert
        
        int min_val(int, double a[]);//returnerer index til minste
        int max_val(int, double a[]);//returnerer index til strste  
        double random_double(double low, double high);
        int random_int(double low, double high);
        
        void lock_BoundPH2(double lock[NB],Number milp[NB]);
        int count;
        void feas_pump_start_bound();
        
        friend class MyTNLP;
        friend class Stack;
        //friend Node operator-(Node& n1, Node& n2);
};

typedef Node* NodePtr;


// Node operator-(Node& n1, Node& n2){
//     double num=n1.get_obj()-n2.get_obj();
//     Node result;
//     return result;
// }
// 
// class CompareNodeParentObj{
// public:
//     bool operator()(Node& n1, Node& n2){ 
//         if (n1.get_parent_obj()<n2.get_parent_obj()){
//             return true;}
//         else{
//             return false;}
//     }
// };
// 
// class CompareNodeObj{
// public:
//     bool operator()(Node& n1, Node& n2){ 
//         if (n1.get_obj()<n2.get_obj()){
//             return true;}
//         else{
//             return false;}
//     }
// };
//         

class Stack {
    private:
        double best_LB;
        double upper_bound;
        static int ipopt_max_iter;
        static int alt1;
        static int alt2;
        static int alt3;
        static int stat_fail;
        static int h1;
        static int h2;
        static int bb;
        
    public:
        list<Node> BB;      //The Branch&Bound list of problems to be solved, in depth-first order
        
        list<Node> rand_start;    //Multiple starting points. Each node is ready for solving with set starting point
        double dist[82];          //Multistart distance/difference computation
        bool nodes_in_cluster(const Node& n1, const Node& n2);
        
        //priority_queue<Node, list<Node>, CompareNodeObj> sorted_rand_start;
        
        Node bestNode;      //The best node so-far is saved into this node
        
        int check_LB(double, const Node&, int);
        void init_LB(){best_LB=0;};
        double get_best_LB(){return best_LB;};
        double get_upper_bound(){return upper_bound;};
        void set_upper_bound(double ub){upper_bound=ub;};
        
        static void increase_ipopt_iter();
        static void increase_alt1();
        static void increase_alt2();
        static void increase_alt3();
        static void increase_stat_fail();
        static void increase_h1();
        static void increase_h2();
        static void increase_bb();
        int get_imi(){return ipopt_max_iter;};
        int get_alt1(){return alt1;};
        int get_alt2(){return alt2;};
        int get_alt3(){return alt3;};
        int get_stat_fail(){return stat_fail;};
                
        void res_file(int, int, int, double);
        void time_file(const Node&, int, int, double, double, double, double, double, int, int, int, int);
        
        double random_double(double, double);
        double random_int(int, int);
        void multiple_start(double a[I+J][3][2]);
        int are_equal(const Node& n1, const Node& n2);
                
        //void update_BBlist(SmartPtr<MyTNLP>&);
        
        //void print_BBlist();
        //void print_donelist();
                
        friend class Node;
};

class ProblemHeuristic1
{
private:
//Global constants belonging to the class, related to size of problem

 
 
 double CapacityS[S];
 int PriorityManifold[I+J];
 int PriorityTotal[I+J];
 int PriorityPipe[L];
 int WellSepPriority[I];
 int WellS1Priority[J1];
 int WellS2Priority[J2];
 int PipePriority[L];
 
 double nlp[NB];
 
 double compareSM1[L1][J1];
 double compareSM2[L2][J2];
 double compareS[S][L];
 double compareW[S][I];
 
 double sorted_WellSep[S*I];
 double sorted_WellPipe[J*L];
 double sorted_PipeSep[S*L];
 
 bool binary_evaluate[NB];
 
public:
 
 Number roundedSol[NB];
 
 void set_CapacityS ();
 void priorityWells ();
 void save_nlp(const double*);
 void evaluate_nlp();
 void process_nlp();
 void print_solValues();
 
 
 
 void routing_pipeSeparator();
 void routing_wellSeparator();
 void routing_wellPipe();
 void get_roundedSol();
 
 bool illigalSM1(int l, double j[L1][J1]);
 bool illigalSM2(int l, double j[L2][J2]);
 bool illigalS(int s, double j[S][L]);
 bool illigalW(int s, double j[S][L], double k[S][I]);
 
};

typedef ProblemHeuristic1* PH1_Ptr; 

class ProblemHeuristic2
{
private:
//Global constants belonging to the class, related to size of problem
 
 double CapacityS[S];
 int PriorityManifold[I+J];
 int PriorityTotal[I+J];
 int PriorityPipe[L];
 int WellSepPriority[I];
 int WellS1Priority[J1];
 int WellS2Priority[J2];
 int PipePriority[L];
 
 double nlp[NB];
 
 double compareSM1[L1][J1];
 double compareSM2[L2][J2];
 double compareS[S][L];
 double compareW[S][I];
 
 double sorted_WellSep[S*I];
 double sorted_WellPipe[J*L];
 double sorted_PipeSep[S*L];
 
 bool binary_evaluate[NB];
 
public:
 
 Number roundedSol[NB];
 double locked[NB];
  double sorted_lockedWellSep[S*I];
 double sorted_lockedWellPipe[J*L];
 double sorted_lockedPipeSep[S*L];
 
 void set_CapacityS ();
 void priorityWells ();
 void save_nlp(const double*);
 void evaluate_nlp();
 void process_nlp();
 void print_solValues();
 
 
 
 void routing_pipeSeparator();
 void routing_wellSeparator();
 void routing_wellPipe();
 void get_roundedSol();
 
 bool illigalSM1(int l, double j[L1][J1]);
 bool illigalSM2(int l, double j[L2][J2]);
 bool illigalS(int s, double j[S][L]);
 bool illigalW(int s, double j[S][L], double k[S][I]);
 
};

typedef ProblemHeuristic2* PH2_Ptr; 


class MyTNLP : public TNLP
{
private:
//     //Constants related to size of problem
//     static const int M = 2;    //Subsea manifolds
//     static const int I = 6;    //Platform wells
//     static const int J = 10;   //Subsea wells
//     static const int J1= 6;    //Subsea wells Manifold 1
//     static const int J2= 4;    //Subsea wells Manifold 2
//     static const int L = 4;    //Pipelines
//     static const int P = 3;    //Phases
//     static const int S = 3;    //Separators    
    
    //Dimension of the welldata files:
    static const int gdim = 12;  //gas injection break points,
    static const int pdim = 12;  //p^W break points
    static const int ddim = 3;   //columns in the data file

    //Dimension of the pressuredrop files
    static const int dim = 30;
    static const int xdim = 30;
    static const int ydim = 30;
    static const int zdim = 30;
    
    //Separator Pressures, set in function readWells
    double PSEPA;              
    double PSEPB;
    double PSEPC;

    //Platform Well Data matrices
    double P01[gdim][pdim][ddim]; 
    double P02[gdim][pdim][ddim]; 
    double P03[gdim][pdim][ddim]; 
    double P04[gdim][pdim][ddim]; 
    double P05[gdim][pdim][ddim]; 
    double P06[gdim][pdim][ddim]; 

    //Subsea Well Data matrices
    double S01[gdim][pdim][ddim];
    double S02[gdim][pdim][ddim];
    double S03[gdim][pdim][ddim];
    double S04[gdim][pdim][ddim]; 
    double S05[gdim][pdim][ddim]; 
    double S06[gdim][pdim][ddim];
    double S07[gdim][pdim][ddim];
    double S08[gdim][pdim][ddim]; 
    double S09[gdim][pdim][ddim]; 
    double S010[gdim][pdim][ddim];

    //Fill inn WC and GOR information
    double wellspecI[I][2];
    double wellspecJ[J][2];
    void readWspecs();

    //Other
    int itt;                       //Used in simulator and printsome to count how many times the simulator is called
    double minmax_w[I+J][3][2];    //Array containing min and max values for all wells minmax_w[well][qGL/pW/oil][min/max]

public:
  friend class Node;
  double bounds[NV][2];
  double start_pnt[NV];
  void equate_bounds(const Node&);
  void equate_start(Node&);
  void extract_mmw(Node&);
  double solution[NV];
  double obj;
  double s;
  const double* const_sol;
  
  
    double FPJac_w[J+I][2][20];  //[Well][pressure/gas lift]
    double FPJac_p[L][3][20];
    void stineprint();
    
    void save_solPH2(const double* adr);
    
  
    // Default constructor
    MyTNLP(){}

    // Virtual destructor
    virtual ~MyTNLP(){}
  
    /** Copy constructor.*/   
    MyTNLP(const MyTNLP &other){}

    /** Assignment operator. no data = nothing to assign*/
    //MyTNLP& operator=(const MyTNLP&) {}

  
    //Functions to read Well data
    void readWells(); //called from main to read all the well data
    void readWellData(const string &filen, double data[gdim][pdim][ddim]);//called from readWells() -> reads and saves one well
    void minmax_wells(int, double data[gdim][pdim][ddim]);//Saves the min/max values into the array minmax_w
    void set_arrptr();

    //Functions to read Pressure Drop data
    double gas[dim];
    double oil[dim];
    double wat[dim];
    double dP1[dim][dim][dim][4];
    double dP2[dim][dim][dim][4];
    double dP3[dim][dim][dim][4];
    double dP4[dim][dim][dim][4];
    void readDeltaPData(const string &filen, double data[dim][dim][dim][4]); //called form readPipes() -> reads and saves one pipeline
    void readPipes(); //called from main to read all the pipeline data

    //Making an array to hold pointers to the Pipeline Tables
    typedef double (*dPtrArray)[dim][dim][4];
    dPtrArray dParrptr[L];
    void set_dParrPtr();

    typedef double (*PtrArray)[pdim][ddim];
    //Making an array to hold pointers to the Platform Well Tables
    PtrArray parrptr[I];

    //Making an array to hold pointers to the Subsea Well Tables
    PtrArray sarrptr[J];

    //Functions to write output files
    void printsome(int, int, int, double, double, double, double, double val[12]); //for  printe ut og sjekke datainnlesing, kan slettes etterhvert
    void set_itt(){itt=0;};

    //Simulators
    double * ratesim(int, double, double, double (*)[pdim][ddim]);
    double * PressureDrop(int, double, double, double, double (*)[dim][dim][4]);

    //General Functions
    int round(double);
    int min_val(int, double a[]);//returnerer index til minste
    int max_val(int, double a[]);//returnerer index til strste
    int closest_zero(int, double a[]);//returnerer index til positivt tall nrmest null

    
  /** \name Overloaded functions specific to a TMINLP.*/
  //@{
  /** Pass the type of the variables (INTEGER, BINARY, CONTINUOUS) to the optimizer.
     \param n size of var_types (has to be equal to the number of variables in the problem)
  \param var_types types of the variables (has to be filled by function).
  */
//   virtual bool get_variables_types(Index n, VariableType* var_types);
 
  /** Pass info about linear and nonlinear variables.*/
  virtual bool get_variables_linearity(Index n, Ipopt::TNLP::LinearityType* var_types);

  /** Pass the type of the constraints (LINEAR, NON_LINEAR) to the optimizer.
  \param m size of const_types (has to be equal to the number of constraints in the problem)
  \param const_types types of the constraints (has to be filled by function).
  */
  virtual bool get_constraints_linearity(Index m, Ipopt::TNLP::LinearityType* const_types);
//@}  
    
  /** \name Overloaded functions defining a TNLP.
     * This group of function implement the various elements needed to define and solve a TNLP.
     * They are the same as those in a standard Ipopt NLP problem*/
  //@{
  /** Method to pass the main dimensions of the problem to Ipopt.
        \param n number of variables in problem.
        \param m number of constraints.
        \param nnz_jac_g number of nonzeroes in Jacobian of constraints system.
        \param nnz_h_lag number of nonzeroes in Hessian of the Lagrangean.
        \param index_style indicate wether arrays are numbered from 0 (C-style) or
        from 1 (Fortran).
        \return true in case of success.*/
  virtual bool get_nlp_info(Index& n, Index&m, Index& nnz_jac_g,
                            Index& nnz_h_lag, TNLP::IndexStyleEnum& index_style);
  
  /** Method to pass the bounds on variables and constraints to Ipopt. 
       \param n size of x_l and x_u (has to be equal to the number of variables in the problem)
       \param x_l lower bounds on variables (function should fill it).
       \param x_u upper bounds on the variables (function should fill it).
       \param m size of g_l and g_u (has to be equal to the number of constraints in the problem).
       \param g_l lower bounds of the constraints (function should fill it).
       \param g_u upper bounds of the constraints (function should fill it).
  \return true in case of success.*/
  virtual bool get_bounds_info(Index n, Number* x_l, Number* x_u,
                               Index m, Number* g_l, Number* g_u);
  
  /** Method to to pass the starting point for optimization to Ipopt.
    \param init_x do we initialize primals?
    \param x pass starting primal points (function should fill it if init_x is 1).
    \param m size of lambda (has to be equal to the number of constraints in the problem).
    \param init_lambda do we initialize duals of constraints? 
    \param lambda lower bounds of the constraints (function should fill it).
    \return true in case of success.*/
  virtual bool get_starting_point(Index n, bool init_x, Number* x,
                                  bool init_z, Number* z_L, Number* z_U,
                                  Index m, bool init_lambda,
                                  Number* lambda);
  
  /** Method which compute the value of the objective function at point x.
    \param n size of array x (has to be the number of variables in the problem).
    \param x point where to evaluate.
    \param new_x Is this the first time we evaluate functions at this point? 
    (in the present context we don't care).
    \param obj_value value of objective in x (has to be computed by the function).
    \return true in case of success.*/
  virtual bool eval_f(Index n, const Number* x, bool new_x, Number& obj_value);

  /** Method which compute the gradient of the objective at a point x.
    \param n size of array x (has to be the number of variables in the problem).
    \param x point where to evaluate.
    \param new_x Is this the first time we evaluate functions at this point? 
    (in the present context we don't care).
    \param grad_f gradient of objective taken in x (function has to fill it).
    \return true in case of success.*/
  virtual bool eval_grad_f(Index n, const Number* x, bool new_x, Number* grad_f);

  /** Method which compute the value of the functions defining the constraints at a point
    x.
    \param n size of array x (has to be the number of variables in the problem).
    \param x point where to evaluate.
    \param new_x Is this the first time we evaluate functions at this point? 
    (in the present context we don't care).
    \param m size of array g (has to be equal to the number of constraints in the problem)
    \param grad_f values of the constraints (function has to fill it).
    \return true in case of success.*/
  virtual bool eval_g(Index n, const Number* x, bool new_x, Index m, Number* g);

  /** Method to compute the Jacobian of the functions defining the constraints.
    If the parameter values==NULL fill the arrays iCol and jRow which store the position of
    the non-zero element of the Jacobian.
    If the paramenter values!=NULL fill values with the non-zero elements of the Jacobian.
    \param n size of array x (has to be the number of variables in the problem).
    \param x point where to evaluate.
    \param new_x Is this the first time we evaluate functions at this point? 
    (in the present context we don't care).
    \param m size of array g (has to be equal to the number of constraints in the problem)
    \param grad_f values of the constraints (function has to fill it).
    \return true in case of success.*/
  virtual bool eval_jac_g(Index n, const Number* x, bool new_x,
                          Index m, Index nele_jac, Index* iRow, Index *jCol,
                          Number* values);
  
  /** Method to compute the Jacobian of the functions defining the constraints.
    If the parameter values==NULL fill the arrays iCol and jRow which store the position of
    the non-zero element of the Jacobian.
    If the paramenter values!=NULL fill values with the non-zero elements of the Jacobian.
    \param n size of array x (has to be the number of variables in the problem).
    \param x point where to evaluate.
    \param new_x Is this the first time we evaluate functions at this point? 
    (in the present context we don't care).
    \param m size of array g (has to be equal to the number of constraints in the problem)
    \param grad_f values of the constraints (function has to fill it).
    \return true in case of success.*/
  virtual bool eval_h(Index n, const Number* x, bool new_x,
                      Number obj_factor, Index m, const Number* lambda,
                      bool new_lambda, Index nele_hess, Index* iRow,
                      Index* jCol, Number* values);

  
  virtual void finalize_solution(SolverReturn status, Index n,
                               const Number* x, const Number* z_L,
                               const Number* z_U, Index m, const Number* g,
                               const Number* lambda, Number obj_value,
                               const IpoptData* ip_data,
                               IpoptCalculatedQuantities* ip_cq);
  
  
//   /** Method called by Ipopt at the end of optimization.*/  
//   virtual void finalize_solution(TMINLP::SolverReturn status,
//                                  Index n, const Number* x, Number obj_value);
// 
//   //@}
// 
//   virtual const SosInfo * sosConstraints() const{return NULL;}
//   virtual const BranchingInfo* branchingInfo() const{return NULL;}
  
//   
//   void printSolutionAtEndOfAlgorithm(){
//     printSol_ = true;}
//   
//    bool printSol_;
   
   
   
};

#endif
