// (C) Copyright Carnegie Mellon University 2006
// All Rights Reserved.
// This code is published under the Common Public License.
//
// Authors :
// P. Bonami, Carnegie Mellon University
//
// Date :  03/17/2006
#include "MyTNLP.hpp"
#include "BonAmplInterface.hpp"

Node::Node(const Node& n){
    
    probnum=n.probnum;
    rootnum=n.rootnum;
    objf=n.objf;
    parent_objf=n.parent_objf;
    stat=n.stat;
    level=n.level;
        
    for(int i=0; i<NV; i++){
        boundz[i][0]=n.boundz[i][0];
        boundz[i][1]=n.boundz[i][1];
    }
    
    for(int i=0; i<NV; i++){
        start[i]=n.start[i];
    }
    
    for(int i=0; i<NV; i++){
        sol[i]=n.sol[i];
    }
    
    //cout<<"Copy Constructor-----------------------------"<<probnum<<"  "<<n.probnum<<endl;
    
}


Node& Node::operator=(const Node& rtSide){
    if (this == &rtSide){
        return *this;
    }
    else {
       
        nodes_iterate(); 
        probnum=total_nodes;
        
        for(int i=0; i<NV; i++){
            boundz[i][0]=rtSide.boundz[i][0];
            boundz[i][1]=rtSide.boundz[i][1];
        }
    
        for(int i=0; i<NV; i++){
            start[i]=rtSide.sol[i];
        }
    
        parent_objf=rtSide.objf;
        parent_stat=rtSide.stat;
        level=rtSide.level;
        rootnum=rtSide.rootnum;
    
        //cout<<"Overloaded Assignment---------------------------------------------"<<probnum<<endl;
    
        return *this;
    }
}

// Node::~Node(){
//     
//     cout<<"    Destructor is called, Node "<<probnum<<" is deleted     "<<endl;
// }

void Node::set_boundz(){
    
    int s=0;
    int e=0;
    int k=0; //brukes i minmax sammenheng, teller brnnnr
    
    // Binary: 18
    
    // x_is = 18
    e=(S*I);
    for(int i = s ; i < e ; i++){
        boundz[i][0] = 0.0; boundz[i][1] = 1.0;}
    
    // y_mjl = 20
    s=e;
    e=s+(J*2);
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = 1.0;}
    
    // z_mls = 18
    s=e;
    e=s+(L*S);
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = 1.0;}

    // Continuous: 95
    
    // q^WI_pi = 18
    s=e;
    e=s+(P*I);
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = DBL_MAX;}
    
    // q^WJ_mpj = 30
    s=e;
    e=s+(P*J);
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = DBL_MAX;}
    
    // q^GLI_i = 6 and q^GLJ_j = 10
    s=e;
    e=s+I+J;
    for(int i = s ; i < e ; i++){
		boundz[i][0] = mmw[k][0][0]; boundz[i][1] = mmw[k][0][1];
        k++;
        //boundz[i][0] = 40; boundz[i][1]=200;
    }
    
    // q^L_mpl = 12
    s=e;
    e=s+(L*P);
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = DBL_MAX;}
    
    // p^WI_i = 6 and p^WJ_j = 10
    s=e;
    e=s+I+J;
    k=0;
    for(int i = s ; i < e ; i++){
		boundz[i][0] = mmw[k][1][0]; boundz[i][1] = mmw[k][1][1];
        k++;
//         if(i<6){
//         boundz[i][0] = 20; boundz[i][1] = 50;
//         }else{
//         boundz[i][0] = 50; boundz[i][1] = 105;  
//         }
    
    }
    
    // p^M_ml = 4
    s=e;
    e=s+L;
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = DBL_MAX;}   
    
    // p^P_ml = 4
    s=e;
    e=s+L;
    for(int i = s ; i < e ; i++){
		boundz[i][0] = 0.0; boundz[i][1] = DBL_MAX;}     
    
//     for(int i=0; i<NV; i++){
//         cout<<"----------Var x["<<i<<"]:   ["<<boundz[i][0]<<"  "<<boundz[i][1]<<"]"<<endl;
//     }
//     
//     for(int i=0; i<(I+J); i++){
//         for(int j=0; j<3; j++){
//             cout<<"ZZZZZZZmmw x["<<i<<"]:   ["<<mmw[i][j][0]<<"  "<<mmw[i][j][1]<<"]"<<endl;
//         }
//     }
//     
    
}


void Node::set_start(){
    
//     for(int i=0; i<NV; i++){
//         start[i]=0;
//     }


 //	x_is	=	18														
    start[0]	=	0;	start[1]	=	0.6;	start[2]	=	0;									
    start[3]	=	0;	start[4]	=	0;	start[5]	=	0;									
    start[6]	=	0;	start[7]	=	0;	start[8]	=	0;									
    start[9]	=	0;	start[10]	=	0;	start[11]	=	0;									
    start[12]	=	0;	start[13]	=	0;	start[14]	=	0;									
    start[15]	=	0;	start[16]	=	0;	start[17]	=	0;									
																	
    //	y_mjl	=	20														
    start[18]	=	0;	start[19]	=	0;												
    start[20]	=	0.4;	start[21]	=	0;												
    start[22]	=	0.4;	start[23]	=	0;												
    start[24]	=	0;	start[25]	=	0.5;												
    start[26]	=	0.5;	start[27]	=	0;												
    start[28]	=	0.5;	start[29]	=	0;												
    start[30]	=	0;	start[31]	=	0;												
    start[32]	=	0;	start[33]	=	0;												
    start[34]	=	0.5;	start[35]	=	0;												
    start[36]	=	0;	start[37]	=	0;												
																	
//	z_mls	=	18														
    start[38]	=	0;	start[39]	=	1;	start[40]	=	0;									
    start[41]	=	1;	start[42]	=	0;	start[43]	=	0;									
    start[44]	=	0;	start[45]	=	1;	start[46]	=	0;									
    start[47]	=	0;	start[48]	=	0;	start[49]	=	1;									

// //	q^WI_pi	=	18		
//     for(int i=50; i<98; i++){
//         start[i]=0;
//     }  
//     
// //	q^GLI_i	=	6														
//     start[98]	=	180;	start[99]	=	234.163;	start[100]	=	0;	start[101]	=	0;	start[102]	=	5.78754;	start[103]	=	0;																
// 																	
// //	q^GLJ_j	=	10														
//     start[104]	=	20;	start[105]	=	0;	start[106]	=	40;	start[107]	=	90;	start[108]	=	60;	start[109]	=	70;
//     start[110]	=	145.36;	start[111]	=	0;	start[112]	=	30;	start[113]	=	0;						
// 
// //	q^L_mpl	=	12
//     for(int i=114; i<126; i++){
//         start[i]=0;
//     } 
//     
// //	p^WI_i	=	6														
//     start[126]	=	40.0009;	start[127]	=	15;	start[128]	=	55;	start[129]	=	55;	start[130]	=	33.5531;	start[131]	=	50;
// 																
// //	p^WJ_j	=	10														
//     start[132]	=	61.9706;	start[133]	=	139.521;	start[134]	=	78.8106;	start[135]	=	61.9706;	start[136]	=	78.8106;	start[137]	=	78.8106;
//     start[138]	=	62.1627;	start[139]	=	121;	start[140]	=	54.2251;	start[141]	=	120.669;	
// 
// //	p^M_ml+p^P_ml	=	8	
//     for(int i=142; i<150; i++){
//         start[i]=0;
//     } 
    
  
//	q^WI_pi	=	18														
    start[50]	=	206.8;      start[51]	=	335.639;	start[52]	=	0;	start[53]	=	0;	start[54]	=	26.395;	start[55]	=	0;
    start[56]	=	76.3993;	start[57]	=	1289.4;     start[58]	=	0;	start[59]	=	0;	start[60]	=	216.465;	start[61]	=	0;
    start[62]	=	120;        start[63]	=	1120.25;	start[64]	=	0;	start[65]	=	0;	start[66]	=	340;	start[67]	=	0;
																	
//	q^WJ_mpj	=	30														
    start[68]	=	84.9722;	start[69]	=	0.200384;	start[70]	=	57.0498;	start[71]	=	159.535;	start[72]	=	142.48;	start[73]	=	141.107;
    start[74]	=	745.094;	start[75]	=	2.35746;	start[76]	=	221.426;	start[77]	=	869.181;	start[78]	=	863.66;	start[79]	=	815.447;
    start[80]	=	188.959;	start[81]	=	8.86853;	start[82]	=	133.936;	start[83]	=	190.796;	start[84]	=	330.395;	start[85]	=	206.801;
    start[86]	=	240;        start[87]	=	0;          start[88]	=	182.538;	start[89]	=	0.0305391;						
    start[90]	=	1168.4;     start[91]	=	0;          start[92]	=	2157.55;	start[93]	=	0.551248;						
    start[94]	=	560;        start[95]	=	0;          start[96]	=	198.625;	start[97]	=	1.37485;						
																	
																	
//	q^GLI_i	=	6														
    start[98]	=	200;	start[99]	=	234.163;	start[100]	=	0;	start[101]	=	0;	start[102]	=	5.78754;	start[103]	=	0;																
																	
//	q^GLJ_j	=	10														
    start[104]	=	20;	start[105]	=	0;	start[106]	=	40;	start[107]	=	90;	start[108]	=	60;	start[109]	=	70;
    start[110]	=	145.36;	start[111]	=	0;	start[112]	=	30;	start[113]	=	0;						
																	
																	
//	q^L_mpl	=	12														
    start[114]	=	340.837;	start[115]	=	244.507;												
    start[116]	=	1902.89;	start[117]	=	1614.28;												
    start[118]	=	680;        start[119]	=	379.755;												
    start[120]	=	182.569;	start[121]	=	240;												
    start[122]	=	2158.1;     start[123]	=	1168.4;												
    start[124]	=	200;        start[125]	=	560;												
																	
//	p^WI_i	=	6														
    start[126]	=	40.0009;	start[127]	=	15;	start[128]	=	55;	start[129]	=	55;	start[130]	=	33.5531;	start[131]	=	50;
																
//	p^WJ_j	=	10														
    start[132]	=	61.9706;	start[133]	=	139.521;	start[134]	=	78.8106;	start[135]	=	61.9706;	start[136]	=	78.8106;	start[137]	=	78.8106;
    start[138]	=	62.1627;	start[139]	=	121;	start[140]	=	54.2251;	start[141]	=	120.669;						
																																	
//	p^M_ml	=	4														
    start[142]	=	78.8106;	start[143]	=	61.9706;	start[144]	=	54.2251;	start[145]	=	61.9709;						
																	
//	p^P_ml	=	4														
    start[146]	=	16;	start[147]	=	15;	start[148]	=	16;	start[149]	=	17;						
	
}

void Node::set_multi_start(){
    for (int i=0; i<50; i++){
        start[i]=0.5;
    }
    for (int i=50; i<98; i++){
        start[i]=0;
    }
    for (int i=114; i<125; i++){
        start[i]=0;
    }
    for (int i=142; i<150; i++){
        start[i]=0;
    }
}
    

void Node::equate_sol_to_start(){
    for(int i=0; i<NV; i++){
        start[i]=sol[i];
    }
}

void Node::print_start(){
    
    for (int i=0; i<18; i++){
        cout<<start[i]<<"   ";
    }
    cout<<endl;
    for (int i=18; i<38; i++){
        cout<<start[i]<<"   ";
    }
    cout<<endl;
    for(int i=38; i<50; i++){
        cout<<start[i]<<"  ";
    }
    cout<<endl;
//     for(int i=50; i<68; i++){
//         cout<<start[i]<<"  ";
//     }
//     cout<<endl;
//     for(int i=68; i<98; i++){
//         cout<<start[i]<<"  ";
//     }
//     cout<<endl;
    for(int i=98; i<104; i++){
        cout<<start[i]<<"  ";
    }
    //cout<<endl;
    for(int i=104; i<114; i++){
        cout<<start[i]<<"  ";
    }
    cout<<endl;
//     for(int i=114; i<126; i++){
//         cout<<start[i]<<"  ";
//     }
//     cout<<endl;
    for(int i=126; i<132; i++){
        cout<<start[i]<<"  ";
    }
//     cout<<endl;
    for(int i=132; i<142; i++){
        cout<<start[i]<<"  ";
    }
//     cout<<endl;
//     for(int i=142; i<150; i++){
//         cout<<start[i]<<"  ";
//     }
    cout<<endl<<endl;
}

void Node::print_bounds_start(){

//     for(int i=0; i<NV; i++){
//         printf("x[%2d]: [%2.2f]   %2.2f   [%3.2f]\n", i, boundz[i][0], start[i], boundz[i][1]);
//     }
    
    for(int i=0; i<NB; i++){
        printf("x[%2d]: [%2.2f]   %2.2f   [%3.2f]\n", i, boundz[i][0], start[i], boundz[i][1]);
    }
    for(int i=NB; i<98; i++){
        printf("x[%2d]: [%2.2f]   %2.2f   [%3.2f]\n", i, boundz[i][0], start[i], boundz[i][1]);
    }    
    for(int i=98; i<114; i++){
        printf("x[%2d]: [%2.2f]   %2.2f   [%3.2f]\n", i, boundz[i][0], start[i], boundz[i][1]);
    }
    for(int i=126; i<142; i++){
        printf("x[%2d]: [%2.2f]   %2.2f   [%3.2f]\n", i, boundz[i][0], start[i], boundz[i][1]);
    }

    
}
    

void Node::save_sol(const double adr[NV]){
	
    //cout<<"-----Here comes the solution--------------"<<endl<<endl;
	for(int i=0;i<NV;i++){
	sol[i] = adr[i];
	//cout<<"sol "<<i<<": "<<sol[i]<<endl;
	}
}

int Node::is_int_feasible(int frac){
    //Denne m sjekke binritet p alle binrvariable
    double s=0;
    double fpart;
    double ipart;
    for (int i=0; i<NB; i++){
        fpart=modf(sol[i], &ipart);
        s=s+fpart;
    }
    
    fraction = s;
    //cout<<"fractional part: "<<s<<"/"<<frac<<endl;
    if (s<0.001)        
        return 1;
    else
        return 0;
}

bool Node::is_feasible(){
    //This Function can be used to check if the solution in a Node is feasible
    //Use: Check the final best solution to see if its feasible

    //Constraint Group 3
    if (c3() && c4() && c6() && c7() && c8() && c9() && c10() && c11() && cap()){
        
        cout<<"Good Stuff!"<<endl;
        return true;
    }
    else {
        cout<<"That sucks"<<endl;
        bool a;
        a=c3();
        a=c4();
        a=c6();
        a=c7();
        a=c8();
        a=c9();
        a=c10();
        a=c11();
        a=cap();
        return false;
        
    }

}
 
bool Node::c3(){
    int c3[18];
    for (int i=0; i<18; i++){ c3[i]=0;}
    //Topside Well Pressure Balance: (3) in Problem Description File
    int k=126;
    int s=0;
    int e=I;
    
    for(int j = s; j < e ; j++){
        if (sol[k] >= sol[j*3]*SeA){
            c3[j*3] = 1;}
        
        if (sol[k] >=  sol[j*3+1]*SeB){
            c3[j*3+1] = 1;}
        
        if (sol[k] >= sol[j*3+2]*SeC){
            c3[j*3+2] = 1;}
        k++;
    }
    
    int sum=0;
    for(int i=0; i<18; i++){
        sum=sum+c3[i];}
    
    if (sum == 18)
        return true;
    else
        return false;
}

bool Node::c4(){
    int c4[I];
    for (int i=0; i<I; i++){ c4[i]=0;}
    //Topside Well Routing: (4) in Problem Description File
    int s=0;
    int e=I;
    
    for(int j = s; j < e ; j++){
        if ((( sol[(j*3)] + sol[(j*3)+1] + sol[(j*3)+2] ) < 0.001) || (( sol[(j*3)] + sol[(j*3)+1] + sol[(j*3)+2] ) == 1)){
            c4[j] = 1;
        }
    }
    
    int sum=0;
    for (int i=0; i<I; i++){
        sum=sum+c4[i];
    }
    
    if (sum==I)
        return true;
    else
        return false;
    
}

bool Node::c6(){

    //Manifold 1 Pipeline Pressure Drop: (6) in Problem Description File
    double pd1 = sol[142] - sol[146];
    double pd2 = sol[143] - sol[147];
    double pd3 = sol[144] - sol[148];
    double pd4 = sol[145] - sol[149];
    
    cout<<"Pressure Drops: "<<pd1<<"  "<<pd2<<"  "<<pd3<<"  "<<pd4<<"  "<<endl;
    
    if ( ((pd1)>= 0) && ((pd2)>= 0) && ((pd3)>= 0) && ((pd4)>= 0) ){
        return true;
    }else{
        return false;
    }

}

bool Node::c7(){
    int c7[20];
    for (int i=0; i<20; i++){ c7[i]=0;}
    
    //Subsea Manifold 1 Well Pressure Balance: (7) in Problem Description File
    int a = 132;
    int b = 142;
    int c = 18;
    int s=0;
    int e=J1;
    for(int j = s; j < e ; j++){
        if (sol[a] >= (sol[b]*sol[c])){
            c7[j]=1;
        }else{c7[j]=0;}
        c++;
        if (sol[a] >= (sol[b+1]*sol[c])){
            c7[j+J1]=1;
        }else{c7[j+J1]=0;}
        c++;
        a++;
    }
    
    //Subsea Manifold 2 Well Pressure Balance: (7) in Problem Description File
    a = 138;
    b = 144;
    c = 30;
    s=2*J1;
    e=(2*J1)+J2;
    for(int j = s; j < e ; j++){
        if (sol[a] >= (sol[b]*sol[c])){
            c7[j]=1;
        }else{c7[j]=0;}
        c++;
        if (sol[a] >= (sol[b+1]*sol[c])){
            c7[j+J2]=1;
        }else{c7[j+J2]=0;}
        c++;
        a++;
    }
    
    int sum=0;
    cout<<"Manifold Pressure: ";
    for (int i=0; i<20; i++){
        sum=sum+c7[i];
        cout<<c7[i]<<"-";
    }
    cout<<endl;
    
    if (sum == 20)
        return true;
    else 
        return false;
}

bool Node::c8(){
    int c8[J1+J2];
    for (int i=0; i<J1+J2; i++){ c8[i]=0;}
    //Subsea Well Routing: (8) in Problem Description File
    int s=0;
    int e=J1+J2;
    
    int a=18;
    for(int j = s; j < e ; j++){
        if ( (( sol[a] + sol[a+1] ) < 0.001) || (( sol[a] + sol[a+1] ) == 1)){
            c8[j] = 1;
        }else{c8[j] = 0;}
        a=a+2;
    }
    
    int sum=0;
    for (int i=0; i<(J1+J2); i++){
        sum=sum+c8[i];
    }
    
    if (sum==J1+J2)
        return true;
    else
        return false;
}

bool Node::c9(){
    double mb[P*L];
    int i=0;
    //Subsea Manifold 1 Mass Balances: (9) in Problem Description File
    int k=114;   int i1=18;   int i2=68;
    mb[i] = sol[k] - (sol[i1]*sol[i2]) - (sol[i1+2]*sol[i2+1]) - (sol[i1+4]*sol[i2+2]) - (sol[i1+6]*sol[i2+3]) - (sol[i1+8]*sol[i2+4]) - (sol[i1+10]*sol[i2+5]);
    i++;    k++;
    mb[i] = sol[k] - (sol[i1+1]*sol[i2]) - (sol[i1+3]*sol[i2+1]) - (sol[i1+5]*sol[i2+2]) - (sol[i1+7]*sol[i2+3]) - (sol[i1+9]*sol[i2+4]) - (sol[i1+11]*sol[i2+5]);
    i++;    k++;    i2=i2+J1;
    mb[i] = sol[k] - (sol[i1]*sol[i2]) - (sol[i1+2]*sol[i2+1]) - (sol[i1+4]*sol[i2+2]) - (sol[i1+6]*sol[i2+3]) - (sol[i1+8]*sol[i2+4]) - (sol[i1+10]*sol[i2+5]);
    i++;    k++;
    mb[i] = sol[k] - (sol[i1+1]*sol[i2]) - (sol[i1+3]*sol[i2+1]) - (sol[i1+5]*sol[i2+2]) - (sol[i1+7]*sol[i2+3]) - (sol[i1+9]*sol[i2+4]) - (sol[i1+11]*sol[i2+5]);
    i++;    k++;    i2=i2+J1;
    mb[i] = sol[k] - (sol[i1]*sol[i2]) - (sol[i1+2]*sol[i2+1]) - (sol[i1+4]*sol[i2+2]) - (sol[i1+6]*sol[i2+3]) - (sol[i1+8]*sol[i2+4]) - (sol[i1+10]*sol[i2+5]);
    i++;    k++;
    mb[i] = sol[k] - (sol[i1+1]*sol[i2]) - (sol[i1+3]*sol[i2+1]) - (sol[i1+5]*sol[i2+2]) - (sol[i1+7]*sol[i2+3]) - (sol[i1+9]*sol[i2+4]) - (sol[i1+11]*sol[i2+5]);
    i++;
    
    //Subsea Manifold 2 Mass Balances: (9) in Problem Description File
    k=120;   i1=30;   i2=86;
    mb[i] = sol[k] - (sol[i1]*sol[i2]) - (sol[i1+2]*sol[i2+1]) - (sol[i1+4]*sol[i2+2]) - (sol[i1+6]*sol[i2+3]);
    i++;    k++;
    mb[i] = sol[k] - (sol[i1+1]*sol[i2]) - (sol[i1+3]*sol[i2+1]) - (sol[i1+5]*sol[i2+2]) - (sol[i1+7]*sol[i2+3]);
    i++;    k++;    i2=i2+J2;
    mb[i] = sol[k] - (sol[i1]*sol[i2]) - (sol[i1+2]*sol[i2+1]) - (sol[i1+4]*sol[i2+2]) - (sol[i1+6]*sol[i2+3]);
    i++;    k++;
    mb[i] = sol[k] - (sol[i1+1]*sol[i2]) - (sol[i1+3]*sol[i2+1]) - (sol[i1+5]*sol[i2+2]) - (sol[i1+7]*sol[i2+3]);
    i++;    k++;    i2=i2+J2;
    mb[i] = sol[k] - (sol[i1]*sol[i2]) - (sol[i1+2]*sol[i2+1]) - (sol[i1+4]*sol[i2+2]) - (sol[i1+6]*sol[i2+3]);
    i++;    k++;
    mb[i] = sol[k] - (sol[i1+1]*sol[i2]) - (sol[i1+3]*sol[i2+1]) - (sol[i1+5]*sol[i2+2]) - (sol[i1+7]*sol[i2+3]);
    
    cout<<"Mass Balances: ";
    for (int i=0; i<(L*P); i++){
        cout<<mb[i]<<"   ";
    }
    cout<<endl;
    
    return true;
 
}

bool Node::c10(){
    int c10[S*L];
    for (int i=0; i<S*L; i++){ c10[i]=0;}
    //Separator Pressure Balance: (10) in Problem Description File
    if (sol[146] >= (sol[38]*SeA))
        c10[0] = 1;
    
    if (sol[146] >= (sol[39]*SeB))
        c10[1] = 1;
    
    if (sol[146] >= (sol[40]*SeC))
        c10[2] = 1;
    
    if (sol[147] >= (sol[41]*SeA))
        c10[3] = 1;
    
    if (sol[147] >= (sol[42]*SeB))
        c10[4] = 1;
    
    if (sol[147] >= (sol[43]*SeC))
        c10[5] = 1;
    
    if (sol[148] >= (sol[44]*SeA))
        c10[6] = 1;
    
    if (sol[148] >= (sol[45]*SeB))
        c10[7] = 1;
    
    if (sol[148] >= (sol[46]*SeC))
        c10[8] = 1;
    
    if (sol[149] >= (sol[47]*SeA))
        c10[9] = 1;
    
    if (sol[149] >= (sol[48]*SeB))
        c10[10] = 1;
    
    if (sol[149] >= (sol[49]*SeC))
        c10[11] = 1;
    
    
    int sum=0;
    cout<<"SEP Pressure: ";
    for (int i=0; i<S*L; i++){
        sum=sum+c10[i];
        cout<<c10[i]<<"***";
    }
    cout<<endl;
    if (sum == 12)
        return true;
    else   
        return false;      
}

bool Node::c11(){
    int c11[L];
    //Separator Routing: (11) in Problem Description File
    for (int i=0; i<L; i++){ c11[i]=0;}
    int s=0;
    int e=L;
    int a=38;
    for(int j = s; j < e ; j++){
        if ((( sol[a] + sol[a+1] + sol[a+2] ) < 0.001) || (( sol[a] + sol[a+1] + sol[a+2] ) == 1)){
            c11[j] = 1;
        }
        a=a+3;
    }
    
    int sum=0;
    for (int i=0; i<L; i++){
        sum=sum+c11[i];
    }
    
    if (sum==L)
        return true;
    else
        return false;
    
    
}

bool Node::cap(){
    double cap[5];
    
    double c0=1100;
    double c1=2000;
    double ca=2400;
    double cb=2400;
    double cc=2000;
    
    
    //Gas Lift Capacity: (12) in Problem Description File
    cap[0]=  sol[98]*(sol[0]+sol[1]+sol[2]) + sol[99]*(sol[3]+sol[4]+sol[5])
            +sol[100]*(sol[6]+sol[7]+sol[8]) + sol[101]*(sol[9]+sol[10]+sol[11])
            +sol[102]*(sol[12]+sol[13]+sol[14]) + sol[103]*(sol[15]+sol[16]+sol[17])
            
            +sol[104]*(sol[18]+sol[19]) + sol[105]*(sol[20]+sol[21])
            +sol[106]*(sol[22]+sol[23]) + sol[107]*(sol[24]+sol[25])
            +sol[108]*(sol[26]+sol[27]) + sol[109]*(sol[28]+sol[29])
            +sol[110]*(sol[30]+sol[31]) + sol[111]*(sol[32]+sol[33])
            +sol[112]*(sol[34]+sol[35]) + sol[113]*(sol[36]+sol[37]);
    printf("Gas Lift: %5.2f of %5.2f. Slack= %5.2f\n", cap[0],c0,c0-cap[0]);
    
    //Gas Production Capacity: (13) in Problem Description File
    cap[1] = sol[50]*(sol[0]+sol[1]+sol[2]) + sol[51]*(sol[3]+sol[4]+sol[5])
            +sol[52]*(sol[6]+sol[7]+sol[8]) + sol[53]*(sol[9]+sol[10]+sol[11])
            +sol[54]*(sol[12]+sol[13]+sol[14]) + sol[55]*(sol[15]+sol[16]+sol[17])
            +sol[114]+sol[115]+sol[120]+sol[121];
    printf("Gas Prod: %5.2f of %5.2f. Slack= %5.2f\n", cap[1],c1, c1-cap[1]);
    
    //Water Production Capacity: (14) in Problem Description File
    
    //Separator A
    cap[2]=    sol[62]*sol[0] + sol[63]*sol[3] + sol[64]*sol[6] + sol[65]*sol[9] + sol[66]*sol[12] + sol[67]*sol[15]
            +sol[118]*sol[38] + sol[119]*sol[41] + sol[124]*sol[44] + sol[125]*sol[47];
    printf("Sep A: %5.2f of %5.2f. Slack= %5.2f\n", cap[2],ca, ca-cap[2]);
    
    //Separator B
    cap[3]=    sol[62]*sol[1] + sol[63]*sol[4] + sol[64]*sol[7] + sol[65]*sol[10] + sol[66]*sol[13] + sol[67]*sol[16]
            +sol[118]*sol[39] + sol[119]*sol[42] + sol[124]*sol[45] + sol[125]*sol[48];
    printf("Sep B: %5.2f of %5.2f. Slack= %5.2f\n", cap[3], cb,cb-cap[3]);
    
    //Separator C
    cap[4]=    sol[62]*sol[2] + sol[63]*sol[5] + sol[64]*sol[8] + sol[65]*sol[11] + sol[66]*sol[14] + sol[67]*sol[17]
            +sol[118]*sol[40] + sol[119]*sol[43] + sol[124]*sol[46] + sol[125]*sol[49];
    printf("Sep C: %5.2f of %5.2f. Slack= %5.2f\n", cap[4],cc, cc-cap[4]);

    if ( (cap[0]<=c0) && (cap[1]<=c1) && (cap[2]<=ca) && (cap[3]<=cb) && (cap[4]<=cc))
        return true;
    else
        return false;

}

void Node::set_mmw(double minmax_w[I+J][3][2]){
    
    for(int i=0; i<(I+J); i++){
       for (int j=0; j<3; j++){
           mmw[i][j][0] = minmax_w[i][j][0];
           mmw[i][j][1] = minmax_w[i][j][1];
           //cout<<"mmw x["<<i<<"]:   ["<<node.mmw[i][j][0]<<"  "<<node.mmw[i][j][1]<<"]"<<endl;
       }
        
        
    }
}

int Node::var_choice(){

//MOST FRACTIONAL    
//     double fpart[NB];
//     double ipart[NB];
// 
//     for (int i=0; i<NB; i++){
//         fpart[i] = modf(sol[i], &ipart[i]);
//     }
//     
//     int r=max_val(NB, fpart);
//     return r;

//CLOSEST TO 0.5
//     double a[NB];
//     
//     for(int i=0; i<NB; i++){
//         a[i] = abs(0.5-sol[i]);
//         //cout<<sol[i]<<"--->"<<a[i]<<endl;
//     }
//     
//     int r=min_val(NB, a);
//     
//     return r;
    
//CLOSEST TO 0 OR 1
    double a[NB];
    
    for(int i=0; i<NB; i++){
        if (sol[i] == 0 || sol[i] == 1){
            a[i] = 1.0;
            //cout<<sol[i]<<"--->"<<a[i]<<endl;
        }
        else if (sol[i] < 0.5){
            a[i]=sol[i];
            //cout<<sol[i]<<"--->"<<a[i]<<endl;
        }
        else{
            a[i]=1-sol[i];
            //cout<<sol[i]<<"--->"<<a[i]<<endl;
        }
    }
    
    int r=min_val(NB, a);
    //cout<<"r is var nr "<<r<<" with value "<<sol[r]<<endl;
    return r;
    
        
}

void Node::next_generation(Node& k1, Node& k2, int var){    
   
    //Fix the branching variable to zero in the first problem, both in bounds and startpoint
    k1.boundz[var][0]=0;
    k1.boundz[var][1]=0;
    k1.start[var]=0;
    
       
    //Fix the branching variable to one in the second problem, both in bounds and startpoint
    k2.boundz[var][0]=1;
    k2.boundz[var][1]=1;
    k2.start[var]=1;
        //Also fix the remaining variables in the variable's 'group' to zero in the 1 branch
        int a=100;
        int b=100;
        if (Node::groups[var][0] > -1){
            a = Node::groups[var][0];
            k2.boundz[a][0]=0;
            k2.boundz[a][1]=0;
            k2.start[a]=0;
        }
        
        if (Node::groups[var][1] > -1){
            b = Node::groups[var][1];
            k2.boundz[b][0]=0;
            k2.boundz[b][1]=0;
            k2.start[b]=0;
        }
        
        //cout<<"BranchingVar:"<<var<<"  vars in group to be set to zero:"<<a<<" & "<<b<<endl;
}

void Node::nodes_iterate(){
    total_nodes++;    
}

void Node::print_node() const{
   
    printf("NODE: %-5d OFV: %5.2f\n", 
                probnum, objf);
    
    twells_open();
    cout<<endl;
    swells_open();
    cout<<endl;
    for(int i=0; i<18; i++){
        if(sol[i]<0.0001){cout<<"0"<<"  ";}
        else{cout<<sol[i]<<"  ";}
    }
    cout<<endl;
    for(int i=18; i<38; i++){
        if(sol[i]<0.0001){cout<<"0"<<"  ";}
        else{cout<<sol[i]<<"  ";}
    }
    cout<<endl;
    for(int i=38; i<50; i++){
        if(sol[i]<0.0001){cout<<"0"<<"  ";}
        else{cout<<sol[i]<<"  ";}
    }
    cout<<endl;
    line_to_sep();
    cout<<endl;
    for(int i=50; i<68; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=68; i<98; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=98; i<104; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=104; i<114; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=114; i<126; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=126; i<132; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=132; i<142; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    for(int i=142; i<150; i++){
        cout<<sol[i]<<"  ";
    }
    cout<<endl;
    
    printf("\nWriting solution file solution.txt\n");
    FILE* fp = fopen("solution.txt", "w");

    // For this example, we write the solution to the console
    fprintf(fp, "\n\nSolution of the primal variables, x\n");
    for (Index i=0; i<NV; i++) {
        fprintf(fp, "%e\n", sol[i]); //x[%d] =
    }
  
    fprintf(fp, "\n\nObjective value\n");
    fprintf(fp, "f(x*) = %e\n", objf);
    fclose(fp);
    
}

void Node::twells_open() const{
    int temp[I];
    int k=3;
    for(int i=0; i<I; i++){
        if (sol[i*k]+sol[(i*k)+1]+sol[(i*k)+2] == 1){
            temp[i] = 1;
        }else{
            temp[i] = 0;
        }
    }
    
    cout<<"Topside Wells On/Off: "<<temp[0]<<"  "<<temp[1]<<"  "<<temp[2]<<"  "<<temp[3]<<"  "<<temp[4]<<"  "<<temp[5]; 
}

void Node::swells_open() const{
    int temp[J];
    int k=18;
    for(int i=0; i<J; i++){
        if (sol[k]+sol[k+1] == 1){
            temp[i] = 1;
        }else{
            temp[i] = 0;
        }
        k=k+2;
    }
    
    cout<<"Subsea Wells On/Off: "<<temp[0]<<"  "<<temp[1]<<"  "<<temp[2]<<"  "<<temp[3]<<"  "<<temp[4]<<"  " 
            <<temp[5]<<"  " <<temp[6]<<"  " <<temp[7]<<"  " <<temp[8]<<"  "<<temp[9];
}

void Node::line_to_sep() const{
    cout<<"Lines to Separators: ";
    
    int k=38;
    for (int i=0; i<L; i++){
        if (sol[k] == 1){
            cout<<"L"<<i+1<<"-SA ";
        }
        else if (sol[k+1] == 1){
            cout<<"L"<<i+1<<"-SB ";
        }
        else if (sol[k+2] == 1){
            cout<<"L"<<i+1<<"-SC ";
        }
        k=k+3;
    }
    cout<<endl;

}

int Node::min_val(int size, double a[]){
    int index=0;
    double temp=a[0];
    for (int i=1; i<size; i++){
        if (a[i]==-1){
        }
        else if (a[i]<temp){
            temp=a[i];
            index=i;
        }
    }
    return index;
}
        
int Node::max_val(int size, double a[]){
    int index=0;
    double temp=a[0];
    for (int i=1; i<size; i++){
        if (a[i] == -1){
        }
        else if (a[i]>temp){
            temp=a[i];
            index=i;
        }
    }
    return index;
}

int Node::total_nodes=-1;

int Node::groups[][2];

void Node::Groups(){  
    for(int i=0; i<18; i=i+3){
        Node::groups[i][0]=i+1;   Node::groups[i][1]=i+2;
        Node::groups[i+1][0]=i;   Node::groups[i+1][1]=i+2;
        Node::groups[i+2][0]=i;   Node::groups[i+2][1]=i+1;
    }
    
    for(int i=18; i<38; i=i+2){
        Node::groups[i][0]=i+1;   Node::groups[i][1]=-1;
        Node::groups[i+1][0]=i;   Node::groups[i+1][1]=-1;
    }
    
        for(int i=38; i<50; i=i+3){
        Node::groups[i][0]=i+1;   Node::groups[i][1]=i+2;
        Node::groups[i+1][0]=i;   Node::groups[i+1][1]=i+2;
        Node::groups[i+2][0]=i;   Node::groups[i+2][1]=i+1;
    }
    
//     cout<<"Node::groups"<<endl;
//     for(int i=0; i<NB; i++){
//         cout<<Node::groups[i][0]<<"   "<<Node::groups[i][1]<<endl;
//     }
}

void Node::set_heur_bin(double a[NB], Node& n){
    for(int i=0; i<NB; i++){
        n.start[i] = a[i];
        //cout<<a[i]<<endl;
    }
    
    for(int i=0; i<NB; i++){
            n.boundz[i][0]=a[i];
            n.boundz[i][1]=a[i];
            //cout<<n.boundz[i][0]<<"        "<<n.boundz[i][1]<<endl;
    }
    
}

void Node::printing_(double best_OFV){
FILE* fp = fopen("ut.txt", "a");

fprintf(fp, "PNum: %5d OFV: %9.2f BLB: %9.2f Binaries: %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f %5.5f\n", 
        probnum, objf, best_OFV,
        sol[38], sol[39], sol[40], sol[41], sol[42], sol[43], sol[44], sol[45], sol[46], sol[47], sol[48], sol[49]);


    fclose(fp);

}

void Node::lock_BoundPH2(double locked[NB],double milp[NB]){
    
    count=0;
    for (int i=0; i<NB; i++){
    if(locked[i]==0){
        boundz[i][0]=milp[i];
        boundz[i][1]=milp[i];
        start[i]=milp[i];
        //cout<<"l_bound["<<i<<"]: "<<l_bound[i]<<endl;
        //cout<<"u_bound["<<i<<"]: "<<u_bound[i]<<endl;
        count++;
        //cout<<"count="<<count<<endl;
    }
    }
    
}

void Node::feas_pump_start_bound(){
//Save binary variables

//Root Node    
start[0]=1;  boundz[0][0]=1;  boundz[0][1]=1;
start[1]=0;  boundz[1][0]=0;  boundz[1][1]=0;
start[2]=0;  boundz[2][0]=0;  boundz[2][1]=0;
start[3]=0;  boundz[3][0]=0;  boundz[3][1]=0;
start[4]=0;  boundz[4][0]=0;  boundz[4][1]=0;
start[5]=1;  boundz[5][0]=1;  boundz[5][1]=1;
start[6]=1;  boundz[6][0]=1;  boundz[6][1]=1;
start[7]=0;  boundz[7][0]=0;  boundz[7][1]=0;
start[8]=0;  boundz[8][0]=0;  boundz[8][1]=0;
start[9]=0;  boundz[9][0]=0;  boundz[9][1]=0;
start[10]=1;  boundz[10][0]=1;  boundz[10][1]=1;
start[11]=0;  boundz[11][0]=0;  boundz[11][1]=0;
start[12]=0;  boundz[12][0]=0;  boundz[12][1]=0;
start[13]=1;  boundz[13][0]=1;  boundz[13][1]=1;
start[14]=0;  boundz[14][0]=0;  boundz[14][1]=0;
start[15]=0;  boundz[15][0]=0;  boundz[15][1]=0;
start[16]=0;  boundz[16][0]=0;  boundz[16][1]=0;
start[17]=1;  boundz[17][0]=1;  boundz[17][1]=1;
start[18]=0;  boundz[18][0]=0;  boundz[18][1]=0;
start[19]=1;  boundz[19][0]=1;  boundz[19][1]=1;
start[20]=0;  boundz[20][0]=0;  boundz[20][1]=0;
start[21]=0;  boundz[21][0]=0;  boundz[21][1]=0;
start[22]=0;  boundz[22][0]=0;  boundz[22][1]=0;
start[23]=1;  boundz[23][0]=1;  boundz[23][1]=1;
start[24]=1;  boundz[24][0]=1;  boundz[24][1]=1;
start[25]=0;  boundz[25][0]=0;  boundz[25][1]=0;
start[26]=1;  boundz[26][0]=1;  boundz[26][1]=1;
start[27]=0;  boundz[27][0]=0;  boundz[27][1]=0;
start[28]=0;  boundz[28][0]=0;  boundz[28][1]=0;
start[29]=1;  boundz[29][0]=1;  boundz[29][1]=1;
start[30]=0;  boundz[30][0]=0;  boundz[30][1]=0;
start[31]=1;  boundz[31][0]=1;  boundz[31][1]=1;
start[32]=0;  boundz[32][0]=0;  boundz[32][1]=0;
start[33]=1;  boundz[33][0]=1;  boundz[33][1]=1;
start[34]=1;  boundz[34][0]=1;  boundz[34][1]=1;
start[35]=0;  boundz[35][0]=0;  boundz[35][1]=0;
start[36]=1;  boundz[36][0]=1;  boundz[36][1]=1;
start[37]=0;  boundz[37][0]=0;  boundz[37][1]=0;
start[38]=0;  boundz[38][0]=0;  boundz[38][1]=0;
start[39]=1;  boundz[39][0]=1;  boundz[39][1]=1;
start[40]=0;  boundz[40][0]=0;  boundz[40][1]=0;
start[41]=1;  boundz[41][0]=1;  boundz[41][1]=1;
start[42]=0;  boundz[42][0]=0;  boundz[42][1]=0;
start[43]=0;  boundz[43][0]=0;  boundz[43][1]=0;
start[44]=1;  boundz[44][0]=1;  boundz[44][1]=1;
start[45]=0;  boundz[45][0]=0;  boundz[45][1]=0;
start[46]=0;  boundz[46][0]=0;  boundz[46][1]=0;
start[47]=1;  boundz[47][0]=1;  boundz[47][1]=1;
start[48]=0;  boundz[48][0]=0;  boundz[48][1]=0;
start[49]=0;  boundz[49][0]=0;  boundz[49][1]=0;
    

    
// Random Starting Point 1
// start[0]=0;  boundz[0][0]=0;  boundz[0][1]=0;
// start[1]=0;  boundz[1][0]=0;  boundz[1][1]=0;
// start[2]=1;  boundz[2][0]=1;  boundz[2][1]=1;
// start[3]=0;  boundz[3][0]=0;  boundz[3][1]=0;
// start[4]=0;  boundz[4][0]=0;  boundz[4][1]=0;
// start[5]=1;  boundz[5][0]=1;  boundz[5][1]=1;
// start[6]=0;  boundz[6][0]=0;  boundz[6][1]=0;
// start[7]=0;  boundz[7][0]=0;  boundz[7][1]=0;
// start[8]=1;  boundz[8][0]=1;  boundz[8][1]=1;
// start[9]=1;  boundz[9][0]=1;  boundz[9][1]=1;
// start[10]=0;  boundz[10][0]=0;  boundz[10][1]=0;
// start[11]=0;  boundz[11][0]=0;  boundz[11][1]=0;
// start[12]=0;  boundz[12][0]=0;  boundz[12][1]=0;
// start[13]=1;  boundz[13][0]=1;  boundz[13][1]=1;
// start[14]=0;  boundz[14][0]=0;  boundz[14][1]=0;
// start[15]=0;  boundz[15][0]=0;  boundz[15][1]=0;
// start[16]=1;  boundz[16][0]=1;  boundz[16][1]=1;
// start[17]=0;  boundz[17][0]=0;  boundz[17][1]=0;
// start[18]=1;  boundz[18][0]=1;  boundz[18][1]=1;
// start[19]=0;  boundz[19][0]=0;  boundz[19][1]=0;
// start[20]=0;  boundz[20][0]=0;  boundz[20][1]=0;
// start[21]=0;  boundz[21][0]=0;  boundz[21][1]=0;
// start[22]=1;  boundz[22][0]=1;  boundz[22][1]=1;
// start[23]=0;  boundz[23][0]=0;  boundz[23][1]=0;
// start[24]=0;  boundz[24][0]=0;  boundz[24][1]=0;
// start[25]=1;  boundz[25][0]=1;  boundz[25][1]=1;
// start[26]=0;  boundz[26][0]=0;  boundz[26][1]=0;
// start[27]=1;  boundz[27][0]=1;  boundz[27][1]=1;
// start[28]=1;  boundz[28][0]=1;  boundz[28][1]=1;
// start[29]=0;  boundz[29][0]=0;  boundz[29][1]=0;
// start[30]=0;  boundz[30][0]=0;  boundz[30][1]=0;
// start[31]=1;  boundz[31][0]=1;  boundz[31][1]=1;
// start[32]=0;  boundz[32][0]=0;  boundz[32][1]=0;
// start[33]=1;  boundz[33][0]=1;  boundz[33][1]=1;
// start[34]=1;  boundz[34][0]=1;  boundz[34][1]=1;
// start[35]=0;  boundz[35][0]=0;  boundz[35][1]=0;
// start[36]=0;  boundz[36][0]=0;  boundz[36][1]=0;
// start[37]=0;  boundz[37][0]=0;  boundz[37][1]=0;
// start[38]=1;  boundz[38][0]=1;  boundz[38][1]=1;
// start[39]=0;  boundz[39][0]=0;  boundz[39][1]=0;
// start[40]=0;  boundz[40][0]=0;  boundz[40][1]=0;
// start[41]=0;  boundz[41][0]=0;  boundz[41][1]=0;
// start[42]=1;  boundz[42][0]=1;  boundz[42][1]=1;
// start[43]=0;  boundz[43][0]=0;  boundz[43][1]=0;
// start[44]=1;  boundz[44][0]=1;  boundz[44][1]=1;
// start[45]=0;  boundz[45][0]=0;  boundz[45][1]=0;
// start[46]=0;  boundz[46][0]=0;  boundz[46][1]=0;
// start[47]=1;  boundz[47][0]=1;  boundz[47][1]=1;
// start[48]=0;  boundz[48][0]=0;  boundz[48][1]=0;
// start[49]=0;  boundz[49][0]=0;  boundz[49][1]=0;
// start[50]=126.046;		
// start[51]=341.678;		
// start[52]=114.796;		
// start[53]=175.28;		
// start[54]=187.996;		
// start[55]=61.6899;		
// start[56]=86.0955;		
// start[57]=1292;		
// start[58]=328.431;		
// start[59]=426.456;		
// start[60]=714.623;		
// start[61]=397.084;		
// start[62]=135.23;		
// start[63]=1122.5;		
// start[64]=723.558;		
// start[65]=866.618;		
// start[66]=1122.45;		
// start[67]=753.217;		
// start[68]=102.192;		
// start[69]=44.1604;		
// start[70]=60.0179;		
// start[71]=209.058;		
// start[72]=95.1687;		
// start[73]=117.846;		
// start[74]=764.357;		
// start[75]=17.353;		
// start[76]=259.951;		
// start[77]=834.426;		
// start[78]=891.799;		
// start[79]=890.956;		
// start[80]=193.844;		
// start[81]=65.2804;		
// start[82]=157.239;		
// start[83]=183.167;		
// start[84]=341.16;		
// start[85]=225.95;		
// start[86]=193.546;		
// start[87]=4.23366;		
// start[88]=159.605;		
// start[89]=19.5354;		
// start[90]=1176.27;		
// start[91]=97.4938;		
// start[92]=2257.18;		
// start[93]=33.8167;		
// start[94]=563.774;		
// start[95]=156.396;		
// start[96]=207.797;		
// start[97]=84.341;		
// start[98]=118.383;		
// start[99]=239.997;		
// start[100]=81.657;		
// start[101]=139.799;		
// start[102]=119.964;		
// start[103]=33.8941;		
// start[104]=35.54;		
// start[105]=42.6854;		
// start[106]=40.0016;		
// start[107]=142.304;		
// start[108]=10.0019;		
// start[109]=40.1551;		
// start[110]=98.2686;		
// start[111]=0.00242976;		
// start[112]=0.0230119;		
// start[113]=17.662;		
// start[114]=280.056;		
// start[115]=304.227;		
// start[116]=1915.26;		
// start[117]=1726.23;		
// start[118]=577.033;		
// start[119]=524.326;		
// start[120]=159.605;		
// start[121]=197.78;		
// start[122]=2257.18;		
// start[123]=1273.76;		
// start[124]=207.797;		
// start[125]=720.17;		
// start[126]=15.0014;		
// start[127]=15.0002;		
// start[128]=15.0006;		
// start[129]=10.0005;		
// start[130]=12.0007;		
// start[131]=12.0073;		
// start[132]=63.865;		
// start[133]=145.459;		
// start[134]=65.4021;		
// start[135]=69.6115;		
// start[136]=69.6127;		
// start[137]=63.865;		
// start[138]=57.1239;		
// start[139]=72.1379;		
// start[140]=48.3786;		
// start[141]=115.039;		
// start[142]=63.8646;		
// start[143]=69.6113;		
// start[144]=48.3785;		
// start[145]=57.1237;		
// start[146]=10.0002;		
// start[147]=12.0002;		
// start[148]=10.0001;		
// start[149]=10.0002;		


//Random Starting Point 2
// start[0]=1;  boundz[0][0]=1;  boundz[0][1]=1;
// start[1]=0;  boundz[1][0]=0;  boundz[1][1]=0;
// start[2]=0;  boundz[2][0]=0;  boundz[2][1]=0;
// start[3]=0;  boundz[3][0]=0;  boundz[3][1]=0;
// start[4]=0;  boundz[4][0]=0;  boundz[4][1]=0;
// start[5]=1;  boundz[5][0]=1;  boundz[5][1]=1;
// start[6]=1;  boundz[6][0]=1;  boundz[6][1]=1;
// start[7]=0;  boundz[7][0]=0;  boundz[7][1]=0;
// start[8]=0;  boundz[8][0]=0;  boundz[8][1]=0;
// start[9]=1;  boundz[9][0]=1;  boundz[9][1]=1;
// start[10]=0;  boundz[10][0]=0;  boundz[10][1]=0;
// start[11]=0;  boundz[11][0]=0;  boundz[11][1]=0;
// start[12]=0;  boundz[12][0]=0;  boundz[12][1]=0;
// start[13]=1;  boundz[13][0]=1;  boundz[13][1]=1;
// start[14]=0;  boundz[14][0]=0;  boundz[14][1]=0;
// start[15]=0;  boundz[15][0]=0;  boundz[15][1]=0;
// start[16]=0;  boundz[16][0]=0;  boundz[16][1]=0;
// start[17]=1;  boundz[17][0]=1;  boundz[17][1]=1;
// start[18]=0;  boundz[18][0]=0;  boundz[18][1]=0;
// start[19]=1;  boundz[19][0]=1;  boundz[19][1]=1;
// start[20]=0;  boundz[20][0]=0;  boundz[20][1]=0;
// start[21]=0;  boundz[21][0]=0;  boundz[21][1]=0;
// start[22]=0;  boundz[22][0]=0;  boundz[22][1]=0;
// start[23]=1;  boundz[23][0]=1;  boundz[23][1]=1;
// start[24]=1;  boundz[24][0]=1;  boundz[24][1]=1;
// start[25]=0;  boundz[25][0]=0;  boundz[25][1]=0;
// start[26]=1;  boundz[26][0]=1;  boundz[26][1]=1;
// start[27]=0;  boundz[27][0]=0;  boundz[27][1]=0;
// start[28]=0;  boundz[28][0]=0;  boundz[28][1]=0;
// start[29]=1;  boundz[29][0]=1;  boundz[29][1]=1;
// start[30]=0;  boundz[30][0]=0;  boundz[30][1]=0;
// start[31]=1;  boundz[31][0]=1;  boundz[31][1]=1;
// start[32]=0;  boundz[32][0]=0;  boundz[32][1]=0;
// start[33]=0;  boundz[33][0]=0;  boundz[33][1]=0;
// start[34]=1;  boundz[34][0]=1;  boundz[34][1]=1;
// start[35]=0;  boundz[35][0]=0;  boundz[35][1]=0;
// start[36]=0;  boundz[36][0]=0;  boundz[36][1]=0;
// start[37]=0;  boundz[37][0]=0;  boundz[37][1]=0;
// start[38]=1;  boundz[38][0]=1;  boundz[38][1]=1;
// start[39]=0;  boundz[39][0]=0;  boundz[39][1]=0;
// start[40]=0;  boundz[40][0]=0;  boundz[40][1]=0;
// start[41]=0;  boundz[41][0]=0;  boundz[41][1]=0;
// start[42]=1;  boundz[42][0]=1;  boundz[42][1]=1;
// start[43]=0;  boundz[43][0]=0;  boundz[43][1]=0;
// start[44]=1;  boundz[44][0]=1;  boundz[44][1]=1;
// start[45]=0;  boundz[45][0]=0;  boundz[45][1]=0;
// start[46]=0;  boundz[46][0]=0;  boundz[46][1]=0;
// start[47]=0;  boundz[47][0]=0;  boundz[47][1]=0;
// start[48]=1;  boundz[48][0]=1;  boundz[48][1]=1;
// start[49]=0;  boundz[49][0]=0;  boundz[49][1]=0;
// start[50]=96.7124;		
// start[51]=341.68;		
// start[52]=30.0167;		
// start[53]=121.205;		
// start[54]=290.49;		
// start[55]=101.559;		
// start[56]=85.7619;		
// start[57]=1292;		
// start[58]=297.487;		
// start[59]=407.483;		
// start[60]=740.45;		
// start[61]=462.603;		
// start[62]=134.706;		
// start[63]=1122.5;		
// start[64]=655.386;		
// start[65]=828.062;		
// start[66]=1163.02;		
// start[67]=877.498;		
// start[68]=101.292;		
// start[69]=10.2575;		
// start[70]=58.6261;		
// start[71]=174.037;		
// start[72]=105.713;		
// start[73]=145.599;		
// start[74]=703.517;		
// start[75]=51.0062;		
// start[76]=241.896;		
// start[77]=868.462;		
// start[78]=1002.23;		
// start[79]=877.862;		
// start[80]=178.415;		
// start[81]=191.88;		
// start[82]=146.318;		
// start[83]=190.638;		
// start[84]=383.407;		
// start[85]=222.629;		
// start[86]=228.137;		
// start[87]=24.6437;		
// start[88]=159.603;		
// start[89]=24.7226;		
// start[90]=1200.84;		
// start[91]=138.81;		
// start[92]=2257.22;		
// start[93]=104.073;		
// start[94]=575.55;		
// start[95]=222.675;		
// start[96]=207.801;		
// start[97]=259.564;		
// start[98]=89.0795;		
// start[99]=240;		
// start[100]=0.0002244;		
// start[101]=87.3023;		
// start[102]=220;		
// start[103]=69.1765;		
// start[104]=39.9452;		
// start[105]=5.92198;		
// start[106]=40;		
// start[107]=104.56;		
// start[108]=10;		
// start[109]=69.0496;		
// start[110]=130.869;		
// start[111]=18.6194;		
// start[112]=0.0175778;		
// start[113]=18.9569;		
// start[114]=279.75;		
// start[115]=305.517;		
// start[116]=1870.7;		
// start[117]=1823.28;		
// start[118]=574.045;		
// start[119]=547.362;		
// start[120]=159.603;		
// start[121]=228.137;		
// start[122]=2257.22;		
// start[123]=1200.84;		
// start[124]=207.801;		
// start[125]=575.55;		
// start[126]=10.0001;		
// start[127]=15;		
// start[128]=13.6403;		
// start[129]=10.0001;		
// start[130]=12;		
// start[131]=15;		
// start[132]=71.7024;		
// start[133]=71.864;		
// start[134]=71.7026;		
// start[135]=63.7501;		
// start[136]=63.7502;		
// start[137]=71.7024;		
// start[138]=56.9143;		
// start[139]=78.1654;		
// start[140]=48.3763;		
// start[141]=94.1127;		
// start[142]=63.7501;		
// start[143]=71.7024;		
// start[144]=48.3763;		
// start[145]=56.9143;		
// start[146]=10;		
// start[147]=12;		
// start[148]=10;		
// start[149]=12;		

//Random Starting Point 3
// start[0]=0;  boundz[0][0]=0;  boundz[0][1]=0;
// start[1]=1;  boundz[1][0]=1;  boundz[1][1]=1;
// start[2]=0;  boundz[2][0]=0;  boundz[2][1]=0;
// start[3]=0;  boundz[3][0]=0;  boundz[3][1]=0;
// start[4]=1;  boundz[4][0]=1;  boundz[4][1]=1;
// start[5]=0;  boundz[5][0]=0;  boundz[5][1]=0;
// start[6]=0;  boundz[6][0]=0;  boundz[6][1]=0;
// start[7]=1;  boundz[7][0]=1;  boundz[7][1]=1;
// start[8]=0;  boundz[8][0]=0;  boundz[8][1]=0;
// start[9]=1;  boundz[9][0]=1;  boundz[9][1]=1;
// start[10]=0;  boundz[10][0]=0;  boundz[10][1]=0;
// start[11]=0;  boundz[11][0]=0;  boundz[11][1]=0;
// start[12]=0;  boundz[12][0]=0;  boundz[12][1]=0;
// start[13]=0;  boundz[13][0]=0;  boundz[13][1]=0;
// start[14]=1;  boundz[14][0]=1;  boundz[14][1]=1;
// start[15]=0;  boundz[15][0]=0;  boundz[15][1]=0;
// start[16]=0;  boundz[16][0]=0;  boundz[16][1]=0;
// start[17]=1;  boundz[17][0]=1;  boundz[17][1]=1;
// start[18]=1;  boundz[18][0]=1;  boundz[18][1]=1;
// start[19]=0;  boundz[19][0]=0;  boundz[19][1]=0;
// start[20]=0;  boundz[20][0]=0;  boundz[20][1]=0;
// start[21]=0;  boundz[21][0]=0;  boundz[21][1]=0;
// start[22]=1;  boundz[22][0]=1;  boundz[22][1]=1;
// start[23]=0;  boundz[23][0]=0;  boundz[23][1]=0;
// start[24]=0;  boundz[24][0]=0;  boundz[24][1]=0;
// start[25]=1;  boundz[25][0]=1;  boundz[25][1]=1;
// start[26]=0;  boundz[26][0]=0;  boundz[26][1]=0;
// start[27]=1;  boundz[27][0]=1;  boundz[27][1]=1;
// start[28]=1;  boundz[28][0]=1;  boundz[28][1]=1;
// start[29]=0;  boundz[29][0]=0;  boundz[29][1]=0;
// start[30]=0;  boundz[30][0]=0;  boundz[30][1]=0;
// start[31]=1;  boundz[31][0]=1;  boundz[31][1]=1;
// start[32]=0;  boundz[32][0]=0;  boundz[32][1]=0;
// start[33]=1;  boundz[33][0]=1;  boundz[33][1]=1;
// start[34]=1;  boundz[34][0]=1;  boundz[34][1]=1;
// start[35]=0;  boundz[35][0]=0;  boundz[35][1]=0;
// start[36]=0;  boundz[36][0]=0;  boundz[36][1]=0;
// start[37]=0;  boundz[37][0]=0;  boundz[37][1]=0;
// start[38]=1;  boundz[38][0]=1;  boundz[38][1]=1;
// start[39]=0;  boundz[39][0]=0;  boundz[39][1]=0;
// start[40]=0;  boundz[40][0]=0;  boundz[40][1]=0;
// start[41]=0;  boundz[41][0]=0;  boundz[41][1]=0;
// start[42]=1;  boundz[42][0]=1;  boundz[42][1]=1;
// start[43]=0;  boundz[43][0]=0;  boundz[43][1]=0;
// start[44]=1;  boundz[44][0]=1;  boundz[44][1]=1;
// start[45]=0;  boundz[45][0]=0;  boundz[45][1]=0;
// start[46]=0;  boundz[46][0]=0;  boundz[46][1]=0;
// start[47]=1;  boundz[47][0]=1;  boundz[47][1]=1;
// start[48]=0;  boundz[48][0]=0;  boundz[48][1]=0;
// start[49]=0;  boundz[49][0]=0;  boundz[49][1]=0;
// start[50]=71.9088;		
// start[51]=343.017;		
// start[52]=27.606;		
// start[53]=231.887;		
// start[54]=241.429;		
// start[55]=100.66;		
// start[56]=79.6369;		
// start[57]=1308.99;		
// start[58]=273.598;		
// start[59]=426.214;		
// start[60]=716.403;		
// start[61]=461.154;		
// start[62]=125.085;		
// start[63]=1137.26;		
// start[64]=602.756;		
// start[65]=866.127;		
// start[66]=1125.25;		
// start[67]=874.75;		
// start[68]=109.661;		
// start[69]=23.4505;		
// start[70]=60.0905;		
// start[71]=176.266;		
// start[72]=98.5276;		
// start[73]=116.842;		
// start[74]=771.076;		
// start[75]=41.495;		
// start[76]=260.915;		
// start[77]=821.247;		
// start[78]=926.99;		
// start[79]=881.211;		
// start[80]=195.548;		
// start[81]=156.1;		
// start[82]=157.822;		
// start[83]=180.274;		
// start[84]=354.622;		
// start[85]=223.479;		
// start[86]=205.585;		
// start[87]=1.77894;		
// start[88]=165.051;		
// start[89]=45.5082;		
// start[90]=1195.74;		
// start[91]=40.9892;		
// start[92]=2257.77;		
// start[93]=72.0802;		
// start[94]=573.107;		
// start[95]=65.7535;		
// start[96]=207.851;		
// start[97]=179.772;		
// start[98]=64.8211;		
// start[99]=240;		
// start[100]=4.15078e-05;		
// start[101]=196.426;		
// start[102]=173.228;		
// start[103]=68.3792;		
// start[104]=42.4228;		
// start[105]=19.9235;		
// start[106]=40;		
// start[107]=110.566;		
// start[108]=10;		
// start[109]=40;		
// start[110]=108.73;		
// start[111]=6.90645e-06;		
// start[112]=5.4266;		
// start[113]=41.5149;		
// start[114]=286.593;		
// start[115]=274.794;		
// start[116]=1913.2;		
// start[117]=1748.24;		
// start[118]=576.849;		
// start[119]=534.896;		
// start[120]=165.051;		
// start[121]=207.364;		
// start[122]=2257.77;		
// start[123]=1236.73;		
// start[124]=207.851;		
// start[125]=638.861;		
// start[126]=12;		
// start[127]=12;		
// start[128]=18.0947;		
// start[129]=10;		
// start[130]=15;		
// start[131]=15;		
// start[132]=64.8324;		
// start[133]=95.0317;		
// start[134]=65.0646;		
// start[135]=67.7444;		
// start[136]=67.7444;		
// start[137]=64.8324;		
// start[138]=55.7373;		
// start[139]=86.5201;		
// start[140]=48.5776;		
// start[141]=111.741;		
// start[142]=64.8324;		
// start[143]=67.7444;		
// start[144]=48.5776;		
// start[145]=55.7373;		
// start[146]=10;		
// start[147]=12;		
// start[148]=10;		
// start[149]=10;		


//Random Starting Point 4
// start[0]=1;  boundz[0][0]=1;  boundz[0][1]=1;
// start[1]=0;  boundz[1][0]=0;  boundz[1][1]=0;
// start[2]=0;  boundz[2][0]=0;  boundz[2][1]=0;
// start[3]=0;  boundz[3][0]=0;  boundz[3][1]=0;
// start[4]=1;  boundz[4][0]=1;  boundz[4][1]=1;
// start[5]=0;  boundz[5][0]=0;  boundz[5][1]=0;
// start[6]=0;  boundz[6][0]=0;  boundz[6][1]=0;
// start[7]=1;  boundz[7][0]=1;  boundz[7][1]=1;
// start[8]=0;  boundz[8][0]=0;  boundz[8][1]=0;
// start[9]=0;  boundz[9][0]=0;  boundz[9][1]=0;
// start[10]=0;  boundz[10][0]=0;  boundz[10][1]=0;
// start[11]=1;  boundz[11][0]=1;  boundz[11][1]=1;
// start[12]=0;  boundz[12][0]=0;  boundz[12][1]=0;
// start[13]=0;  boundz[13][0]=0;  boundz[13][1]=0;
// start[14]=1;  boundz[14][0]=1;  boundz[14][1]=1;
// start[15]=0;  boundz[15][0]=0;  boundz[15][1]=0;
// start[16]=1;  boundz[16][0]=1;  boundz[16][1]=1;
// start[17]=0;  boundz[17][0]=0;  boundz[17][1]=0;
// start[18]=1;  boundz[18][0]=1;  boundz[18][1]=1;
// start[19]=0;  boundz[19][0]=0;  boundz[19][1]=0;
// start[20]=0;  boundz[20][0]=0;  boundz[20][1]=0;
// start[21]=0;  boundz[21][0]=0;  boundz[21][1]=0;
// start[22]=0;  boundz[22][0]=0;  boundz[22][1]=0;
// start[23]=1;  boundz[23][0]=1;  boundz[23][1]=1;
// start[24]=1;  boundz[24][0]=1;  boundz[24][1]=1;
// start[25]=0;  boundz[25][0]=0;  boundz[25][1]=0;
// start[26]=0;  boundz[26][0]=0;  boundz[26][1]=0;
// start[27]=1;  boundz[27][0]=1;  boundz[27][1]=1;
// start[28]=0;  boundz[28][0]=0;  boundz[28][1]=0;
// start[29]=1;  boundz[29][0]=1;  boundz[29][1]=1;
// start[30]=0;  boundz[30][0]=0;  boundz[30][1]=0;
// start[31]=1;  boundz[31][0]=1;  boundz[31][1]=1;
// start[32]=0;  boundz[32][0]=0;  boundz[32][1]=0;
// start[33]=1;  boundz[33][0]=1;  boundz[33][1]=1;
// start[34]=1;  boundz[34][0]=1;  boundz[34][1]=1;
// start[35]=0;  boundz[35][0]=0;  boundz[35][1]=0;
// start[36]=0;  boundz[36][0]=0;  boundz[36][1]=0;
// start[37]=0;  boundz[37][0]=0;  boundz[37][1]=0;
// start[38]=1;  boundz[38][0]=1;  boundz[38][1]=1;
// start[39]=0;  boundz[39][0]=0;  boundz[39][1]=0;
// start[40]=0;  boundz[40][0]=0;  boundz[40][1]=0;
// start[41]=1;  boundz[41][0]=1;  boundz[41][1]=1;
// start[42]=0;  boundz[42][0]=0;  boundz[42][1]=0;
// start[43]=0;  boundz[43][0]=0;  boundz[43][1]=0;
// start[44]=1;  boundz[44][0]=1;  boundz[44][1]=1;
// start[45]=0;  boundz[45][0]=0;  boundz[45][1]=0;
// start[46]=0;  boundz[46][0]=0;  boundz[46][1]=0;
// start[47]=1;  boundz[47][0]=1;  boundz[47][1]=1;
// start[48]=0;  boundz[48][0]=0;  boundz[48][1]=0;
// start[49]=0;  boundz[49][0]=0;  boundz[49][1]=0;
// start[50]=97.1795;		
// start[51]=343.015;		
// start[52]=12.336;		
// start[53]=197.619;		
// start[54]=289.112;		
// start[55]=132.214;		
// start[56]=85.831;		
// start[57]=1308.99;		
// start[58]=122.251;		
// start[59]=412.721;		
// start[60]=726.049;		
// start[61]=523.709;		
// start[62]=134.814;		
// start[63]=1137.26;		
// start[64]=269.329;		
// start[65]=838.706;		
// start[66]=1140.4;		
// start[67]=993.408;		
// start[68]=89.3902;		
// start[69]=147.422;		
// start[70]=58.2234;		
// start[71]=160.199;		
// start[72]=93.7329;		
// start[73]=112.019;		
// start[74]=795.755;		
// start[75]=100.446;		
// start[76]=236.665;		
// start[77]=964.017;		
// start[78]=876.782;		
// start[79]=825.899;		
// start[80]=201.807;		
// start[81]=377.867;		
// start[82]=143.154;		
// start[83]=211.613;		
// start[84]=335.415;		
// start[85]=209.451;		
// start[86]=191.26;		
// start[87]=7.55474;		
// start[88]=159.584;		
// start[89]=205.028;		
// start[90]=1144.54;		
// start[91]=174.065;		
// start[92]=2257.18;		
// start[93]=249.689;		
// start[94]=548.567;		
// start[95]=279.229;		
// start[96]=207.798;		
// start[97]=622.738;		
// start[98]=89.5406;		
// start[99]=239.998;		
// start[100]=0.000813789;		
// start[101]=163.28;		
// start[102]=219.992;		
// start[103]=95.5546;		
// start[104]=20.0003;		
// start[105]=138.884;		
// start[106]=40.0002;		
// start[107]=83.0775;		
// start[108]=10.0003;		
// start[109]=40.0009;		
// start[110]=98.552;		
// start[111]=0.000339123;		
// start[112]=0.00147282;		
// start[113]=191.195;		
// start[114]=249.589;		
// start[115]=263.976;		
// start[116]=1759.77;		
// start[117]=1939.35;		
// start[118]=413.42;		
// start[119]=688.02;		
// start[120]=159.584;		
// start[121]=198.815;		
// start[122]=2257.18;		
// start[123]=1318.61;		
// start[124]=207.798;		
// start[125]=827.796;		
// start[126]=10.0002;		
// start[127]=12;		
// start[128]=20.3525;		
// start[129]=15;		
// start[130]=15;		
// start[131]=12.0002;		
// start[132]=56.2104;		
// start[133]=99.075;		
// start[134]=73.5191;		
// start[135]=56.2104;		
// start[136]=70.4672;		
// start[137]=70.4672;		
// start[138]=60.8639;		
// start[139]=60.8643;		
// start[140]=48.3776;		
// start[141]=101.823;		
// start[142]=56.2104;		
// start[143]=70.4672;		
// start[144]=48.3776;		
// start[145]=60.8639;		
// start[146]=10;		
// start[147]=10;		
// start[148]=10;		
// start[149]=10;		


//Save values of continuous variables
    for(int i=NB; i<NV; i++){
        start[i]=sol[i];
        sol[i]=0;
    }

}

void Node::read_save_start(const string &filen, int r) {

    double data[NV+1];
    cout<<"Reading startpoints from file"<<endl;
	// Opening the well data file
	ifstream inFile(filen.c_str());
    
	//Now begin reading in the data	
        
		for(int i=0;i<(NV+1);i++){
				inFile >> data[i];
                //cout<<data[i]<< "  "<<endl;
            }

	// Closing the well data file
	inFile.close();
    
    set_parent_obj(-data[0]);
    save_obj(0);
    set_rootnum(r);
    
    //cout<<"PARENT OBJ  "<<parent_objf<<"   OBJ  "<<objf<<endl;
    
    for(int i=1; i<(NV+1); i++){
        start[i-1] = data[i];
    }
    
}

// void Stack::print_BBlist(){
//     
//     cout<<"_____________________________________BB LISTEN______________________________________________"<<endl;
//     list<Node>::iterator i;
//     for (i = BB.begin(); i != BB.end(); i++){
//         printf("Probnr: %-5d LB=[%4.2f %4.2f %4.2f %4.2f] UB=[%4.2f %4.2e %4.2e %4.2f] ST=[%4.2f %4.2e %4.2e %4.2f]\n", 
//                 i->probnum,
//                 i->boundz[0][0], i->boundz[1][0], i->boundz[2][0], i->boundz[3][0],
//                 i->boundz[0][1], i->boundz[1][1], i->boundz[2][1], i->boundz[3][1],
//                 i->start[0], i->start[1], i->start[2], i->start[3]);
//     }
//     cout<<"_______________________________________________________________________________________________"<<endl;
//     
// }
// 
// void Stack::print_donelist(){
//     
//     cout<<"_____________________________________Done LISTEN______________________________________________"<<endl;
//     list<Node>::iterator i;
//     for (i = done.begin(); i != done.end(); i++){
//         printf("Probnr: %-5d OFV: %5.2f VAR=[%4.2f %4.2f %4.2f %4.2f] LB=[%4.2f %4.2f %4.2f %4.2f] UB=[%4.2f %4.2e %4.2e %4.2f] ST=[%4.2f %4.2e %4.2e %4.2f]\n", 
//                 i->objf, i->probnum, i->sol[0], i->sol[1], i->sol[2], i->sol[3], 
//                 i->boundz[0][0], i->boundz[1][0], i->boundz[2][0], i->boundz[3][0],
//                 i->boundz[0][1], i->boundz[1][1], i->boundz[2][1], i->boundz[3][1],
//                 i->start[0], i->start[1], i->start[2], i->start[3]);
//     }
//     cout<<"_______________________________________________________________________________________________"<<endl;
//     
// }

int Stack::check_LB(double current_OFV, const Node& n, int h){
    //cout<<"Updating the LB"<<endl;

    if (current_OFV > best_LB){
        best_LB=current_OFV;
        cout<<"LB UPDATE: "<<best_LB<<endl;

        if (h==1){ //Heuristikk 1
            bestNode.probnum=-100;  increase_h1();}
        else if(h==2){//Heuristikk 2
            bestNode.probnum=-200;  increase_h2();}
        else if (h==3){//Feasibility Pump
            bestNode.probnum=-300;}
        else if (h==4){//BB with s==1
            bestNode.probnum=-400;}
        else{//Regular BB            
            bestNode.probnum=n.probnum; increase_bb();}
        
        bestNode.objf = n.objf;
    
        for(int i=0; i<NV; i++){
            bestNode.boundz[i][0]=n.boundz[i][0];
            bestNode.boundz[i][1]=n.boundz[i][1];
        }
    
        for(int i=0; i<NV; i++){
            bestNode.start[i]=n.start[i];
        }
    
        for(int i=0; i<NV; i++){
            bestNode.sol[i]=n.sol[i];
        }
        return 1;
    }
    else{
        return 0;
    }

}

double Stack::random_double(double low, double high){

    double range=(high-low);
    double num=fmod(rand(), range)+low;
    return num;
}

double Stack::random_int(int low, int high){

    double randNum=(rand()%(high-low))+low;
    return randNum;
}

void Stack::multiple_start(double minmax[I+J][3][2]){
    srand(time(NULL));
    
    for (int j=0; j<num_multi; j++){    
        //For each random start point to be generated, make a node
        Node *po = new Node(NULL);
        //Set the gas injection and wellhead pressure for each well, drawn randomly between its boundaries
        int xp=126; int xg=98;
        for (int i=0; i<(I+J); i++){
            po->set_mmw(minmax);
            po->set_boundz();
            po->set_multi_start();
            //Random gas injection value
            po->start[i+xg]=random_double(minmax[i][0][0], minmax[i][0][1]);
            //Random wellhead pressure value
            po->start[i+xp]=random_double(minmax[i][1][0], minmax[i][1][1]);
            
        }
        
        //Randomly set the binary values
        for(int i=0; i<50; i++){
            po->start[i]=random_double(0.01,0.99);
        }

        //po->print_bounds_start();
        rand_start.push_back(*po);
        delete po;
    }
    

    cout<<"Number of nodes for multiple start plan: "<<rand_start.size()<<endl;  
//     list<Node>::iterator it = rand_start.begin();
//     for (it=rand_start.begin(); it!=rand_start.end(); it++){
//         it->print_start();
//     }
    
    
}

int Stack::are_equal(const Node& n1, const Node& n2){
    //Check the objective function
    int a=0;
    if (n1.get_obj() == n2.get_obj()){
        a=1;
    }
    
    //Check the variable values
    int b=0;
    for (int i=0; i<NV; i++){
        if (n1.sol[i]==n2.sol[i]){
            b=b+1;
        }
    }
    
    cout<<"a+b[151]="<<a+b<<endl;
    return (a+b);
}
    

bool Stack::nodes_in_cluster(const Node& n1, const Node& n2){
    
if (n1.probnum==n2.probnum){
    cout<<"Equal Nodes"<<endl;
    return true;
}
else{
    //Start of variables of interest
    int pn=126;
    int qn=98;
    
    //The binary variable differences
    for(int i=0; i<50; i++){
        if ( fabs(n1.sol[i] - n2.sol[i]) < cluster_bin){
            dist[i]=1;
        }
        else{
            dist[i]=0;
        }
    }
    
    //The gas injection variable differences
    for(int i=50; i<65; i++){
        if ( fabs(n1.sol[qn] - n2.sol[qn]) < cluster_q){
            dist[i]=1;
            qn++;
        }
        else{
            dist[i]=0;
            qn++;
        }
    }
    
    //The pressure variable differences
    for(int i=65; i<81; i++){
        if ( fabs(n1.sol[pn] - n2.sol[pn]) < cluster_p){
            dist[i]=1;
            pn++;
        }
        else{
            dist[i]=0;
            pn++;
        }
    }
    
    int n=floor(cluster_percentage*cluster_nv);
    int sum=0;
    
    for(int i=0; i<cluster_nv; i++){
        sum=sum+dist[i];
    }
    
    for(int i=0; i<cluster_nv; i++){
        dist[i]=0;
    }                
    
    cout<<"Sum="<<sum<<"   n="<<n<<endl;
    
    if (sum>=n){
        return true;
    }
    else{
        return false;
    }

}


}



// void Stack::update_BBlist(SmartPtr<MyTNLP>& t){
//     
//     list<Node>::iterator i;
//     for (i = BB.begin(); i != BB.end(); i++){
//         if ( (0.9*best_LB <= i->parent_objf) && (i->parent_objf <= best_LB)){
//             cout<<0.9*best_LB<<"<="<<i->parent_objf<<"<="<<best_LB<<endl;
// //             t->equate_bounds(*head);
// //             t->equate_start(*head);
// //         
// //             //Setup completed, call ipopt and return status of run
// //             ApplicationReturnStatus status = ipoptApplication->Initialize();
// //             status=ipoptApplication->OptimizeTNLP(GetRawPtr(t));
// //             i->save_stat(t->s);
// //             cout<<"STATUS IS: "<<status<<endl;
// //             if (tnlp->s == 0){
// //                 i->save_sol(t->solution);
// //                 i->save_obj(t->obj);
// //                 if ( i->get_obj() > (bbList.get_best_LB()) ){
// //                 }
// //             }
// //         }
//         
//             
//         }
//     }

int Stack::ipopt_max_iter=0;

int Stack::alt1=0;
int Stack::alt2=0;
int Stack::alt3=0;
int Stack::stat_fail=0;
int Stack::h1=0;
int Stack::h2=0;
int Stack::bb=0;
    
void Stack::increase_ipopt_iter(){
    ipopt_max_iter++;    
}

void Stack::increase_alt1(){
    alt1++;
}

void Stack::increase_alt2(){
    alt2++;
}

void Stack::increase_alt3(){
    alt3++;
}

void Stack::increase_stat_fail(){
    stat_fail++;
}

void Stack::increase_h1(){
    h1++;
}

void Stack::increase_h2(){
    h2++;
}

void Stack::increase_bb(){
    bb++;
}


    
void Stack::res_file(int size, int a, int b, double timee){
    FILE* fp = fopen("res.txt", "a");

    printf("RES FILE UPDATE!");
   
    if (a==1){
    fprintf(fp, "Total Nodes   Open   Solves    Time   Alt1  Alt2  Alt3  H1  H2  BB  Best Node  Objective  UpperBound\n");
    }
    
    fprintf(fp, "%11d %6d %8d %7.1f %6d %5d %5d %3d %3d %3d %9d %10.2f %10.2f\n", Node::total_nodes, size, b, timee,
            get_alt1(), get_alt2(), get_alt3(), Stack::h1, Stack::h2, Stack::bb,
            bestNode.probnum, bestNode.objf, upper_bound);
    
    //get_stat_fail() 
    
    
//     fprintf(fp, "Total Nodes Created: %3d     Status 1 Nodes: %3d  Open Nodes: %3d\n", Node::total_nodes, get_imi(), size);
//     fprintf(fp, "Alternative Nodes: %5d  %5d  %5d  =%5d\n", get_alt1(), get_alt2(), get_alt3(), get_alt1()+get_alt2()+get_alt3());
//     fprintf(fp, "BEST NODE: %-5d OFV: %5.2f\n\n", bestNode.probnum, bestNode.objf);
    
    fclose(fp);
}

void Stack::time_file(const Node& n, int msr, int mss, double search, double bbt, double fp, double h1, double h2, int r, int it, int h1it, int h2SN){
    
    ofstream myfile;
    myfile.open("time.txt", fstream::out | fstream::app);
    
    
    if (myfile.is_open()){
        myfile<<endl;
        myfile<<"Multistart Run Time: "<<msr<<endl;
        myfile<<"Multistart Sort Time: "<<mss<<endl;
        myfile<<"Branch&Bound Time: "<<bbt<<endl;
        myfile<<"System Time: "<<search<<endl;
        myfile<<"Feasibility Pump Time: "<<fp<<endl;
        myfile<<"Heuristic 1 Time: "<<h1<<endl;
        myfile<<"System Time: "<<h2<<endl;
        myfile<<"Heur1 It: "<<h1it<<endl;
        myfile<<"Heur2 NLP runs: "<<h2SN<<endl;
        
        myfile<<"NODE: "<<n.probnum<<"   OFV: "<<n.objf<<"   Nodes: "<<Node::total_nodes<<"   rounds: "<<r<<"   solves: "<<it<<endl;

        myfile<<"SOLUTION - VARIABLES"<<endl;
        n.twells_open();
        myfile<<endl;
        n.swells_open();
        myfile<<endl;
        n.line_to_sep();
        myfile<<endl;
        for(int i=0; i<18; i++){
            if(n.sol[i]<0.0001){myfile<<"0"<<"  ";}
            else{myfile<<n.sol[i]<<"  ";}
        }
        myfile<<endl;
        for(int i=18; i<38; i++){
            if(n.sol[i]<0.0001){myfile<<"0"<<"  ";}
            else{myfile<<n.sol[i]<<"  ";}
        }
        myfile<<endl;
        for(int i=38; i<50; i++){
            if(n.sol[i]<0.0001){myfile<<"0"<<"  ";}
            else{myfile<<n.sol[i]<<"  ";}
        }
        myfile<<endl;
        myfile<<endl;
        for(int i=50; i<68; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=68; i<98; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=98; i<104; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=104; i<114; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=114; i<126; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=126; i<132; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=132; i<142; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;
        for(int i=142; i<150; i++){
        myfile<<n.sol[i]<<"  ";
        }
        myfile<<endl;

        myfile.close();
    }
    
}
    

