//Functions to make simulator from tables + their derivatives
double * MyTMINLP::simulator(double pW, double qGL, double (*data)[pdim][ddim]){
 
    double * values = new double[12];//This will be returned by the function:
                                    //values[0]=gas, values[1]=oil
                                    //values[2]=water, values[3]=pBHP
                                    //values[4]=dg/dp, values[5]=do/dp
                                    //values[6]=dw/dp, values[7]=db/dp
                                    //values[8]=dg/dq, values[9]=do/dq
                                    //values[10]=dw/dq, values[11]=db/dq
    
    //cout<<"Simulator call for:  ---pW: "<<pW<<"---  and ---qGL: "<<qGL<<"---"<<endl;
    
                                    
    //1) FIND THE INDEXES BELOW p^W and q^GL
    double qcol[gdim];
    double pcol[pdim];
    //cout<<"Printing qcol"<<endl;
    for (int i=0; i<gdim; i++){
        //cout<<i<<": "<<data[i][0][0]<<endl;
        qcol[i]=(qGL/data[i][0][0])-1;
        //cout<<i<<": "<<qcol[i]<<endl;
    }
    for (int i=0; i<pdim; i++){
        pcol[i]=(pW/data[0][i][1])-1;
    } 
    
    
    
    int iiq=closest_zero(gdim, qcol);
    int iip=closest_zero(pdim, pcol);
    
    if(iip<0){iip=0;}
    else if(iip>pdim){iip=pdim;}
    
    if(iiq<0){iiq=0;}
    else if(iiq>gdim){iiq=gdim;}
    
    //cout<<"iip: "<<iip<<"--------iiq: "<<iiq<<endl;

    //2) FIND THE 3 NEAREST POINTS, ALLOCATE WEIGHTS AND CALCULATE VALUES
    
    //2.a) Calculate the Euclidean distances, start at the lower values and go counter-clockwise
    double dist[4];
    dist[0] = sqrt( pow(pW-data[0][iip][1],2) + pow((qGL-data[iiq][0][0])/1000,2) );
    dist[1] = sqrt( pow(pW-data[0][iip][1],2) + pow((qGL-data[iiq+1][0][0])/1000,2));
    dist[2] = sqrt( pow(pW-data[0][iip+1][1],2) + pow((qGL-data[iiq+1][0][0])/1000,2));
    dist[3] = sqrt( pow(pW-data[0][iip+1][1],2) + pow((qGL-data[iiq][0][0])/1000,2));

    double weights[4]; // Holds the weights (coefficients) of each point
    
    //2.b) Allocate 1/4 weight if all points are equidistant  
    if ((dist[0]==dist[1]) && (dist[1]==dist[2]) && (dist[2]==dist[3]) ){ //The point is in the middle - equal distance from all surrounding points
        //Find values with weighting 0.25 on each point
        for (int i=0; i<4; i++){
            values[i] = 0.25*(data[iiq][iip][i+2] + data[iiq+1][iip][i+2] + data[iiq+1][iip+1][i+2] + data[iiq][iip+1][i+2]);
        }
        values[0] = (values[0]/gscale); //Gas scaling
    } 
    //2.c) Or solve the linear system to get the right weights, where the weights have to sum to one
    else{
        int k=max_val(4, dist); //Index of point farthest away
        int m=min_val(4, dist); //Index of point closest
        int t[2]; //Holds indices to the two points in the middle
        double c[2]; //Hold the coefficients related to each middle index in t[2]
        int j=0;
        for (int i=0; i<4; i++){
            if ( (i!=k) && (i!=m) ){
                t[j]=i;
                j++;
            }
        }
        weights[k] =0; //The point farthest away gets weight zero
        
        //Find the Coefficients
        c[0]= (dist[t[0]]/dist[m]);
        c[1]= (dist[t[1]]/dist[m]);
        //Find the Weights
        weights[m] = 1 / ( (1/c[0]) + (1/c[1]) + 1);
        weights[t[0]] = (weights[m] / c[0]);
        weights[t[1]] = (weights[m] / c[1]);
        
        for (int j=0; j<4; j++){ //Calculate the gas, oil, water and pBHP values
            
            
            values[j] = ((weights[0]*data[iiq][iip][j+2]) + (weights[1]*data[iiq+1][iip][j+2])
            + (weights[2]*data[iiq+1][iip+1][j+2]) + (weights[3]*data[iiq][iip+1][j+2]));
        }
        values[0] = (values[0]/gscale); //Gas scaling
        
    }
    //3) CALCULATING THE pW DERIVATIVES
    
    //3.a) Calculate weight (relative to upper breakpoint of qGL)
    double vp= 1.0 -(  ((data[iiq+1][0][0]/gscale) - qGL)/((data[iiq+1][0][0] - data[iiq][0][0])/gscale)   );
    
    //3.b) Calculate difference in gas, oil, water and pBHP values for lower and upper pW breakpoint
    //for each lower and upper qGL value
    double dp_gl=(data[iiq][iip+1][2]-data[iiq][iip][2])/(data[0][iip+1][1] - data[0][iip][1]); 
    double dp_gu=(data[iiq+1][iip+1][2]-data[iiq+1][iip][2])/(data[0][iip+1][1] - data[0][iip][1]);
    values[4] = ((vp*dp_gu) + (1-vp)*(dp_gl))/gscale;
    
    double dp_ol=(data[iiq][iip+1][3]-data[iiq][iip][3])/(data[0][iip+1][1] - data[0][iip][1]);
    double dp_ou=(data[iiq+1][iip+1][3]-data[iiq+1][iip][3])/(data[0][iip+1][1] - data[0][iip][1]);
    values[5] = ((vp*dp_ou) + (1-vp)*(dp_ol));
    
    double dp_wl=(data[iiq][iip+1][4]-data[iiq][iip][4])/(data[0][iip+1][1] - data[0][iip][1]);
    double dp_wu=(data[iiq+1][iip+1][4]-data[iiq+1][iip][4])/(data[0][iip+1][1] - data[0][iip][1]);
    values[6] = ((vp*dp_wu) + (1-vp)*(dp_wl));
    
    double dp_bl=(data[iiq][iip+1][5]-data[iiq][iip][5])/(data[0][iip+1][1] - data[0][iip][1]);
    double dp_bu=(data[iiq+1][iip+1][5]-data[iiq+1][iip][5])/(data[0][iip+1][1] - data[0][iip][1]);
    values[7] = ((vp*dp_bu) + (1-vp)*(dp_bl));
    
    //4) CALCULATING THE qGL DERIVATIVES
    
    //4.a) Calculate weight (relative to upper breakpoint of pW)
    double vq= 1.0 -((data[0][iip+1][1] - pW)/(data[0][iip+1][1] - data[0][iip][1]));
    
    //4.b) Calculate difference in gas, oil, water and pBHP values for lower and upper qGL breakpoint
    //for each lower and upper pW value
    double dq_gl= ((data[iiq+1][iip][2]-data[iiq][iip][2])/gscale)/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    double dq_gu=((data[iiq+1][iip+1][2]-data[iiq][iip+1][2])/gscale)/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    values[8] = (vq*dq_gu) + (1-vq)*(dq_gl);
    
    double dq_ol=(data[iiq+1][iip][3]-data[iiq][iip][3])/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    double dq_ou=(data[iiq+1][iip+1][3]-data[iiq][iip+1][3])/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    values[9] = (vq*dq_ou) + (1-vq)*(dq_ol);
    
    double dq_wl=(data[iiq+1][iip][4]-data[iiq][iip][4])/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    double dq_wu=(data[iiq+1][iip+1][4]-data[iiq][iip+1][4])/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    values[10] = (vq*dq_wu) + (1-vq)*(dq_wl);
    
    double dq_bl=(data[iiq+1][iip][5]-data[iiq][iip][5])/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    double dq_bu=(data[iiq+1][iip+1][5]-data[iiq][iip+1][5])/((data[iiq+1][0][0] - data[iiq][0][0])/gscale);
    values[11] = (vq*dq_bu) + (1-vq)*(dq_bl);
    
    
    //5) WRITING TO FILE
    itt++;
    printsome(itt, iip, iiq, pW, qGL, values, dist, weights);
    
    //For Testing Purposes - Remove Later
//     cout<<"------------Simulation Call----------- callnum: "<<itt<<"      PW: "<<pW<<" ----- QGL: "<<qGL<<endl;
//     for (int i=0; i<12; i++){
//         cout<<"v["<<i<<"]:"<<values[i]<<", ";
//     }
//     cout<<endl;   
    
    return values;
    

}